@@ -959,7 +959,7 @@ create({ constructor: SourcesView, proto: MenuContainer.prototype }, {
  * Utility functions for handling sources.
  */
 let SourceUtils = {
-  _labelsCache: new Map(),
+  _labelsCache: new Map(), // Can't use WeakMaps because keys are strings.
   _groupsCache: new Map(),
 
   /**
@@ -968,8 +968,8 @@ let SourceUtils = {
    * This should be done every time the content location changes.
    */
   clearCache: function SU_clearCache() {
-    this._labelsCache = new Map();
-    this._groupsCache = new Map();
+    this._labelsCache.clear();
+    this._groupsCache.clear();
   },
 
   /**
@@ -987,7 +987,7 @@ let SourceUtils = {
     }
 
     let sourceLabel = this.trimUrl(aUrl);
-    let unicodeLabel = this.convertToUnicode(window.unescape(sourceLabel));
+    let unicodeLabel = NetworkHelper.convertToUnicode(unescape(sourceLabel));
     this._labelsCache.set(aUrl, unicodeLabel);
     return unicodeLabel;
   },
@@ -1015,7 +1015,12 @@ let SourceUtils = {
       return "";
     }
 
-    let { scheme, hostPort, directory, fileName } = uri;
+    let { scheme, directory, fileName } = uri;
+    let hostPort;
+    // Add-on SDK jar: URLs will cause accessing hostPort to throw.
+    if (scheme != "jar") {
+      hostPort = uri.hostPort;
+    }
     let lastDir = directory.split("/").reverse()[1];
     let group = [];
 
@@ -1036,7 +1041,7 @@ let SourceUtils = {
     }
 
     let groupLabel = group.join(" ");
-    let unicodeLabel = this.convertToUnicode(window.unescape(groupLabel));
+    let unicodeLabel = NetworkHelper.convertToUnicode(unescape(groupLabel));
     this._groupsCache.set(aUrl, unicodeLabel)
     return unicodeLabel;
   },
@@ -1181,31 +1186,6 @@ let SourceUtils = {
     }
     // Give up.
     return aUrl.spec;
-  },
-
-  /**
-   * Convert a given string, encoded in a given character set, to unicode.
-   *
-   * @param string aString
-   *        A string.
-   * @param string aCharset [optional]
-   *        A character set.
-   * @return string
-   *         A unicode string.
-   */
-  convertToUnicode: function SU_convertToUnicode(aString, aCharset) {
-    // Decoding primitives.
-    let converter = Cc["@mozilla.org/intl/scriptableunicodeconverter"]
-        .createInstance(Ci.nsIScriptableUnicodeConverter);
-
-    try {
-      if (aCharset) {
-        converter.charset = aCharset;
-      }
-      return converter.ConvertToUnicode(aString);
-    } catch(e) {
-      return aString;
-    }
   }
 };
 
