@@ -21,6 +21,7 @@ nsDOMKeyboardEvent::nsDOMKeyboardEvent(mozilla::dom::EventTarget* aOwner,
     mEventIsInternal = true;
     mEvent->time = PR_Now();
   }
+  SetIsDOMBinding();
 }
 
 nsDOMKeyboardEvent::~nsDOMKeyboardEvent()
@@ -45,7 +46,7 @@ NS_IMETHODIMP
 nsDOMKeyboardEvent::GetAltKey(bool* aIsDown)
 {
   NS_ENSURE_ARG_POINTER(aIsDown);
-  *aIsDown = static_cast<nsInputEvent*>(mEvent)->IsAlt();
+  *aIsDown = AltKey();
   return NS_OK;
 }
 
@@ -53,7 +54,7 @@ NS_IMETHODIMP
 nsDOMKeyboardEvent::GetCtrlKey(bool* aIsDown)
 {
   NS_ENSURE_ARG_POINTER(aIsDown);
-  *aIsDown = static_cast<nsInputEvent*>(mEvent)->IsControl();
+  *aIsDown = CtrlKey();
   return NS_OK;
 }
 
@@ -61,7 +62,7 @@ NS_IMETHODIMP
 nsDOMKeyboardEvent::GetShiftKey(bool* aIsDown)
 {
   NS_ENSURE_ARG_POINTER(aIsDown);
-  *aIsDown = static_cast<nsInputEvent*>(mEvent)->IsShift();
+  *aIsDown = ShiftKey();
   return NS_OK;
 }
 
@@ -69,7 +70,7 @@ NS_IMETHODIMP
 nsDOMKeyboardEvent::GetMetaKey(bool* aIsDown)
 {
   NS_ENSURE_ARG_POINTER(aIsDown);
-  *aIsDown = static_cast<nsInputEvent*>(mEvent)->IsMeta();
+  *aIsDown = MetaKey();
   return NS_OK;
 }
 
@@ -79,7 +80,16 @@ nsDOMKeyboardEvent::GetModifierState(const nsAString& aKey,
 {
   NS_ENSURE_ARG_POINTER(aState);
 
-  *aState = GetModifierStateInternal(aKey);
+  *aState = GetModifierState(aKey);
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsDOMKeyboardEvent::GetKey(nsAString& aKeyName)
+{
+  if (!mEventIsInternal) {
+    static_cast<nsKeyEvent*>(mEvent)->GetDOMKeyName(aKeyName);
+  }
   return NS_OK;
 }
 
@@ -87,69 +97,63 @@ NS_IMETHODIMP
 nsDOMKeyboardEvent::GetCharCode(uint32_t* aCharCode)
 {
   NS_ENSURE_ARG_POINTER(aCharCode);
+  *aCharCode = CharCode();
+  return NS_OK;
+}
 
+uint32_t
+nsDOMKeyboardEvent::CharCode()
+{
   switch (mEvent->message) {
   case NS_KEY_UP:
   case NS_KEY_DOWN:
-    *aCharCode = 0;
-    break;
+    return 0;
   case NS_KEY_PRESS:
-    *aCharCode = ((nsKeyEvent*)mEvent)->charCode;
-    break;
-  default:
-    *aCharCode = 0;
-    break;
+    return static_cast<nsKeyEvent*>(mEvent)->charCode;
   }
-  return NS_OK;
+  return 0;
 }
 
 NS_IMETHODIMP
 nsDOMKeyboardEvent::GetKeyCode(uint32_t* aKeyCode)
 {
   NS_ENSURE_ARG_POINTER(aKeyCode);
+  *aKeyCode = KeyCode();
+  return NS_OK;
+}
 
+uint32_t
+nsDOMKeyboardEvent::KeyCode()
+{
   switch (mEvent->message) {
   case NS_KEY_UP:
   case NS_KEY_PRESS:
   case NS_KEY_DOWN:
-    *aKeyCode = ((nsKeyEvent*)mEvent)->keyCode;
-    break;
-  default:
-    *aKeyCode = 0;
-    break;
+    return static_cast<nsKeyEvent*>(mEvent)->keyCode;
   }
-
-  return NS_OK;
+  return 0;
 }
 
-/* virtual */
-nsresult
-nsDOMKeyboardEvent::Which(uint32_t* aWhich)
+uint32_t
+nsDOMKeyboardEvent::Which()
 {
-  NS_ENSURE_ARG_POINTER(aWhich);
-
   switch (mEvent->message) {
     case NS_KEY_UP:
     case NS_KEY_DOWN:
-      return GetKeyCode(aWhich);
+      return KeyCode();
     case NS_KEY_PRESS:
       //Special case for 4xp bug 62878.  Try to make value of which
       //more closely mirror the values that 4.x gave for RETURN and BACKSPACE
       {
         uint32_t keyCode = ((nsKeyEvent*)mEvent)->keyCode;
         if (keyCode == NS_VK_RETURN || keyCode == NS_VK_BACK) {
-          *aWhich = keyCode;
-          return NS_OK;
+          return keyCode;
         }
-        return GetCharCode(aWhich);
+        return CharCode();
       }
-      break;
-    default:
-      *aWhich = 0;
-      break;
   }
 
-  return NS_OK;
+  return 0;
 }
 
 NS_IMETHODIMP
@@ -157,7 +161,7 @@ nsDOMKeyboardEvent::GetLocation(uint32_t* aLocation)
 {
   NS_ENSURE_ARG_POINTER(aLocation);
 
-  *aLocation = static_cast<nsKeyEvent*>(mEvent)->location;
+  *aLocation = Location();
   return NS_OK;
 }
 
