@@ -3,10 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/Util.h"
 
 #include "nsMathMLElement.h"
-#include "nsDOMClassInfoID.h" // for eDOMClassInfo_MathElement_id.
+#include "base/compiler_specific.h"
+#include "mozilla/Util.h"
 #include "nsGkAtoms.h"
 #include "nsCRT.h"
 #include "nsRuleData.h"
@@ -18,14 +18,17 @@
 #include "nsIPresShell.h"
 #include "nsPresContext.h"
 #include "mozAutoDocUpdate.h"
+#include "nsIScriptError.h"
+#include "nsContentUtils.h"
+
+#include "mozilla/dom/ElementBinding.h"
 
 using namespace mozilla;
+using namespace mozilla::dom;
 
 //----------------------------------------------------------------------
 // nsISupports methods:
 
-DOMCI_NODE_DATA(MathMLElement, nsMathMLElement)
-
 NS_INTERFACE_TABLE_HEAD(nsMathMLElement)
   NS_NODE_OFFSET_AND_INTERFACE_TABLE_BEGIN(nsMathMLElement)
     NS_INTERFACE_TABLE_ENTRY(nsMathMLElement, nsIDOMNode)
@@ -34,12 +37,54 @@ NS_INTERFACE_TABLE_HEAD(nsMathMLElement)
     NS_INTERFACE_TABLE_ENTRY(nsMathMLElement, Link)
   NS_OFFSET_AND_INTERFACE_TABLE_END
   NS_ELEMENT_INTERFACE_TABLE_TO_MAP_SEGUE
-  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(MathMLElement)
 NS_ELEMENT_INTERFACE_MAP_END
 
 NS_IMPL_ADDREF_INHERITED(nsMathMLElement, nsMathMLElementBase)
 NS_IMPL_RELEASE_INHERITED(nsMathMLElement, nsMathMLElementBase)
 
+static nsresult 
+WarnDeprecated(const PRUnichar* aDeprecatedAttribute, 
+               const PRUnichar* aFavoredAttribute, nsIDocument* aDocument)
+{
+  const PRUnichar *argv[] = 
+    { aDeprecatedAttribute, aFavoredAttribute };
+  return nsContentUtils::
+          ReportToConsole(nsIScriptError::warningFlag, "MathML", aDocument,
+                          nsContentUtils::eMATHML_PROPERTIES,
+                          "DeprecatedSupersededBy", argv, 2);
+}
+
+static nsresult 
+ReportLengthParseError(const nsString& aValue, nsIDocument* aDocument)
+{
+  const PRUnichar *arg = aValue.get();
+  return nsContentUtils::
+         ReportToConsole(nsIScriptError::errorFlag, "MathML", aDocument,
+                         nsContentUtils::eMATHML_PROPERTIES,
+                         "LengthParsingError", &arg, 1);
+}
+
+static nsresult
+ReportParseErrorNoTag(const nsString& aValue, 
+                      nsIAtom*        aAtom,
+                      nsIDocument*    aDocument)
+{
+  const PRUnichar *argv[] = 
+    { aValue.get(), aAtom->GetUTF16String() };
+  return nsContentUtils::
+         ReportToConsole(nsIScriptError::errorFlag, "MathML", aDocument,
+                         nsContentUtils::eMATHML_PROPERTIES,
+                         "AttributeParsingErrorNoTag", argv, 2);
+}
+
+nsMathMLElement::nsMathMLElement(already_AddRefed<nsINodeInfo> aNodeInfo)
+: nsMathMLElementBase(aNodeInfo),
+  ALLOW_THIS_IN_INITIALIZER_LIST(Link(this)),
+  mIncrementScriptLevel(false)
+{
+  SetIsDOMBinding();
+}
+
 nsresult
 nsMathMLElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
                             nsIContent* aBindingParent,
@@ -47,7 +92,7 @@ nsMathMLElement::BindToTree(nsIDocument* aDocument, nsIContent* aParent,
 {
   static const char kMathMLStyleSheetURI[] = "resource://gre-resources/mathml.css";
 
-  Link::ResetLinkState(false);
+  Link::ResetLinkState(false, Link::ElementHasHref());
 
   nsresult rv = nsMathMLElementBase::BindToTree(aDocument, aParent,
                                                 aBindingParent,
@@ -82,7 +127,7 @@ nsMathMLElement::UnbindFromTree(bool aDeep, bool aNullParent)
 {
   // If this link is ever reinserted into a document, it might
   // be under a different xml:base, so forget the cached state now.
-  Link::ResetLinkState(false);
+  Link::ResetLinkState(false, Link::ElementHasHref());
   
   nsIDocument* doc = GetCurrentDoc();
   if (doc) {
@@ -93,12 +138,20 @@ nsMathMLElement::UnbindFromTree(bool aDeep, bool aNullParent)
 }
 
 bool
-nsMathMLElement::ParseAttribute(PRInt32 aNamespaceID,
+nsMathMLElement::ParseAttribute(int32_t aNamespaceID,
                                 nsIAtom* aAttribute,
                                 const nsAString& aValue,
                                 nsAttrValue& aResult)
 {
   if (aNamespaceID == kNameSpaceID_None) {
+    if (Tag() == nsGkAtoms::math && aAttribute == nsGkAtoms::mode) {
+      WarnDeprecated(nsGkAtoms::mode->GetUTF16String(),
+                     nsGkAtoms::display->GetUTF16String(), OwnerDoc());
+    }
+    if (aAttribute == nsGkAtoms::color) {
+      WarnDeprecated(nsGkAtoms::color->GetUTF16String(),
+                     nsGkAtoms::mathcolor_->GetUTF16String(), OwnerDoc());
+    }
     if (aAttribute == nsGkAtoms::color ||
         aAttribute == nsGkAtoms::mathcolor_ ||
         aAttribute == nsGkAtoms::background ||
@@ -111,31 +164,31 @@ nsMathMLElement::ParseAttribute(PRInt32 aNamespaceID,
                                              aValue, aResult);
 }
 
-static nsGenericElement::MappedAttributeEntry sMtableStyles[] = {
+static Element::MappedAttributeEntry sMtableStyles[] = {
   { &nsGkAtoms::width },
-  { nsnull }
+  { nullptr }
 };
 
-static nsGenericElement::MappedAttributeEntry sTokenStyles[] = {
+static Element::MappedAttributeEntry sTokenStyles[] = {
   { &nsGkAtoms::mathsize_ },
   { &nsGkAtoms::fontsize_ },
   { &nsGkAtoms::color },
   { &nsGkAtoms::fontfamily_ },
-  { nsnull }
+  { nullptr }
 };
 
-static nsGenericElement::MappedAttributeEntry sEnvironmentStyles[] = {
+static Element::MappedAttributeEntry sEnvironmentStyles[] = {
   { &nsGkAtoms::scriptlevel_ },
   { &nsGkAtoms::scriptminsize_ },
   { &nsGkAtoms::scriptsizemultiplier_ },
   { &nsGkAtoms::background },
-  { nsnull }
+  { nullptr }
 };
 
-static nsGenericElement::MappedAttributeEntry sCommonPresStyles[] = {
+static Element::MappedAttributeEntry sCommonPresStyles[] = {
   { &nsGkAtoms::mathcolor_ },
   { &nsGkAtoms::mathbackground_ },
-  { nsnull }
+  { nullptr }
 };
 
 bool
@@ -211,9 +264,9 @@ nsMathMLElement::GetAttributeMappingFunction() const
 /* static */ bool
 nsMathMLElement::ParseNamedSpaceValue(const nsString& aString,
                                       nsCSSValue&     aCSSValue,
-                                      PRUint32        aFlags)
+                                      uint32_t        aFlags)
 {
-   PRInt32 i = 0;
+   int32_t i = 0;
    // See if it is one of the 'namedspace' (ranging -7/18em, -6/18, ... 7/18em)
    if (aString.EqualsLiteral("veryverythinmathspace")) {
      i = 1;
@@ -293,14 +346,19 @@ nsMathMLElement::ParseNamedSpaceValue(const nsString& aString,
 /* static */ bool
 nsMathMLElement::ParseNumericValue(const nsString& aString,
                                    nsCSSValue&     aCSSValue,
-                                   PRUint32        aFlags)
+                                   uint32_t        aFlags,
+                                   nsIDocument*    aDocument)
 {
   nsAutoString str(aString);
   str.CompressWhitespace(); // aString is const in this code...
 
-  PRInt32 stringLength = str.Length();
-  if (!stringLength)
+  int32_t stringLength = str.Length();
+  if (!stringLength) {
+    if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+      ReportLengthParseError(aString, aDocument);
+    }
     return false;
+  }
 
   if (ParseNamedSpaceValue(aString, aCSSValue, aFlags)) {
     return true;
@@ -309,7 +367,7 @@ nsMathMLElement::ParseNumericValue(const nsString& aString,
   nsAutoString number, unit;
 
   // see if the negative sign is there
-  PRInt32 i = 0;
+  int32_t i = 0;
   PRUnichar c = str[0];
   if (c == '-') {
     number.Append(c);
@@ -320,8 +378,12 @@ nsMathMLElement::ParseNumericValue(const nsString& aString,
   bool gotDot = false;
   for ( ; i < stringLength; i++) {
     c = str[i];
-    if (gotDot && c == '.')
+    if (gotDot && c == '.') {
+      if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+        ReportLengthParseError(aString, aDocument);
+      }
       return false;  // two dots encountered
+    }
     else if (c == '.')
       gotDot = true;
     else if (!nsCRT::IsAsciiDigit(c)) {
@@ -334,24 +396,47 @@ nsMathMLElement::ParseNumericValue(const nsString& aString,
   }
 
   // Convert number to floating point
-  PRInt32 errorCode;
+  nsresult errorCode;
   float floatValue = number.ToFloat(&errorCode);
-  if (NS_FAILED(errorCode))
+  if (NS_FAILED(errorCode)) {
+    if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+      ReportLengthParseError(aString, aDocument);
+    }
     return false;
-  if (floatValue < 0 && !(aFlags & PARSE_ALLOW_NEGATIVE))
+  }
+  if (floatValue < 0 && !(aFlags & PARSE_ALLOW_NEGATIVE)) {
+    if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+      ReportLengthParseError(aString, aDocument);
+    }
     return false;
+  }
 
   nsCSSUnit cssUnit;
   if (unit.IsEmpty()) {
     if (aFlags & PARSE_ALLOW_UNITLESS) {
       // no explicit unit, this is a number that will act as a multiplier
-      cssUnit = eCSSUnit_Number;
+      if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+        nsContentUtils::ReportToConsole(nsIScriptError::warningFlag,
+                                        "MathML", aDocument,
+                                        nsContentUtils::eMATHML_PROPERTIES,
+                                        "UnitlessValuesAreDeprecated");
+      }
+      if (aFlags & CONVERT_UNITLESS_TO_PERCENT) {
+        aCSSValue.SetPercentValue(floatValue);
+        return true;
+      }
+      else
+        cssUnit = eCSSUnit_Number;
     } else {
       // We are supposed to have a unit, but there isn't one.
       // If the value is 0 we can just call it "pixels" otherwise
       // this is illegal.
-      if (floatValue != 0.0)
+      if (floatValue != 0.0) {
+        if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+          ReportLengthParseError(aString, aDocument);
+        }
         return false;
+      }
       cssUnit = eCSSUnit_Pixel;
     }
   }
@@ -367,8 +452,12 @@ nsMathMLElement::ParseNumericValue(const nsString& aString,
   else if (unit.EqualsLiteral("mm")) cssUnit = eCSSUnit_Millimeter;
   else if (unit.EqualsLiteral("pt")) cssUnit = eCSSUnit_Point;
   else if (unit.EqualsLiteral("pc")) cssUnit = eCSSUnit_Pica;
-  else // unexpected unit
+  else { // unexpected unit
+    if (!(aFlags & PARSE_SUPPRESS_WARNINGS)) {
+      ReportLengthParseError(aString, aDocument);
+    }
     return false;
+  }
 
   aCSSValue.SetFloatValue(floatValue, cssUnit);
   return true;
@@ -397,11 +486,15 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
       str.CompressWhitespace();
       // MathML numbers can't have leading '+'
       if (str.Length() > 0 && str.CharAt(0) != '+') {
-        PRInt32 errorCode;
+        nsresult errorCode;
         float floatValue = str.ToFloat(&errorCode);
         // Negative scriptsizemultipliers are not parsed
         if (NS_SUCCEEDED(errorCode) && floatValue >= 0.0f) {
           scriptSizeMultiplier->SetFloatValue(floatValue, eCSSUnit_Number);
+        } else {
+          ReportParseErrorNoTag(str,
+                                nsGkAtoms::scriptsizemultiplier_,
+                                aData->mPresContext->Document());
         }
       }
     }
@@ -415,14 +508,20 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
     // default: 8pt
     //
     // We don't allow negative values.
-    // XXXfredw Should we allow unitless values? (bug 411227)
-    // XXXfredw Does a relative unit give a multiple of the default value?
+    // Unitless and percent values give a multiple of the default value.
     //
     value = aAttributes->GetAttr(nsGkAtoms::scriptminsize_);
     nsCSSValue* scriptMinSize = aData->ValueForScriptMinSize();
     if (value && value->Type() == nsAttrValue::eString &&
         scriptMinSize->GetUnit() == eCSSUnit_Null) {
-      ParseNumericValue(value->GetStringValue(), *scriptMinSize, 0);
+      ParseNumericValue(value->GetStringValue(), *scriptMinSize,
+                        PARSE_ALLOW_UNITLESS | CONVERT_UNITLESS_TO_PERCENT,
+                        aData->mPresContext->Document());
+
+      if (scriptMinSize->GetUnit() == eCSSUnit_Percent) {
+        scriptMinSize->SetFloatValue(8.0 * scriptMinSize->GetPercentValue(),
+                                     eCSSUnit_Point);
+      }
     }
 
     // scriptlevel
@@ -443,8 +542,8 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
       nsAutoString str(value->GetStringValue());
       str.CompressWhitespace();
       if (str.Length() > 0) {
-        PRInt32 errorCode;
-        PRInt32 intValue = str.ToInteger(&errorCode);
+        nsresult errorCode;
+        int32_t intValue = str.ToInteger(&errorCode);
         if (NS_SUCCEEDED(errorCode)) {
           // This is kind of cheesy ... if the scriptlevel has a sign,
           // then it's a relative value and we store the nsCSSValue as an
@@ -456,6 +555,10 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
           } else {
             scriptLevel->SetFloatValue(intValue, eCSSUnit_Number);
           }
+        } else {
+          ReportParseErrorNoTag(str,
+                                nsGkAtoms::scriptlevel_,
+                                aData->mPresContext->Document());
         }
       }
     }
@@ -479,28 +582,34 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
     // default: inherited
     //
     // In both cases, we don't allow negative values.
-    // XXXfredw Should we allow unitless values? (bug 411227)
-    // XXXfredw Does a relative unit give a multiple of the default value?
+    // Unitless values give a multiple of the default value.
     //  
     bool parseSizeKeywords = true;
     value = aAttributes->GetAttr(nsGkAtoms::mathsize_);
     if (!value) {
       parseSizeKeywords = false;
       value = aAttributes->GetAttr(nsGkAtoms::fontsize_);
+      if (value) {
+        WarnDeprecated(nsGkAtoms::fontsize_->GetUTF16String(),
+                       nsGkAtoms::mathsize_->GetUTF16String(),
+                       aData->mPresContext->Document());
+      }
     }
     nsCSSValue* fontSize = aData->ValueForFontSize();
     if (value && value->Type() == nsAttrValue::eString &&
         fontSize->GetUnit() == eCSSUnit_Null) {
       nsAutoString str(value->GetStringValue());
-      if (!ParseNumericValue(str, *fontSize, 0) &&
-          parseSizeKeywords) {
+      if (!ParseNumericValue(str, *fontSize, PARSE_SUPPRESS_WARNINGS |
+                             PARSE_ALLOW_UNITLESS | CONVERT_UNITLESS_TO_PERCENT,
+                             nullptr)
+          && parseSizeKeywords) {
         static const char sizes[3][7] = { "small", "normal", "big" };
-        static const PRInt32 values[NS_ARRAY_LENGTH(sizes)] = {
+        static const int32_t values[NS_ARRAY_LENGTH(sizes)] = {
           NS_STYLE_FONT_SIZE_SMALL, NS_STYLE_FONT_SIZE_MEDIUM,
           NS_STYLE_FONT_SIZE_LARGE
         };
         str.CompressWhitespace();
-        for (PRUint32 i = 0; i < ArrayLength(sizes); ++i) {
+        for (uint32_t i = 0; i < ArrayLength(sizes); ++i) {
           if (str.EqualsASCII(sizes[i])) {
             fontSize->SetIntValue(values[i], eCSSUnit_Enumerated);
             break;
@@ -519,6 +628,11 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
     // 
     value = aAttributes->GetAttr(nsGkAtoms::fontfamily_);
     nsCSSValue* fontFamily = aData->ValueForFontFamily();
+    if (value) {
+      WarnDeprecated(nsGkAtoms::fontfamily_->GetUTF16String(),
+                     nsGkAtoms::mathvariant_->GetUTF16String(),
+                     aData->mPresContext->Document());
+    }
     if (value && value->Type() == nsAttrValue::eString &&
         fontFamily->GetUnit() == eCSSUnit_Null) {
       fontFamily->SetStringValue(value->GetStringValue(), eCSSUnit_Families);
@@ -548,6 +662,11 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
       aAttributes->GetAttr(nsGkAtoms::mathbackground_);
     if (!value) {
       value = aAttributes->GetAttr(nsGkAtoms::background);
+      if (value) {
+        WarnDeprecated(nsGkAtoms::background->GetUTF16String(),
+                       nsGkAtoms::mathbackground_->GetUTF16String(),
+                       aData->mPresContext->Document());
+      }
     }
     nsCSSValue* backgroundColor = aData->ValueForBackgroundColor();
     if (value && backgroundColor->GetUnit() == eCSSUnit_Null) {
@@ -579,6 +698,11 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
     const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::mathcolor_);
     if (!value) {
       value = aAttributes->GetAttr(nsGkAtoms::color);
+      if (value) {
+        WarnDeprecated(nsGkAtoms::color->GetUTF16String(),
+                       nsGkAtoms::mathcolor_->GetUTF16String(),
+                       aData->mPresContext->Document());
+      }
     }
     nscolor color;
     nsCSSValue* colorValue = aData->ValueForColor();
@@ -595,7 +719,8 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
       const nsAttrValue* value = aAttributes->GetAttr(nsGkAtoms::width);
       // This does not handle auto and unitless values
       if (value && value->Type() == nsAttrValue::eString) {
-        ParseNumericValue(value->GetStringValue(), *width, 0);
+        ParseNumericValue(value->GetStringValue(), *width, 0,
+                          aData->mPresContext->Document());
       }
     }
   }
@@ -605,7 +730,7 @@ nsMathMLElement::MapMathMLAttributesInto(const nsMappedAttributes* aAttributes,
 nsresult
 nsMathMLElement::PreHandleEvent(nsEventChainPreVisitor& aVisitor)
 {
-  nsresult rv = nsGenericElement::PreHandleEvent(aVisitor);
+  nsresult rv = Element::PreHandleEvent(aVisitor);
   NS_ENSURE_SUCCESS(rv, rv);
 
   return PreHandleEventForLinks(aVisitor);
@@ -627,7 +752,7 @@ nsMathMLElement::IntrinsicState() const
 }
 
 bool
-nsMathMLElement::IsNodeOfType(PRUint32 aFlags) const
+nsMathMLElement::IsNodeOfType(uint32_t aFlags) const
 {
   return !(aFlags & ~eCONTENT);
 }
@@ -646,7 +771,7 @@ nsMathMLElement::SetIncrementScriptLevel(bool aIncrementScriptLevel,
 }
 
 bool
-nsMathMLElement::IsFocusable(PRInt32 *aTabIndex, bool aWithMouse)
+nsMathMLElement::IsFocusable(int32_t *aTabIndex, bool aWithMouse)
 {
   nsCOMPtr<nsIURI> uri;
   if (IsLink(getter_AddRefs(uri))) {
@@ -673,7 +798,7 @@ nsMathMLElement::IsLink(nsIURI** aURI) const
       tag == nsGkAtoms::none         ||
       tag == nsGkAtoms::malignmark_  ||
       tag == nsGkAtoms::maligngroup_) {
-    *aURI = nsnull;
+    *aURI = nullptr;
     return false;
   }
 
@@ -697,13 +822,13 @@ nsMathMLElement::IsLink(nsIURI** aURI) const
     // result is poorly specified. Either way, we return false.
     
     static nsIContent::AttrValuesArray sTypeVals[] =
-      { &nsGkAtoms::_empty, &nsGkAtoms::simple, nsnull };
+      { &nsGkAtoms::_empty, &nsGkAtoms::simple, nullptr };
     
     static nsIContent::AttrValuesArray sShowVals[] =
-      { &nsGkAtoms::_empty, &nsGkAtoms::_new, &nsGkAtoms::replace, nsnull };
+      { &nsGkAtoms::_empty, &nsGkAtoms::_new, &nsGkAtoms::replace, nullptr };
     
     static nsIContent::AttrValuesArray sActuateVals[] =
-      { &nsGkAtoms::_empty, &nsGkAtoms::onRequest, nsnull };
+      { &nsGkAtoms::_empty, &nsGkAtoms::onRequest, nullptr };
     
     // Optimization: check for href first for early return
     href = mAttrsAndChildren.GetAttr(nsGkAtoms::href,
@@ -733,7 +858,7 @@ nsMathMLElement::IsLink(nsIURI** aURI) const
     return !!*aURI;
   }
 
-  *aURI = nsnull;
+  *aURI = nullptr;
   return false;
 }
 
@@ -749,7 +874,7 @@ nsMathMLElement::GetLinkTarget(nsAString& aTarget)
   if (aTarget.IsEmpty()) {
 
     static nsIContent::AttrValuesArray sShowVals[] =
-      { &nsGkAtoms::_new, &nsGkAtoms::replace, nsnull };
+      { &nsGkAtoms::_new, &nsGkAtoms::replace, nullptr };
     
     switch (FindAttrValueIn(kNameSpaceID_XLink, nsGkAtoms::show,
                             sShowVals, eCaseMatters)) {
@@ -773,11 +898,11 @@ already_AddRefed<nsIURI>
 nsMathMLElement::GetHrefURI() const
 {
   nsCOMPtr<nsIURI> hrefURI;
-  return IsLink(getter_AddRefs(hrefURI)) ? hrefURI.forget() : nsnull;
+  return IsLink(getter_AddRefs(hrefURI)) ? hrefURI.forget() : nullptr;
 }
 
 nsresult
-nsMathMLElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
+nsMathMLElement::SetAttr(int32_t aNameSpaceID, nsIAtom* aName,
                          nsIAtom* aPrefix, const nsAString& aValue,
                          bool aNotify)
 {
@@ -792,14 +917,18 @@ nsMathMLElement::SetAttr(PRInt32 aNameSpaceID, nsIAtom* aName,
   if (aName == nsGkAtoms::href &&
       (aNameSpaceID == kNameSpaceID_None ||
        aNameSpaceID == kNameSpaceID_XLink)) {
-    Link::ResetLinkState(!!aNotify);
+    if (aNameSpaceID == kNameSpaceID_XLink) {
+      WarnDeprecated(NS_LITERAL_STRING("xlink:href").get(),
+                     NS_LITERAL_STRING("href").get(), OwnerDoc());
+    }
+    Link::ResetLinkState(!!aNotify, true);
   }
 
   return rv;
 }
 
 nsresult
-nsMathMLElement::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr,
+nsMathMLElement::UnsetAttr(int32_t aNameSpaceID, nsIAtom* aAttr,
                            bool aNotify)
 {
   nsresult rv = nsMathMLElementBase::UnsetAttr(aNameSpaceID, aAttr, aNotify);
@@ -812,8 +941,16 @@ nsMathMLElement::UnsetAttr(PRInt32 aNameSpaceID, nsIAtom* aAttr,
   if (aAttr == nsGkAtoms::href &&
       (aNameSpaceID == kNameSpaceID_None ||
        aNameSpaceID == kNameSpaceID_XLink)) {
-    Link::ResetLinkState(!!aNotify);
+    // Note: just because we removed a single href attr doesn't mean there's no href,
+    // since there are 2 possible namespaces.
+    Link::ResetLinkState(!!aNotify, Link::ElementHasHref());
   }
 
   return rv;
 }
+
+JSObject*
+nsMathMLElement::WrapNode(JSContext *aCx, JS::Handle<JSObject*> aScope)
+{
+  return ElementBinding::Wrap(aCx, aScope, this);
+}
