@@ -15,6 +15,9 @@
 #include "ImageContainer.h"
 #include "Layers.h"
 #include "mozilla/layers/LayersTypes.h"
+#include "WinUtils.h"
+
+using namespace mozilla::widget;
 
 #ifndef MOZ_SAMPLE_TYPE_FLOAT32
 #error We expect 32bit float audio samples on desktop for the Windows Media Foundation media backend.
@@ -102,7 +105,10 @@ WMFReader::OnDecodeThreadFinish()
 bool
 WMFReader::InitializeDXVA()
 {
-  if (!Preferences::GetBool("media.windows-media-foundation.use-dxva", false)) {
+  if (!Preferences::GetBool("media.windows-media-foundation.use-dxva", false) ||
+      WinUtils::GetWindowsVersion() == WinUtils::VISTA_VERSION) {
+    // Don't use DXVA on Vista until bug 901944's fix has time to bake on
+    // the release train.
     return false;
   }
 
