@@ -11,7 +11,6 @@
 #include "nsINameSpaceManager.h"
 #include "nsIURI.h"
 #include "nsIURL.h"
-#include "nsIDOMEventTarget.h"
 #include "nsIChannel.h"
 #include "nsXPIDLString.h"
 #include "nsReadableUtils.h"
@@ -35,12 +34,15 @@
 #include "nsCRT.h"
 #include "nsContentUtils.h"
 #include "nsTextFragment.h"
+#include "nsTextNode.h"
 
 #include "nsIScriptContext.h"
 #include "nsIScriptError.h"
 
 #include "nsIStyleRuleProcessor.h"
 #include "nsXBLResourceLoader.h"
+#include "mozilla/dom/CDATASection.h"
+#include "mozilla/dom/Comment.h"
 #include "mozilla/dom/Element.h"
 
 #ifdef MOZ_XUL
@@ -315,9 +317,8 @@ nsXBLPrototypeBinding::SetBasePrototype(nsXBLPrototypeBinding* aBinding)
 already_AddRefed<nsIContent>
 nsXBLPrototypeBinding::GetBindingElement()
 {
-  nsIContent* result = mBinding;
-  NS_IF_ADDREF(result);
-  return result;
+  nsCOMPtr<nsIContent> result = mBinding;
+  return result.forget();
 }
 
 void
@@ -514,12 +515,8 @@ nsXBLPrototypeBinding::AttributeChanged(nsIAtom* aAttribute,
           nsAutoString value;
           aChangedElement->GetAttr(aNameSpaceID, aAttribute, value);
           if (!value.IsEmpty()) {
-            nsCOMPtr<nsIContent> textContent;
-            NS_NewTextNode(getter_AddRefs(textContent),
-                           realElement->NodeInfo()->NodeInfoManager());
-            if (!textContent) {
-              continue;
-            }
+            nsRefPtr<nsTextNode> textContent =
+              new nsTextNode(realElement->NodeInfo()->NodeInfoManager());
 
             textContent->SetText(value, true);
             realElement->AppendChildTo(textContent, true);
@@ -726,15 +723,12 @@ nsXBLPrototypeBinding::GetImmediateChild(nsIAtom* aTag)
  
 nsresult
 nsXBLPrototypeBinding::InitClass(const nsCString& aClassName,
-                                 JSContext * aContext, JSObject * aGlobal,
-                                 JSObject * aScriptObject,
-                                 JSObject** aClassObject,
+                                 JSContext * aContext,
+                                 JS::Handle<JSObject*> aGlobal,
+                                 JS::Handle<JSObject*> aScriptObject,
+                                 JS::MutableHandle<JSObject*> aClassObject,
                                  bool* aNew)
 {
-  NS_ENSURE_ARG_POINTER(aClassObject); 
-
-  *aClassObject = nullptr;
-
   return nsXBLBinding::DoInitJSClass(aContext, aGlobal, aScriptObject,
                                      aClassName, this, aClassObject, aNew);
 }
@@ -893,12 +887,8 @@ bool SetAttrs(nsHashKey* aKey, void* aData, void* aClosure)
                                              kNameSpaceID_XUL) &&
              dst == nsGkAtoms::value && !value.IsEmpty())) {
 
-          nsCOMPtr<nsIContent> textContent;
-          NS_NewTextNode(getter_AddRefs(textContent),
-                         realElement->NodeInfo()->NodeInfoManager());
-          if (!textContent) {
-            continue;
-          }
+          nsRefPtr<nsTextNode> textContent =
+            new nsTextNode(realElement->NodeInfo()->NodeInfoManager());
 
           textContent->SetText(value, false);
           realElement->AppendChildTo(textContent, false);
@@ -1028,7 +1018,7 @@ nsXBLPrototypeBinding::ConstructAttributeTable(nsIContent* aElement)
       // so that we don't have to convert from Unicode to ASCII and then back
 
       char* token = nsCRT::strtok( str, ", ", &newStr );
-      while( token != NULL ) {
+      while( token != nullptr ) {
         // Build an atom out of this attribute.
         nsCOMPtr<nsIAtom> atom;
         int32_t atomNsID = kNameSpaceID_None;
@@ -1137,7 +1127,7 @@ nsXBLPrototypeBinding::ConstructInsertionTable(nsIContent* aContent)
       // so that we don't have to convert from Unicode to ASCII and then back
 
       char* token = nsCRT::strtok( str, "| ", &newStr );
-      while( token != NULL ) {
+      while( token != nullptr ) {
         nsAutoString tok;
         tok.AssignWithConversion(token);
 
@@ -1218,7 +1208,7 @@ nsXBLPrototypeBinding::ConstructInterfaceTable(const nsAString& aImpls)
     // so that we don't have to convert from Unicode to ASCII and then back
 
     char* token = nsCRT::strtok( str, ", ", &newStr );
-    while( token != NULL ) {
+    while( token != nullptr ) {
       // get the InterfaceInfo for the name
       nsCOMPtr<nsIInterfaceInfo> iinfo;
       infoManager->GetInfoForName(token, getter_AddRefs(iinfo));
@@ -1692,18 +1682,17 @@ nsXBLPrototypeBinding::ReadContentNode(nsIObjectInputStream* aStream,
       namespaceID == XBLBinding_Serialize_CommentNode) {
     switch (namespaceID) {
       case XBLBinding_Serialize_TextNode:
-        rv = NS_NewTextNode(getter_AddRefs(content), aNim);
+        content = new nsTextNode(aNim);
         break;
       case XBLBinding_Serialize_CDATANode:
-        rv = NS_NewXMLCDATASection(getter_AddRefs(content), aNim);
+        content = new CDATASection(aNim);
         break;
       case XBLBinding_Serialize_CommentNode:
-        rv = NS_NewCommentNode(getter_AddRefs(content), aNim);
+        content = new Comment(aNim);
         break;
       default:
         break;
     }
-    NS_ENSURE_SUCCESS(rv, rv);
 
     nsAutoString text;
     rv = aStream->ReadString(text);
