@@ -18,7 +18,6 @@
 #include "nsIDOMLoadStatus.h"
 #include "nsAutoPtr.h"
 #include "nsContentUtils.h"
-#include "nsIJSContextStack.h"
 #include "nsEventDispatcher.h"
 #include "nsIObserverService.h"
 #include "nsIScriptGlobalObject.h"
@@ -87,6 +86,7 @@ nsDOMOfflineResourceList::nsDOMOfflineResourceList(nsIURI *aManifestURI,
   , mManifestURI(aManifestURI)
   , mDocumentURI(aDocumentURI)
   , mExposeCacheUpdateStatus(true)
+  , mDontSetDocumentCache(false)
   , mStatus(nsIDOMOfflineResourceList::IDLE)
   , mCachedKeys(nullptr)
   , mCachedKeysCount(0)
@@ -450,6 +450,10 @@ nsDOMOfflineResourceList::Update()
                                      window, getter_AddRefs(update));
   NS_ENSURE_SUCCESS(rv, rv);
 
+  // Since we are invoking the cache update, cache on the document must not
+  // be updated until swapCache() method is called on applicationCache object.
+  mDontSetDocumentCache = true;
+
   return NS_OK;
 }
 
@@ -621,6 +625,15 @@ NS_IMETHODIMP
 nsDOMOfflineResourceList::ApplicationCacheAvailable(nsIApplicationCache *aApplicationCache)
 {
   mAvailableApplicationCache = aApplicationCache;
+
+  if (!mDontSetDocumentCache) {
+    nsCOMPtr<nsIApplicationCacheContainer> appCacheContainer =
+      GetDocumentAppCacheContainer();
+
+    if (appCacheContainer)
+      appCacheContainer->SetApplicationCache(aApplicationCache);
+  }
+
   return NS_OK;
 }
 
@@ -719,6 +732,7 @@ nsDOMOfflineResourceList::UpdateCompleted(nsIOfflineCacheUpdate *aUpdate)
 
   mCacheUpdate->RemoveObserver(this);
   mCacheUpdate = nullptr;
+  mDontSetDocumentCache = false;
 
   if (NS_SUCCEEDED(rv) && succeeded && !partial) {
     if (isUpgrade) {
