@@ -6,11 +6,13 @@
 
 #include "TelephonyCall.h"
 
-#include "nsDOMClassInfo.h"
+#include "nsIDOMCallEvent.h"
 
-#include "CallEvent.h"
-#include "Telephony.h"
 #include "DOMError.h"
+#include "GeneratedEvents.h"
+#include "nsDOMClassInfo.h"
+#include "Telephony.h"
+#include "nsITelephonyProvider.h"
 
 USING_TELEPHONY_NAMESPACE
 
@@ -37,6 +39,14 @@ TelephonyCall::Create(Telephony* aTelephony, const nsAString& aNumber,
   return call.forget();
 }
 
+TelephonyCall::TelephonyCall()
+  : mCallIndex(kOutgoingPlaceholderCallIndex),
+    mCallState(nsITelephonyProvider::CALL_STATE_UNKNOWN),
+    mLive(false),
+    mOutgoing(false)
+{
+}
+
 void
 TelephonyCall::ChangeStateInternal(uint16_t aCallState, bool aFireEvents)
 {
@@ -44,37 +54,37 @@ TelephonyCall::ChangeStateInternal(uint16_t aCallState, bool aFireEvents)
 
   nsString stateString;
   switch (aCallState) {
-    case nsIRadioInterfaceLayer::CALL_STATE_DIALING:
+    case nsITelephonyProvider::CALL_STATE_DIALING:
       stateString.AssignLiteral("dialing");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_ALERTING:
+    case nsITelephonyProvider::CALL_STATE_ALERTING:
       stateString.AssignLiteral("alerting");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_BUSY:
+    case nsITelephonyProvider::CALL_STATE_BUSY:
       stateString.AssignLiteral("busy");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_CONNECTING:
+    case nsITelephonyProvider::CALL_STATE_CONNECTING:
       stateString.AssignLiteral("connecting");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_CONNECTED:
+    case nsITelephonyProvider::CALL_STATE_CONNECTED:
       stateString.AssignLiteral("connected");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_HOLDING:
+    case nsITelephonyProvider::CALL_STATE_HOLDING:
       stateString.AssignLiteral("holding");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_HELD:
+    case nsITelephonyProvider::CALL_STATE_HELD:
       stateString.AssignLiteral("held");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_RESUMING:
+    case nsITelephonyProvider::CALL_STATE_RESUMING:
       stateString.AssignLiteral("resuming");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_DISCONNECTING:
+    case nsITelephonyProvider::CALL_STATE_DISCONNECTING:
       stateString.AssignLiteral("disconnecting");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_DISCONNECTED:
+    case nsITelephonyProvider::CALL_STATE_DISCONNECTED:
       stateString.AssignLiteral("disconnected");
       break;
-    case nsIRadioInterfaceLayer::CALL_STATE_INCOMING:
+    case nsITelephonyProvider::CALL_STATE_INCOMING:
       stateString.AssignLiteral("incoming");
       break;
     default:
@@ -84,11 +94,11 @@ TelephonyCall::ChangeStateInternal(uint16_t aCallState, bool aFireEvents)
   mState = stateString;
   mCallState = aCallState;
 
-  if (aCallState == nsIRadioInterfaceLayer::CALL_STATE_DIALING) {
+  if (aCallState == nsITelephonyProvider::CALL_STATE_DIALING) {
     mOutgoing = true;
   }
 
-  if (aCallState == nsIRadioInterfaceLayer::CALL_STATE_DISCONNECTED) {
+  if (aCallState == nsITelephonyProvider::CALL_STATE_DISCONNECTED) {
     NS_ASSERTION(mLive, "Should be live!");
     mTelephony->RemoveCall(this);
     mLive = false;
@@ -98,27 +108,40 @@ TelephonyCall::ChangeStateInternal(uint16_t aCallState, bool aFireEvents)
   }
 
   if (aFireEvents) {
-    nsRefPtr<CallEvent> event = CallEvent::Create(this);
-    NS_ASSERTION(event, "This should never fail!");
-
-    if (NS_FAILED(event->Dispatch(ToIDOMEventTarget(),
-                                  NS_LITERAL_STRING("statechange")))) {
-      NS_WARNING("Failed to dispatch statechange event!");
+    nsresult rv = DispatchCallEvent(NS_LITERAL_STRING("statechange"), this);
+    if (NS_FAILED(rv)) {
+      NS_WARNING("Failed to dispatch specific event!");
     }
 
     // This can change if the statechange handler called back here... Need to
     // figure out something smarter.
     if (mCallState == aCallState) {
-      event = CallEvent::Create(this);
-      NS_ASSERTION(event, "This should never fail!");
-
-      if (NS_FAILED(event->Dispatch(ToIDOMEventTarget(), stateString))) {
+      rv = DispatchCallEvent(stateString, this);
+      if (NS_FAILED(rv)) {
         NS_WARNING("Failed to dispatch specific event!");
       }
     }
   }
 }
 
+nsresult
+TelephonyCall::DispatchCallEvent(const nsAString& aType,
+                                 nsIDOMTelephonyCall* aCall)
+{
+  MOZ_ASSERT(aCall);
+
+  nsCOMPtr<nsIDOMEvent> event;
+  NS_NewDOMCallEvent(getter_AddRefs(event), this, nullptr, nullptr);
+  NS_ASSERTION(event, "This should never fail!");
+
+  nsCOMPtr<nsIDOMCallEvent> callEvent = do_QueryInterface(event);
+  MOZ_ASSERT(callEvent);
+  nsresult rv = callEvent->InitCallEvent(aType, false, false, aCall);
+  NS_ENSURE_SUCCESS(rv, rv);
+
+  return DispatchTrustedEvent(callEvent);
+}
+
 void
 TelephonyCall::NotifyError(const nsAString& aError)
 {
@@ -128,30 +151,17 @@ TelephonyCall::NotifyError(const nsAString& aError)
   mError = DOMError::CreateWithName(aError);
 
   // Do the state transitions
-  ChangeStateInternal(nsIRadioInterfaceLayer::CALL_STATE_DISCONNECTED, true);
+  ChangeStateInternal(nsITelephonyProvider::CALL_STATE_DISCONNECTED, true);
 
-  // Notify the error event
-  nsRefPtr<CallEvent> event = CallEvent::Create(this);
-  NS_ASSERTION(event, "This should never fail!");
-
-  if (NS_FAILED(event->Dispatch(ToIDOMEventTarget(),
-                                NS_LITERAL_STRING("error")))) {
+  nsresult rv = DispatchCallEvent(NS_LITERAL_STRING("error"), this);
+  if (NS_FAILED(rv)) {
     NS_WARNING("Failed to dispatch error event!");
   }
 }
 
-NS_IMPL_CYCLE_COLLECTION_CLASS(TelephonyCall)
-
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_BEGIN_INHERITED(TelephonyCall,
-                                                  nsDOMEventTargetHelper)
-  NS_IMPL_CYCLE_COLLECTION_TRAVERSE_NATIVE_PTR(tmp->mTelephony->ToISupports(),
-                                               Telephony, "mTelephony")
-NS_IMPL_CYCLE_COLLECTION_TRAVERSE_END
-
-NS_IMPL_CYCLE_COLLECTION_UNLINK_BEGIN_INHERITED(TelephonyCall,
-                                                nsDOMEventTargetHelper)
-  NS_IMPL_CYCLE_COLLECTION_UNLINK_NSCOMPTR(mTelephony)
-NS_IMPL_CYCLE_COLLECTION_UNLINK_END
+NS_IMPL_CYCLE_COLLECTION_INHERITED_1(TelephonyCall,
+                                     nsDOMEventTargetHelper,
+                                     mTelephony)
 
 NS_INTERFACE_MAP_BEGIN_CYCLE_COLLECTION_INHERITED(TelephonyCall)
   NS_INTERFACE_MAP_ENTRY(nsIDOMTelephonyCall)
@@ -187,63 +197,63 @@ TelephonyCall::GetError(nsIDOMDOMError** aError)
 NS_IMETHODIMP
 TelephonyCall::Answer()
 {
-  if (mCallState != nsIRadioInterfaceLayer::CALL_STATE_INCOMING) {
+  if (mCallState != nsITelephonyProvider::CALL_STATE_INCOMING) {
     NS_WARNING("Answer on non-incoming call ignored!");
     return NS_OK;
   }
 
-  nsresult rv = mTelephony->RIL()->AnswerCall(mCallIndex);
+  nsresult rv = mTelephony->Provider()->AnswerCall(mCallIndex);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  ChangeStateInternal(nsIRadioInterfaceLayer::CALL_STATE_CONNECTING, true);
+  ChangeStateInternal(nsITelephonyProvider::CALL_STATE_CONNECTING, true);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 TelephonyCall::HangUp()
 {
-  if (mCallState == nsIRadioInterfaceLayer::CALL_STATE_DISCONNECTING ||
-      mCallState == nsIRadioInterfaceLayer::CALL_STATE_DISCONNECTED) {
+  if (mCallState == nsITelephonyProvider::CALL_STATE_DISCONNECTING ||
+      mCallState == nsITelephonyProvider::CALL_STATE_DISCONNECTED) {
     NS_WARNING("HangUp on previously disconnected call ignored!");
     return NS_OK;
   }
 
-  nsresult rv = mCallState == nsIRadioInterfaceLayer::CALL_STATE_INCOMING ?
-                mTelephony->RIL()->RejectCall(mCallIndex) :
-                mTelephony->RIL()->HangUp(mCallIndex);
+  nsresult rv = mCallState == nsITelephonyProvider::CALL_STATE_INCOMING ?
+                mTelephony->Provider()->RejectCall(mCallIndex) :
+                mTelephony->Provider()->HangUp(mCallIndex);
   NS_ENSURE_SUCCESS(rv, rv);
 
-  ChangeStateInternal(nsIRadioInterfaceLayer::CALL_STATE_DISCONNECTING, true);
+  ChangeStateInternal(nsITelephonyProvider::CALL_STATE_DISCONNECTING, true);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 TelephonyCall::Hold()
 {
-  if (mCallState != nsIRadioInterfaceLayer::CALL_STATE_CONNECTED) {
+  if (mCallState != nsITelephonyProvider::CALL_STATE_CONNECTED) {
     NS_WARNING("Hold non-connected call ignored!");
     return NS_OK;
   }
-  
-  nsresult rv = mTelephony->RIL()->HoldCall(mCallIndex);
+
+  nsresult rv = mTelephony->Provider()->HoldCall(mCallIndex);
   NS_ENSURE_SUCCESS(rv,rv);
-  
-  ChangeStateInternal(nsIRadioInterfaceLayer::CALL_STATE_HOLDING, true);
+
+  ChangeStateInternal(nsITelephonyProvider::CALL_STATE_HOLDING, true);
   return NS_OK;
 }
 
 NS_IMETHODIMP
 TelephonyCall::Resume()
 {
-  if (mCallState != nsIRadioInterfaceLayer::CALL_STATE_HELD) {
+  if (mCallState != nsITelephonyProvider::CALL_STATE_HELD) {
     NS_WARNING("Resume non-held call ignored!");
     return NS_OK;
   }
-  
-  nsresult rv = mTelephony->RIL()->ResumeCall(mCallIndex);
+
+  nsresult rv = mTelephony->Provider()->ResumeCall(mCallIndex);
   NS_ENSURE_SUCCESS(rv,rv);
-  
-  ChangeStateInternal(nsIRadioInterfaceLayer::CALL_STATE_RESUMING, true);
+
+  ChangeStateInternal(nsITelephonyProvider::CALL_STATE_RESUMING, true);
   return NS_OK;
 }
 
