@@ -30,10 +30,12 @@
 #include "DOMBindingInlines.h"
 #include "mozilla/Attributes.h"
 
+using namespace mozilla;
+
+using namespace mozilla::dom;
 USING_WORKERS_NAMESPACE
 
 using mozilla::dom::workers::exceptions::ThrowDOMExceptionForNSResult;
-using mozilla::ErrorResult;
 
 // XXX Need to figure this out...
 #define UNCATCHABLE_EXCEPTION NS_ERROR_OUT_OF_MEMORY
@@ -90,20 +92,24 @@ public:
   WorkerPrivate* mWorkerPrivate;
   XMLHttpRequest* mXMLHttpRequestPrivate;
 
+  // XHR Params:
+  bool mMozAnon;
+  bool mMozSystem;
+
   // Only touched on the main thread.
   nsRefPtr<nsXMLHttpRequest> mXHR;
   nsCOMPtr<nsIXMLHttpRequestUpload> mXHRUpload;
-  PRUint32 mInnerEventStreamId;
-  PRUint32 mInnerChannelId;
-  PRUint32 mOutstandingSendCount;
+  uint32_t mInnerEventStreamId;
+  uint32_t mInnerChannelId;
+  uint32_t mOutstandingSendCount;
 
   // Only touched on the worker thread.
-  PRUint32 mOuterEventStreamId;
-  PRUint32 mOuterChannelId;
-  PRUint64 mLastLoaded;
-  PRUint64 mLastTotal;
-  PRUint64 mLastUploadLoaded;
-  PRUint64 mLastUploadTotal;
+  uint32_t mOuterEventStreamId;
+  uint32_t mOuterChannelId;
+  uint64_t mLastLoaded;
+  uint64_t mLastTotal;
+  uint64_t mLastUploadLoaded;
+  uint64_t mLastUploadTotal;
   bool mIsSyncXHR;
   bool mLastLengthComputable;
   bool mLastUploadLengthComputable;
@@ -111,8 +117,8 @@ public:
   bool mSeenUploadLoadStart;
 
   // Only touched on the main thread.
-  PRUint32 mSyncQueueKey;
-  PRUint32 mSyncEventResponseSyncQueueKey;
+  uint32_t mSyncQueueKey;
+  uint32_t mSyncEventResponseSyncQueueKey;
   bool mUploadEventListenersAttached;
   bool mMainThreadSeenLoadStart;
   bool mInOpen;
@@ -121,15 +127,16 @@ public:
   NS_DECL_ISUPPORTS
   NS_DECL_NSIDOMEVENTLISTENER
 
-  Proxy(XMLHttpRequest* aXHRPrivate)
-  : mWorkerPrivate(nsnull), mXMLHttpRequestPrivate(aXHRPrivate),
+  Proxy(XMLHttpRequest* aXHRPrivate, bool aMozAnon, bool aMozSystem)
+  : mWorkerPrivate(nullptr), mXMLHttpRequestPrivate(aXHRPrivate),
+    mMozAnon(aMozAnon), mMozSystem(aMozSystem),
     mInnerEventStreamId(0), mInnerChannelId(0), mOutstandingSendCount(0),
     mOuterEventStreamId(0), mOuterChannelId(0), mLastLoaded(0), mLastTotal(0),
     mLastUploadLoaded(0), mLastUploadTotal(0), mIsSyncXHR(false),
     mLastLengthComputable(false), mLastUploadLengthComputable(false),
     mSeenLoadStart(false), mSeenUploadLoadStart(false),
-    mSyncQueueKey(PR_UINT32_MAX),
-    mSyncEventResponseSyncQueueKey(PR_UINT32_MAX),
+    mSyncQueueKey(UINT32_MAX),
+    mSyncEventResponseSyncQueueKey(UINT32_MAX),
     mUploadEventListenersAttached(false), mMainThreadSeenLoadStart(false),
     mInOpen(false)
   { }
@@ -148,24 +155,41 @@ public:
     NS_ASSERTION(mWorkerPrivate, "Must have a worker here!");
 
     if (!mXHR) {
+      nsPIDOMWindow* ownerWindow = mWorkerPrivate->GetWindow();
+      if (ownerWindow) {
+        ownerWindow = ownerWindow->GetOuterWindow();
+        if (!ownerWindow) {
+          NS_ERROR("No outer window?!");
+          return false;
+        }
+
+        nsPIDOMWindow* innerWindow = ownerWindow->GetCurrentInnerWindow();
+        if (mWorkerPrivate->GetWindow() != innerWindow) {
+          NS_WARNING("Window has navigated, cannot create XHR here.");
+          return false;
+        }
+      }
+
       mXHR = new nsXMLHttpRequest();
 
+      nsCOMPtr<nsIGlobalObject> global = do_QueryInterface(ownerWindow);
       if (NS_FAILED(mXHR->Init(mWorkerPrivate->GetPrincipal(),
                                mWorkerPrivate->GetScriptContext(),
-                               mWorkerPrivate->GetWindow(),
-                               mWorkerPrivate->GetBaseURI()))) {
-        mXHR = nsnull;
+                               global, mWorkerPrivate->GetBaseURI()))) {
+        mXHR = nullptr;
         return false;
       }
 
+      mXHR->SetParameters(mMozAnon, mMozSystem);
+
       if (NS_FAILED(mXHR->GetUpload(getter_AddRefs(mXHRUpload)))) {
-        mXHR = nsnull;
+        mXHR = nullptr;
         return false;
       }
 
       if (!AddRemoveEventListeners(false, true)) {
-        mXHRUpload = nsnull;
-        mXHR = nsnull;
+        mXHRUpload = nullptr;
+        mXHR = nullptr;
         return false;
       }
     }
@@ -189,11 +213,11 @@ public:
     }
   }
 
-  PRUint32
+  uint32_t
   GetSyncQueueKey()
   {
     AssertIsOnMainThread();
-    return mSyncEventResponseSyncQueueKey == PR_UINT32_MAX ?
+    return mSyncEventResponseSyncQueueKey == UINT32_MAX ?
            mSyncQueueKey :
            mSyncEventResponseSyncQueueKey;
   }
@@ -203,8 +227,8 @@ public:
   {
     AssertIsOnMainThread();
 
-    return mSyncQueueKey == PR_UINT32_MAX &&
-           mSyncEventResponseSyncQueueKey == PR_UINT32_MAX;
+    return mSyncQueueKey == UINT32_MAX &&
+           mSyncEventResponseSyncQueueKey == UINT32_MAX;
   }
 };
 
@@ -239,7 +263,7 @@ ConvertStringToResponseType(const nsAString& aString)
   }
 
   MOZ_NOT_REACHED("Don't know anything about this response type!");
-  return _empty;
+  return XMLHttpRequestResponseType::_empty;
 }
 
 enum
@@ -278,34 +302,6 @@ const char* const sEventStrings[] = {
 
 JS_STATIC_ASSERT(JS_ARRAY_LENGTH(sEventStrings) == STRING_COUNT);
 
-class MainThreadSyncRunnable : public WorkerSyncRunnable
-{
-public:
-  MainThreadSyncRunnable(WorkerPrivate* aWorkerPrivate,
-                         ClearingBehavior aClearingBehavior,
-                         PRUint32 aSyncQueueKey,
-                         bool aBypassSyncEventQueue)
-  : WorkerSyncRunnable(aWorkerPrivate, aSyncQueueKey, aBypassSyncEventQueue,
-                       aClearingBehavior)
-  {
-    AssertIsOnMainThread();
-  }
-
-  bool
-  PreDispatch(JSContext* aCx, WorkerPrivate* aWorkerPrivate)
-  {
-    AssertIsOnMainThread();
-    return true;
-  }
-
-  void
-  PostDispatch(JSContext* aCx, WorkerPrivate* aWorkerPrivate,
-               bool aDispatchResult)
-  {
-    AssertIsOnMainThread();
-  }
-};
-
 class MainThreadProxyRunnable : public MainThreadSyncRunnable
 {
 protected:
@@ -371,7 +367,7 @@ public:
     AssertIsOnMainThread();
 
     mProxy->Teardown();
-    mProxy = nsnull;
+    mProxy = nullptr;
 
     return NS_OK;
   }
@@ -386,16 +382,16 @@ class LoadStartDetectionRunnable MOZ_FINAL : public nsIRunnable,
   XMLHttpRequest* mXMLHttpRequestPrivate;
   nsString mEventType;
   bool mReceivedLoadStart;
-  PRUint32 mChannelId;
+  uint32_t mChannelId;
 
   class ProxyCompleteRunnable : public MainThreadProxyRunnable
   {
     XMLHttpRequest* mXMLHttpRequestPrivate;
-    PRUint32 mChannelId;
+    uint32_t mChannelId;
 
   public:
     ProxyCompleteRunnable(WorkerPrivate* aWorkerPrivate, Proxy* aProxy,
-                          XMLHttpRequest* aXHRPrivate, PRUint32 aChannelId)
+                          XMLHttpRequest* aXHRPrivate, uint32_t aChannelId)
     : MainThreadProxyRunnable(aWorkerPrivate, RunWhenClearing, aProxy),
       mXMLHttpRequestPrivate(aXHRPrivate), mChannelId(aChannelId)
     { }
@@ -422,7 +418,7 @@ class LoadStartDetectionRunnable MOZ_FINAL : public nsIRunnable,
         return true;
       }
 
-      if (mSyncQueueKey != PR_UINT32_MAX) {
+      if (mSyncQueueKey != UINT32_MAX) {
         aWorkerPrivate->StopSyncLoop(mSyncQueueKey, true);
       }
 
@@ -481,16 +477,16 @@ public:
           new ProxyCompleteRunnable(mWorkerPrivate, mProxy,
                                     mXMLHttpRequestPrivate,
                                     mChannelId);
-        if (runnable->Dispatch(nsnull)) {
-          mProxy->mWorkerPrivate = nsnull;
+        if (runnable->Dispatch(nullptr)) {
+          mProxy->mWorkerPrivate = nullptr;
           mProxy->mOutstandingSendCount--;
         }
       }
     }
 
-    mProxy = nsnull;
-    mXHR = nsnull;
-    mXMLHttpRequestPrivate = nsnull;
+    mProxy = nullptr;
+    mXHR = nullptr;
+    mXMLHttpRequestPrivate = nullptr;
     return NS_OK;
   }
 
@@ -527,11 +523,11 @@ class EventRunnable : public MainThreadProxyRunnable
   jsval mResponse;
   nsString mResponseText;
   nsString mStatusText;
-  PRUint64 mLoaded;
-  PRUint64 mTotal;
-  PRUint32 mEventStreamId;
-  PRUint32 mStatus;
-  PRUint16 mReadyState;
+  uint64_t mLoaded;
+  uint64_t mTotal;
+  uint32_t mEventStreamId;
+  uint32_t mStatus;
+  uint16_t mReadyState;
   bool mUploadEvent;
   bool mProgressEvent;
   bool mLengthComputable;
@@ -541,7 +537,7 @@ class EventRunnable : public MainThreadProxyRunnable
 
 public:
   EventRunnable(Proxy* aProxy, bool aUploadEvent, const nsString& aType,
-                bool aLengthComputable, PRUint64 aLoaded, PRUint64 aTotal)
+                bool aLengthComputable, uint64_t aLoaded, uint64_t aTotal)
   : MainThreadProxyRunnable(aProxy->mWorkerPrivate, SkipWhenClearing, aProxy),
     mType(aType), mResponse(JSVAL_VOID), mLoaded(aLoaded), mTotal(aTotal),
     mEventStreamId(aProxy->mInnerEventStreamId), mStatus(0), mReadyState(0),
@@ -576,8 +572,8 @@ public:
       }
     }
     else {
-      jsval response;
-      mResponseResult = xhr->GetResponse(aCx, &response);
+      JS::Rooted<JS::Value> response(aCx);
+      mResponseResult = xhr->GetResponse(aCx, response.address());
       if (NS_SUCCEEDED(mResponseResult)) {
         if (JSVAL_IS_UNIVERSAL(response)) {
           mResponse = response;
@@ -606,7 +602,7 @@ public:
 
     xhr->GetStatusText(mStatusText);
 
-    mReadyState = xhr->GetReadyState();
+    mReadyState = xhr->ReadyState();
 
     return true;
   }
@@ -696,8 +692,8 @@ public:
           nsTArray<nsCOMPtr<nsISupports> > clonedObjects;
           clonedObjects.SwapElements(mClonedObjects);
 
-          jsval response;
-          if (!responseBuffer.read(aCx, &response, callbacks, &clonedObjects)) {
+          JS::Rooted<JS::Value> response(aCx);
+          if (!responseBuffer.read(aCx, response.address(), callbacks, &clonedObjects)) {
             return false;
           }
 
@@ -723,23 +719,23 @@ public:
       return true;
     }
 
-    JSString* type = JS_NewUCStringCopyN(aCx, mType.get(), mType.Length());
+    JS::Rooted<JSString*> type(aCx, JS_NewUCStringCopyN(aCx, mType.get(), mType.Length()));
     if (!type) {
       return false;
     }
 
-    JSObject* event = mProgressEvent ?
+    JS::Rooted<JSObject*> event(aCx, mProgressEvent ?
                       events::CreateProgressEvent(aCx, type, mLengthComputable,
                                                   mLoaded, mTotal) :
                       events::CreateGenericEvent(aCx, type, false, false,
-                                                 false);
+                                                 false));
     if (!event) {
       return false;
     }
 
-    JSObject* target = mUploadEvent ?
+    JS::Rooted<JSObject*> target(aCx, mUploadEvent ?
                        xhr->GetUploadObjectNoCreate()->GetJSObject() :
-                       xhr->GetJSObject();
+                       xhr->GetJSObject());
     MOZ_ASSERT(target);
 
     bool dummy;
@@ -762,17 +758,17 @@ class WorkerThreadProxySyncRunnable : public nsRunnable
 protected:
   WorkerPrivate* mWorkerPrivate;
   nsRefPtr<Proxy> mProxy;
-  PRUint32 mSyncQueueKey;
+  uint32_t mSyncQueueKey;
 
 private:
   class ResponseRunnable : public MainThreadProxyRunnable
   {
-    PRUint32 mSyncQueueKey;
+    uint32_t mSyncQueueKey;
     nsresult mErrorCode;
 
   public:
     ResponseRunnable(WorkerPrivate* aWorkerPrivate, Proxy* aProxy,
-                     PRUint32 aSyncQueueKey, nsresult aErrorCode)
+                     uint32_t aSyncQueueKey, nsresult aErrorCode)
     : MainThreadProxyRunnable(aWorkerPrivate, SkipWhenClearing, aProxy),
       mSyncQueueKey(aSyncQueueKey), mErrorCode(aErrorCode)
     {
@@ -807,18 +803,15 @@ public:
   {
     mWorkerPrivate->AssertIsOnWorkerThread();
 
-    mSyncQueueKey = mWorkerPrivate->CreateNewSyncLoop();
+    AutoSyncLoopHolder syncLoop(mWorkerPrivate);
+    mSyncQueueKey = syncLoop.SyncQueueKey();
 
     if (NS_FAILED(NS_DispatchToMainThread(this, NS_DISPATCH_NORMAL))) {
       JS_ReportError(aCx, "Failed to dispatch to main thread!");
       return false;
     }
 
-    if (!mWorkerPrivate->RunSyncLoop(aCx, mSyncQueueKey)) {
-      return false;
-    }
-
-    return true;
+    return syncLoop.RunAndForget(aCx);
   }
 
   virtual nsresult
@@ -829,14 +822,14 @@ public:
   {
     AssertIsOnMainThread();
 
-    PRUint32 oldSyncQueueKey = mProxy->mSyncEventResponseSyncQueueKey;
+    uint32_t oldSyncQueueKey = mProxy->mSyncEventResponseSyncQueueKey;
     mProxy->mSyncEventResponseSyncQueueKey = mSyncQueueKey;
 
     nsresult rv = MainThreadRun();
 
     nsRefPtr<ResponseRunnable> response =
       new ResponseRunnable(mWorkerPrivate, mProxy, mSyncQueueKey, rv);
-    if (!response->Dispatch(nsnull)) {
+    if (!response->Dispatch(nullptr)) {
       NS_WARNING("Failed to dispatch response!");
     }
 
@@ -867,23 +860,6 @@ public:
   }
 };
 
-class SetMultipartRunnable : public WorkerThreadProxySyncRunnable
-{
-  bool mValue;
-
-public:
-  SetMultipartRunnable(WorkerPrivate* aWorkerPrivate, Proxy* aProxy,
-                       bool aValue)
-  : WorkerThreadProxySyncRunnable(aWorkerPrivate, aProxy), mValue(aValue)
-  { }
-
-  nsresult
-  MainThreadRun()
-  {
-    return mProxy->mXHR->SetMultipart(mValue);
-  }
-};
-
 class SetBackgroundRequestRunnable : public WorkerThreadProxySyncRunnable
 {
   bool mValue;
@@ -948,11 +924,11 @@ public:
 
 class SetTimeoutRunnable : public WorkerThreadProxySyncRunnable
 {
-  PRUint32 mTimeout;
+  uint32_t mTimeout;
 
 public:
   SetTimeoutRunnable(WorkerPrivate* aWorkerPrivate, Proxy* aProxy,
-                     PRUint32 aTimeout)
+                     uint32_t aTimeout)
   : WorkerThreadProxySyncRunnable(aWorkerPrivate, aProxy),
     mTimeout(aTimeout)
   { }
@@ -1035,20 +1011,19 @@ class OpenRunnable : public WorkerThreadProxySyncRunnable
   nsString mUserStr;
   Optional<nsAString> mPassword;
   nsString mPasswordStr;
-  bool mMultipart;
   bool mBackgroundRequest;
   bool mWithCredentials;
-  PRUint32 mTimeout;
+  uint32_t mTimeout;
 
 public:
   OpenRunnable(WorkerPrivate* aWorkerPrivate, Proxy* aProxy,
                const nsAString& aMethod, const nsAString& aURL,
                const Optional<nsAString>& aUser,
                const Optional<nsAString>& aPassword,
-               bool aMultipart, bool aBackgroundRequest, bool aWithCredentials,
-               PRUint32 aTimeout)
+               bool aBackgroundRequest, bool aWithCredentials,
+               uint32_t aTimeout)
   : WorkerThreadProxySyncRunnable(aWorkerPrivate, aProxy), mMethod(aMethod),
-    mURL(aURL), mMultipart(aMultipart),
+    mURL(aURL),
     mBackgroundRequest(aBackgroundRequest), mWithCredentials(aWithCredentials),
     mTimeout(aTimeout)
   {
@@ -1083,11 +1058,6 @@ public:
 
     nsresult rv;
 
-    if (mMultipart) {
-      rv = mProxy->mXHR->SetMultipart(mMultipart);
-      NS_ENSURE_SUCCESS(rv, rv);
-    }
-
     if (mBackgroundRequest) {
       rv = mProxy->mXHR->SetMozBackgroundRequest(mBackgroundRequest);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -1127,14 +1097,14 @@ class SendRunnable : public WorkerThreadProxySyncRunnable
   nsString mStringBody;
   JSAutoStructuredCloneBuffer mBody;
   nsTArray<nsCOMPtr<nsISupports> > mClonedObjects;
-  PRUint32 mSyncQueueKey;
+  uint32_t mSyncQueueKey;
   bool mHasUploadListeners;
 
 public:
   SendRunnable(WorkerPrivate* aWorkerPrivate, Proxy* aProxy,
                const nsAString& aStringBody, JSAutoStructuredCloneBuffer& aBody,
                nsTArray<nsCOMPtr<nsISupports> >& aClonedObjects,
-               PRUint32 aSyncQueueKey, bool aHasUploadListeners)
+               uint32_t aSyncQueueKey, bool aHasUploadListeners)
   : WorkerThreadProxySyncRunnable(aWorkerPrivate, aProxy),
     mStringBody(aStringBody), mSyncQueueKey(aSyncQueueKey),
     mHasUploadListeners(aHasUploadListeners)
@@ -1149,7 +1119,8 @@ public:
     nsCOMPtr<nsIVariant> variant;
 
     if (mBody.data()) {
-      RuntimeService::AutoSafeJSContext cx;
+      AutoSafeJSContext cx;
+      JSAutoRequest ar(cx);
       nsIXPConnect* xpc = nsContentUtils::XPConnect();
       NS_ASSERTION(xpc, "This should never be null!");
 
@@ -1160,9 +1131,9 @@ public:
         ChromeWorkerStructuredCloneCallbacks(true) :
         WorkerStructuredCloneCallbacks(true);
 
-      jsval body;
-      if (mBody.read(cx, &body, callbacks, &mClonedObjects)) {
-        if (NS_FAILED(xpc->JSValToVariant(cx, &body,
+      JS::Rooted<JS::Value> body(cx);
+      if (mBody.read(cx, body.address(), callbacks, &mClonedObjects)) {
+        if (NS_FAILED(xpc->JSValToVariant(cx, body.address(),
                                           getter_AddRefs(variant)))) {
           rv = NS_ERROR_DOM_INVALID_STATE_ERR;
         }
@@ -1191,7 +1162,7 @@ public:
     NS_ASSERTION(!mProxy->mWorkerPrivate, "Should be null!");
     mProxy->mWorkerPrivate = mWorkerPrivate;
 
-    NS_ASSERTION(mProxy->mSyncQueueKey == PR_UINT32_MAX, "Should be unset!");
+    NS_ASSERTION(mProxy->mSyncQueueKey == UINT32_MAX, "Should be unset!");
     mProxy->mSyncQueueKey = mSyncQueueKey;
 
     if (mHasUploadListeners) {
@@ -1300,16 +1271,16 @@ Proxy::Teardown()
     if (mOutstandingSendCount) {
       nsRefPtr<XHRUnpinRunnable> runnable =
         new XHRUnpinRunnable(mWorkerPrivate, mXMLHttpRequestPrivate);
-      if (!runnable->Dispatch(nsnull)) {
+      if (!runnable->Dispatch(nullptr)) {
         NS_RUNTIMEABORT("We're going to hang at shutdown anyways.");
       }
 
-      mWorkerPrivate = nsnull;
+      mWorkerPrivate = nullptr;
       mOutstandingSendCount = 0;
     }
 
-    mXHRUpload = nsnull;
-    mXHR = nsnull;
+    mXHRUpload = nullptr;
+    mXHR = nullptr;
   }
 }
 
@@ -1329,10 +1300,10 @@ Proxy::AddRemoveEventListeners(bool aUpload, bool aAdd)
     do_QueryInterface(static_cast<nsIXMLHttpRequest*>(mXHR.get()));
   NS_ASSERTION(target, "This should never fail!");
 
-  PRUint32 lastEventType = aUpload ? STRING_LAST_EVENTTARGET : STRING_LAST_XHR;
+  uint32_t lastEventType = aUpload ? STRING_LAST_EVENTTARGET : STRING_LAST_XHR;
 
   nsAutoString eventType;
-  for (PRUint32 index = 0; index <= lastEventType; index++) {
+  for (uint32_t index = 0; index <= lastEventType; index++) {
     eventType = NS_ConvertASCIItoUTF16(sEventStrings[index]);
     if (aAdd) {
       if (NS_FAILED(target->AddEventListener(eventType, this, false))) {
@@ -1381,7 +1352,7 @@ Proxy::HandleEvent(nsIDOMEvent* aEvent)
   nsRefPtr<EventRunnable> runnable;
 
   if (mInOpen && type.EqualsASCII(sEventStrings[STRING_readystatechange])) {
-    PRUint16 readyState = 0;
+    uint16_t readyState = 0;
     if (NS_SUCCEEDED(mXHR->GetReadyState(&readyState)) &&
         readyState == nsIXMLHttpRequest::OPENED) {
       mInnerEventStreamId++;
@@ -1390,7 +1361,7 @@ Proxy::HandleEvent(nsIDOMEvent* aEvent)
 
   if (progressEvent) {
     bool lengthComputable;
-    PRUint64 loaded, total;
+    uint64_t loaded, total;
     if (NS_FAILED(progressEvent->GetLengthComputable(&lengthComputable)) ||
         NS_FAILED(progressEvent->GetLoaded(&loaded)) ||
         NS_FAILED(progressEvent->GetTotal(&total))) {
@@ -1405,7 +1376,8 @@ Proxy::HandleEvent(nsIDOMEvent* aEvent)
   }
 
   {
-    RuntimeService::AutoSafeJSContext cx;
+    AutoSafeJSContext cx;
+    JSAutoRequest ar(cx);
     runnable->Dispatch(cx);
   }
 
@@ -1432,9 +1404,9 @@ Proxy::HandleEvent(nsIDOMEvent* aEvent)
 XMLHttpRequest::XMLHttpRequest(JSContext* aCx, WorkerPrivate* aWorkerPrivate)
 : XMLHttpRequestEventTarget(aCx), mJSObject(NULL), mUpload(NULL),
   mWorkerPrivate(aWorkerPrivate),
-  mResponseType(XMLHttpRequestResponseTypeValues::Text), mTimeout(0),
-  mJSObjectRooted(false), mMultipart(false), mBackgroundRequest(false),
-  mWithCredentials(false), mCanceled(false)
+  mResponseType(XMLHttpRequestResponseType::Text), mTimeout(0),
+  mJSObjectRooted(false), mBackgroundRequest(false),
+  mWithCredentials(false), mCanceled(false), mMozAnon(false), mMozSystem(false)
 {
   mWorkerPrivate->AssertIsOnWorkerThread();
 }
@@ -1449,9 +1421,9 @@ void
 XMLHttpRequest::_trace(JSTracer* aTrc)
 {
   if (mUpload) {
-    JS_CALL_OBJECT_TRACER(aTrc, mUpload->GetJSObject(), "mUpload");
+    mUpload->TraceJSObject(aTrc, "mUpload");
   }
-  JS_CALL_VALUE_TRACER(aTrc, mStateData.mResponse, "mResponse");
+  JS_CallValueTracer(aTrc, &mStateData.mResponse, "mResponse");
   XMLHttpRequestEventTarget::_trace(aTrc);
 }
 
@@ -1464,22 +1436,25 @@ XMLHttpRequest::_finalize(JSFreeOp* aFop)
 
 // static
 XMLHttpRequest*
-XMLHttpRequest::Constructor(JSContext* aCx,
-                            JSObject* aGlobal,
-                            const Nullable<MozXMLHttpRequestParametersWorkers>& aParams,
+XMLHttpRequest::Constructor(const WorkerGlobalObject& aGlobal,
+                            const MozXMLHttpRequestParametersWorkers& aParams,
                             ErrorResult& aRv)
 {
-  WorkerPrivate* workerPrivate = GetWorkerPrivateFromContext(aCx);
+  JSContext* cx = aGlobal.GetContext();
+  WorkerPrivate* workerPrivate = GetWorkerPrivateFromContext(cx);
   MOZ_ASSERT(workerPrivate);
 
-  nsRefPtr<XMLHttpRequest> xhr = new XMLHttpRequest(aCx, workerPrivate);
+  nsRefPtr<XMLHttpRequest> xhr = new XMLHttpRequest(cx, workerPrivate);
 
-  if (!Wrap(aCx, aGlobal, xhr)) {
+  if (!Wrap(cx, aGlobal.Get(), xhr)) {
     aRv.Throw(NS_ERROR_FAILURE);
     return NULL;
   }
 
-  // TODO: process aParams. See bug 761227
+  if (workerPrivate->XHRParamsAllowed()) {
+    xhr->mMozAnon = aParams.mMozAnon;
+    xhr->mMozSystem = aParams.mMozSystem;
+  }
 
   xhr->mJSObject = xhr->GetJSObject();
   return xhr;
@@ -1497,9 +1472,9 @@ XMLHttpRequest::ReleaseProxy(ReleaseType aType)
       // need to).
       nsRefPtr<AsyncTeardownRunnable> runnable =
         new AsyncTeardownRunnable(mProxy);
-      mProxy = nsnull;
+      mProxy = nullptr;
 
-      if (NS_DispatchToMainThread(runnable)) {
+      if (NS_FAILED(NS_DispatchToMainThread(runnable))) {
         NS_ERROR("Failed to dispatch teardown runnable!");
       }
     } else {
@@ -1513,9 +1488,9 @@ XMLHttpRequest::ReleaseProxy(ReleaseType aType)
       // We need to make a sync call here.
       nsRefPtr<SyncTeardownRunnable> runnable =
         new SyncTeardownRunnable(mWorkerPrivate, mProxy);
-      mProxy = nsnull;
+      mProxy = nullptr;
 
-      if (!runnable->Dispatch(nsnull)) {
+      if (!runnable->Dispatch(nullptr)) {
         NS_ERROR("Failed to dispatch teardown runnable!");
       }
     }
@@ -1558,7 +1533,7 @@ XMLHttpRequest::MaybeDispatchPrematureAbortEvents(ErrorResult& aRv)
   if (mProxy->mSeenUploadLoadStart) {
     MOZ_ASSERT(mUpload);
 
-    JSObject* target = mUpload->GetJSObject();
+    JS::Rooted<JSObject*> target(GetJSContext(), mUpload->GetJSObject());
     MOZ_ASSERT(target);
 
     DispatchPrematureAbortEvent(target, STRING_abort, true, aRv);
@@ -1575,7 +1550,7 @@ XMLHttpRequest::MaybeDispatchPrematureAbortEvents(ErrorResult& aRv)
   }
 
   if (mProxy->mSeenLoadStart) {
-    JSObject* target = GetJSObject();
+    JS::Rooted<JSObject*> target(GetJSContext(), GetJSObject());
     MOZ_ASSERT(target);
 
     DispatchPrematureAbortEvent(target, STRING_readystatechange, false, aRv);
@@ -1598,7 +1573,7 @@ XMLHttpRequest::MaybeDispatchPrematureAbortEvents(ErrorResult& aRv)
 }
 
 void
-XMLHttpRequest::DispatchPrematureAbortEvent(JSObject* aTarget,
+XMLHttpRequest::DispatchPrematureAbortEvent(JS::Handle<JSObject*> aTarget,
                                             uint8_t aEventType,
                                             bool aUploadTarget,
                                             ErrorResult& aRv)
@@ -1614,13 +1589,13 @@ XMLHttpRequest::DispatchPrematureAbortEvent(JSObject* aTarget,
 
   JSContext* cx = GetJSContext();
 
-  JSString* type = JS_NewStringCopyZ(cx, sEventStrings[aEventType]);
+  JS::Rooted<JSString*> type(cx, JS_NewStringCopyZ(cx, sEventStrings[aEventType]));
   if (!type) {
     aRv.Throw(NS_ERROR_OUT_OF_MEMORY);
     return;
   }
 
-  JSObject* event;
+  JS::Rooted<JSObject*> event(cx);
   if (aEventType == STRING_readystatechange) {
     event = events::CreateGenericEvent(cx, type, false, false, false);
   }
@@ -1681,10 +1656,13 @@ XMLHttpRequest::SendInternal(const nsAString& aStringBody,
   }
 
   AutoUnpinXHR autoUnpin(this);
+  Maybe<AutoSyncLoopHolder> autoSyncLoop;
 
-  PRUint32 syncQueueKey = PR_UINT32_MAX;
-  if (mProxy->mIsSyncXHR) {
-    syncQueueKey = mWorkerPrivate->CreateNewSyncLoop();
+  uint32_t syncQueueKey = UINT32_MAX;
+  bool isSyncXHR = mProxy->mIsSyncXHR;
+  if (isSyncXHR) {
+    autoSyncLoop.construct(mWorkerPrivate);
+    syncQueueKey = autoSyncLoop.ref().SyncQueueKey();
   }
 
   mProxy->mOuterChannelId++;
@@ -1695,19 +1673,28 @@ XMLHttpRequest::SendInternal(const nsAString& aStringBody,
     new SendRunnable(mWorkerPrivate, mProxy, aStringBody, aBody,
                      aClonedObjects, syncQueueKey, hasUploadListeners);
   if (!runnable->Dispatch(cx)) {
+    aRv.Throw(NS_ERROR_FAILURE);
+    return;
+  }
+
+  if (!isSyncXHR)  {
+    autoUnpin.Clear();
+    MOZ_ASSERT(autoSyncLoop.empty());
+    return;
+  }
+
+  // If our sync XHR was canceled during the send call the worker is going
+  // away.  We have no idea how far through the send call we got.  There may
+  // be a ProxyCompleteRunnable in the sync loop, but rather than run the loop
+  // to get it we just let our RAII helpers clean up.
+  if (mCanceled) {
     return;
   }
 
   autoUnpin.Clear();
 
-  // The event loop was spun above, make sure we aren't canceled already.
-  if (mCanceled) {
-    return;
-  }
-
-  if (mProxy->mIsSyncXHR && !mWorkerPrivate->RunSyncLoop(cx, syncQueueKey)) {
+  if (!autoSyncLoop.ref().RunAndForget(cx)) {
     aRv.Throw(NS_ERROR_FAILURE);
-    return;
   }
 }
 
@@ -1744,14 +1731,14 @@ XMLHttpRequest::Open(const nsAString& aMethod, const nsAString& aUrl,
     }
   }
   else {
-    mProxy = new Proxy(this);
+    mProxy = new Proxy(this, mMozAnon, mMozSystem);
   }
 
   mProxy->mOuterEventStreamId++;
 
   nsRefPtr<OpenRunnable> runnable =
     new OpenRunnable(mWorkerPrivate, mProxy, aMethod, aUrl, aUser, aPassword,
-                     mMultipart, mBackgroundRequest, mWithCredentials,
+                     mBackgroundRequest, mWithCredentials,
                      mTimeout);
 
   if (!runnable->Dispatch(GetJSContext())) {
@@ -1841,32 +1828,6 @@ XMLHttpRequest::SetWithCredentials(bool aWithCredentials, ErrorResult& aRv)
   }
 }
 
-void
-XMLHttpRequest::SetMultipart(bool aMultipart, ErrorResult& aRv)
-{
-  mWorkerPrivate->AssertIsOnWorkerThread();
-
-  if (mCanceled) {
-    aRv.Throw(UNCATCHABLE_EXCEPTION);
-    return;
-  }
-
-  mMultipart = aMultipart;
-
-  if (!mProxy) {
-    // Open may not have been called yet, in which case we'll handle the
-    // multipart in OpenRunnable.
-    return;
-  }
-
-  nsRefPtr<SetMultipartRunnable> runnable =
-    new SetMultipartRunnable(mWorkerPrivate, mProxy, aMultipart);
-  if (!runnable->Dispatch(GetJSContext())) {
-    aRv.Throw(NS_ERROR_FAILURE);
-    return;
-  }
-}
-
 void
 XMLHttpRequest::SetMozBackgroundRequest(bool aBackgroundRequest,
                                         ErrorResult& aRv)
@@ -1983,7 +1944,8 @@ XMLHttpRequest::Send(JSObject* aBody, ErrorResult& aRv)
   JSContext* cx = GetJSContext();
 
   jsval valToClone;
-  if (JS_IsArrayBufferObject(aBody, cx) || file::GetDOMBlobFromJSObject(aBody)) {
+  if (JS_IsArrayBufferObject(aBody) || JS_IsArrayBufferViewObject(aBody) ||
+      file::GetDOMBlobFromJSObject(aBody)) {
     valToClone = OBJECT_TO_JSVAL(aBody);
   }
   else {
@@ -2147,7 +2109,7 @@ XMLHttpRequest::SetResponseType(XMLHttpRequestResponseType aResponseType,
 
   // "document" is fine for the main thread but not for a worker. Short-circuit
   // that here.
-  if (aResponseType == XMLHttpRequestResponseTypeValues::Document) {
+  if (aResponseType == XMLHttpRequestResponseType::Document) {
     return;
   }
 
