@@ -15,8 +15,8 @@
 #include "prlong.h"
 #include "nsNetUtil.h"
 #include "nsIProtocolHandler.h"
-
-#include "woff.h"
+#include "nsIPrincipal.h"
+#include "mozilla/Telemetry.h"
 
 #include "opentype-sanitiser.h"
 #include "ots-memory-stream.h"
@@ -24,28 +24,34 @@
 using namespace mozilla;
 
 #ifdef PR_LOGGING
-PRLogModuleInfo *gfxUserFontSet::sUserFontsLog = PR_NewLogModule("userfonts");
+PRLogModuleInfo *
+gfxUserFontSet::GetUserFontsLog()
+{
+    static PRLogModuleInfo *sLog;
+    if (!sLog)
+        sLog = PR_NewLogModule("userfonts");
+    return sLog;
+}
 #endif /* PR_LOGGING */
 
-#define LOG(args) PR_LOG(sUserFontsLog, PR_LOG_DEBUG, args)
-#define LOG_ENABLED() PR_LOG_TEST(sUserFontsLog, PR_LOG_DEBUG)
+#define LOG(args) PR_LOG(GetUserFontsLog(), PR_LOG_DEBUG, args)
+#define LOG_ENABLED() PR_LOG_TEST(GetUserFontsLog(), PR_LOG_DEBUG)
 
-static PRUint64 sFontSetGeneration = LL_INIT(0, 0);
+static uint64_t sFontSetGeneration = 0;
 
 // TODO: support for unicode ranges not yet implemented
 
 gfxProxyFontEntry::gfxProxyFontEntry(const nsTArray<gfxFontFaceSrc>& aFontFaceSrcList,
-             gfxMixedFontFamily *aFamily,
-             PRUint32 aWeight,
-             PRUint32 aStretch,
-             PRUint32 aItalicStyle,
+             uint32_t aWeight,
+             uint32_t aStretch,
+             uint32_t aItalicStyle,
              const nsTArray<gfxFontFeature>& aFeatureSettings,
-             PRUint32 aLanguageOverride,
+             uint32_t aLanguageOverride,
              gfxSparseBitSet *aUnicodeRanges)
-    : gfxFontEntry(NS_LITERAL_STRING("Proxy"), aFamily),
+    : gfxFontEntry(NS_LITERAL_STRING("Proxy")),
       mLoadingState(NOT_LOADING),
       mUnsupportedFormat(false),
-      mLoader(nsnull)
+      mLoader(nullptr)
 {
     mIsProxy = true;
     mSrcList = aFontFaceSrcList;
@@ -66,7 +72,7 @@ gfxFont*
 gfxProxyFontEntry::CreateFontInstance(const gfxFontStyle *aFontStyle, bool aNeedsBold)
 {
     // cannot create an actual font for a proxy entry
-    return nsnull;
+    return nullptr;
 }
 
 gfxUserFontSet::gfxUserFontSet()
@@ -82,14 +88,14 @@ gfxUserFontSet::~gfxUserFontSet()
 gfxFontEntry*
 gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
                             const nsTArray<gfxFontFaceSrc>& aFontFaceSrcList,
-                            PRUint32 aWeight,
-                            PRUint32 aStretch,
-                            PRUint32 aItalicStyle,
+                            uint32_t aWeight,
+                            uint32_t aStretch,
+                            uint32_t aItalicStyle,
                             const nsTArray<gfxFontFeature>& aFeatureSettings,
                             const nsString& aLanguageOverride,
                             gfxSparseBitSet *aUnicodeRanges)
 {
-    gfxProxyFontEntry *proxyEntry = nsnull;
+    gfxProxyFontEntry *proxyEntry = nullptr;
 
     nsAutoString key(aFamilyName);
     ToLowerCase(key);
@@ -108,10 +114,10 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
     }
 
     // construct a new face and add it into the family
-    PRUint32 languageOverride =
+    uint32_t languageOverride =
         gfxFontStyle::ParseFontLanguageOverride(aLanguageOverride);
     proxyEntry =
-        new gfxProxyFontEntry(aFontFaceSrcList, family, aWeight, aStretch,
+        new gfxProxyFontEntry(aFontFaceSrcList, aWeight, aStretch,
                               aItalicStyle,
                               aFeatureSettings,
                               languageOverride,
@@ -149,22 +155,14 @@ gfxUserFontSet::AddFontFace(const nsAString& aFamilyName,
 }
 
 gfxFontEntry*
-gfxUserFontSet::FindFontEntry(const nsAString& aName, 
-                              const gfxFontStyle& aFontStyle, 
-                              bool& aFoundFamily,
+gfxUserFontSet::FindFontEntry(gfxFontFamily *aFamily,
+                              const gfxFontStyle& aFontStyle,
                               bool& aNeedsBold,
                               bool& aWaitForUserFont)
 {
     aWaitForUserFont = false;
-    gfxMixedFontFamily *family = GetFamily(aName);
+    gfxMixedFontFamily *family = static_cast<gfxMixedFontFamily*>(aFamily);
 
-    // no user font defined for this name
-    if (!family) {
-        aFoundFamily = false;
-        return nsnull;
-    }
-
-    aFoundFamily = true;
     gfxFontEntry* fe = family->FindFontForStyle(aFontStyle, aNeedsBold);
 
     // if not a proxy, font has already been loaded
@@ -178,7 +176,7 @@ gfxUserFontSet::FindFontEntry(const nsAString& aName,
     if (proxyEntry->mLoadingState > gfxProxyFontEntry::NOT_LOADING) {
         aWaitForUserFont =
             (proxyEntry->mLoadingState < gfxProxyFontEntry::LOADING_SLOWLY);
-        return nsnull;
+        return nullptr;
     }
 
     // hasn't been loaded yet, start the load process
@@ -186,7 +184,7 @@ gfxUserFontSet::FindFontEntry(const nsAString& aName,
 
     // NOTE that if all sources in the entry fail, this will delete proxyEntry,
     // so we cannot use it again if status==STATUS_END_OF_LIST
-    status = LoadNext(proxyEntry);
+    status = LoadNext(family, proxyEntry);
 
     // if the load succeeded immediately, the font entry was replaced so
     // search again
@@ -200,59 +198,7 @@ gfxUserFontSet::FindFontEntry(const nsAString& aName,
         (proxyEntry->mLoadingState < gfxProxyFontEntry::LOADING_SLOWLY);
 
     // if either loading or an error occurred, return null
-    return nsnull;
-}
-
-// Given a buffer of downloaded font data, do any necessary preparation
-// to make it into usable OpenType.
-// May return the original pointer unchanged, or a newly-allocated
-// block (in which case the passed-in block is NS_Free'd).
-// aLength is updated if necessary to the new length of the data.
-// Returns NULL and NS_Free's the incoming data in case of errors.
-static const PRUint8*
-PrepareOpenTypeData(const PRUint8* aData, PRUint32* aLength)
-{
-    switch(gfxFontUtils::DetermineFontDataType(aData, *aLength)) {
-    
-    case GFX_USERFONT_OPENTYPE:
-        // nothing to do
-        return aData;
-        
-    case GFX_USERFONT_WOFF: {
-        PRUint32 status = eWOFF_ok;
-        PRUint32 bufferSize = woffGetDecodedSize(aData, *aLength, &status);
-        if (WOFF_FAILURE(status)) {
-            break;
-        }
-        PRUint8* decodedData = static_cast<PRUint8*>(NS_Alloc(bufferSize));
-        if (!decodedData) {
-            break;
-        }
-        woffDecodeToBuffer(aData, *aLength,
-                           decodedData, bufferSize,
-                           aLength, &status);
-        // replace original data with the decoded version
-        NS_Free((void*)aData);
-        aData = decodedData;
-        if (WOFF_FAILURE(status)) {
-            // something went wrong, discard the data and return NULL
-            break;
-        }
-        // success, return the decoded data
-        return aData;
-    }
-
-    // xxx - add support for other wrappers here
-
-    default:
-        NS_WARNING("unknown font format");
-        break;
-    }
-
-    // discard downloaded data that couldn't be used
-    NS_Free((void*)aData);
-
-    return nsnull;
+    return nullptr;
 }
 
 // Based on ots::ExpandingMemoryStream from ots-memory-stream.h,
@@ -274,7 +220,7 @@ public:
     // when finished with it
     void* forget() {
         void* p = mPtr;
-        mPtr = nsnull;
+        mPtr = nullptr;
         return p;
     }
 
@@ -324,8 +270,9 @@ private:
 
 #ifdef MOZ_OTS_REPORT_ERRORS
 struct OTSCallbackUserData {
-    gfxUserFontSet    *mFontSet;
-    gfxProxyFontEntry *mProxy;
+    gfxUserFontSet     *mFontSet;
+    gfxMixedFontFamily *mFamily;
+    gfxProxyFontEntry  *mProxy;
 };
 
 /* static */ bool
@@ -342,7 +289,7 @@ gfxUserFontSet::OTSMessage(void *aUserData, const char *format, ...)
     va_end(va);
 
     OTSCallbackUserData *d = static_cast<OTSCallbackUserData*>(aUserData);
-    d->mFontSet->LogMessage(d->mProxy, buf);
+    d->mFontSet->LogMessage(d->mFamily, d->mProxy, buf);
 
     return false;
 }
@@ -350,23 +297,20 @@ gfxUserFontSet::OTSMessage(void *aUserData, const char *format, ...)
 
 // Call the OTS library to sanitize an sfnt before attempting to use it.
 // Returns a newly-allocated block, or NULL in case of fatal errors.
-const PRUint8*
-gfxUserFontSet::SanitizeOpenTypeData(gfxProxyFontEntry *aProxy,
-                                     const PRUint8* aData, PRUint32 aLength,
-                                     PRUint32& aSaneLength, bool aIsCompressed)
+const uint8_t*
+gfxUserFontSet::SanitizeOpenTypeData(gfxMixedFontFamily *aFamily,
+                                     gfxProxyFontEntry *aProxy,
+                                     const uint8_t* aData, uint32_t aLength,
+                                     uint32_t& aSaneLength, bool aIsCompressed)
 {
     // limit output/expansion to 256MB
     ExpandingMemoryStream output(aIsCompressed ? aLength * 2 : aLength,
                                  1024 * 1024 * 256);
-#ifdef MOZ_GRAPHITE
-#define PRESERVE_GRAPHITE true
-#else
-#define PRESERVE_GRAPHITE false
-#endif
 
 #ifdef MOZ_OTS_REPORT_ERRORS
     OTSCallbackUserData userData;
     userData.mFontSet = this;
+    userData.mFamily = aFamily;
     userData.mProxy = aProxy;
 #define ERROR_REPORTING_ARGS &gfxUserFontSet::OTSMessage, &userData,
 #else
@@ -375,19 +319,19 @@ gfxUserFontSet::SanitizeOpenTypeData(gfxProxyFontEntry *aProxy,
 
     if (ots::Process(&output, aData, aLength,
                      ERROR_REPORTING_ARGS
-                     PRESERVE_GRAPHITE)) {
+                     true)) {
         aSaneLength = output.Tell();
-        return static_cast<PRUint8*>(output.forget());
+        return static_cast<uint8_t*>(output.forget());
     } else {
         aSaneLength = 0;
-        return nsnull;
+        return nullptr;
     }
 }
 
 static void
 StoreUserFontData(gfxFontEntry* aFontEntry, gfxProxyFontEntry* aProxy,
                   const nsAString& aOriginalName,
-                  nsTArray<PRUint8>* aMetadata, PRUint32 aMetaOrigLen)
+                  nsTArray<uint8_t>* aMetadata, uint32_t aMetaOrigLen)
 {
     if (!aFontEntry->mUserFontData) {
         aFontEntry->mUserFontData = new gfxUserFontData;
@@ -399,6 +343,7 @@ StoreUserFontData(gfxFontEntry* aFontEntry, gfxProxyFontEntry* aProxy,
         userFontData->mLocalName = src.mLocalName;
     } else {
         userFontData->mURI = src.mURI;
+        userFontData->mPrincipal = aProxy->mPrincipal;
     }
     userFontData->mFormat = src.mFormatFlags;
     userFontData->mRealName = aOriginalName;
@@ -425,10 +370,10 @@ struct WOFFHeader {
 };
 
 void
-gfxUserFontSet::CopyWOFFMetadata(const PRUint8* aFontData,
-                                 PRUint32 aLength,
-                                 nsTArray<PRUint8>* aMetadata,
-                                 PRUint32* aMetaOrigLen)
+gfxUserFontSet::CopyWOFFMetadata(const uint8_t* aFontData,
+                                 uint32_t aLength,
+                                 nsTArray<uint8_t>* aMetadata,
+                                 uint32_t* aMetaOrigLen)
 {
     // This function may be called with arbitrary, unvalidated "font" data
     // from @font-face, so it needs to be careful to bounds-check, etc.,
@@ -440,8 +385,8 @@ gfxUserFontSet::CopyWOFFMetadata(const PRUint8* aFontData,
         return;
     }
     const WOFFHeader* woff = reinterpret_cast<const WOFFHeader*>(aFontData);
-    PRUint32 metaOffset = woff->metaOffset;
-    PRUint32 metaCompLen = woff->metaCompLen;
+    uint32_t metaOffset = woff->metaOffset;
+    uint32_t metaCompLen = woff->metaCompLen;
     if (!metaOffset || !metaCompLen || !woff->metaOrigLen) {
         return;
     }
@@ -459,18 +404,19 @@ gfxUserFontSet::CopyWOFFMetadata(const PRUint8* aFontData,
 // Ownership of aFontData passes in here, and the font set must
 // ensure that it is eventually deleted via NS_Free().
 bool
-gfxUserFontSet::OnLoadComplete(gfxProxyFontEntry *aProxy,
-                               const PRUint8 *aFontData, PRUint32 aLength,
+gfxUserFontSet::OnLoadComplete(gfxMixedFontFamily *aFamily,
+                               gfxProxyFontEntry *aProxy,
+                               const uint8_t *aFontData, uint32_t aLength,
                                nsresult aDownloadStatus)
 {
     // forget about the loader, as we no longer potentially need to cancel it
     // if the entry is obsoleted
-    aProxy->mLoader = nsnull;
+    aProxy->mLoader = nullptr;
 
     // download successful, make platform font using font data
     if (NS_SUCCEEDED(aDownloadStatus)) {
-        gfxFontEntry *fe = LoadFont(aProxy, aFontData, aLength);
-        aFontData = nsnull;
+        gfxFontEntry *fe = LoadFont(aFamily, aProxy, aFontData, aLength);
+        aFontData = nullptr;
 
         if (fe) {
             IncrementGeneration();
@@ -479,7 +425,8 @@ gfxUserFontSet::OnLoadComplete(gfxProxyFontEntry *aProxy,
 
     } else {
         // download failed
-        LogMessage(aProxy, "download failed", nsIScriptError::errorFlag,
+        LogMessage(aFamily, aProxy,
+                   "download failed", nsIScriptError::errorFlag,
                    aDownloadStatus);
     }
 
@@ -488,7 +435,7 @@ gfxUserFontSet::OnLoadComplete(gfxProxyFontEntry *aProxy,
     }
 
     // error occurred, load next src
-    (void)LoadNext(aProxy);
+    (void)LoadNext(aFamily, aProxy);
 
     // We ignore the status returned by LoadNext();
     // even if loading failed, we need to bump the font-set generation
@@ -500,9 +447,10 @@ gfxUserFontSet::OnLoadComplete(gfxProxyFontEntry *aProxy,
 
 
 gfxUserFontSet::LoadStatus
-gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
+gfxUserFontSet::LoadNext(gfxMixedFontFamily *aFamily,
+                         gfxProxyFontEntry *aProxyEntry)
 {
-    PRUint32 numSrc = aProxyEntry->mSrcList.Length();
+    uint32_t numSrc = aProxyEntry->mSrcList.Length();
 
     NS_ASSERTION(aProxyEntry->mSrcIndex < numSrc,
                  "already at the end of the src list for user font");
@@ -532,18 +480,18 @@ gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
                 LOG(("userfonts (%p) [src %d] loaded local: (%s) for (%s) gen: %8.8x\n",
                      this, aProxyEntry->mSrcIndex,
                      NS_ConvertUTF16toUTF8(currSrc.mLocalName).get(),
-                     NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get(),
-                     PRUint32(mGeneration)));
+                     NS_ConvertUTF16toUTF8(aFamily->Name()).get(),
+                     uint32_t(mGeneration)));
                 fe->mFeatureSettings.AppendElements(aProxyEntry->mFeatureSettings);
                 fe->mLanguageOverride = aProxyEntry->mLanguageOverride;
-                StoreUserFontData(fe, aProxyEntry, nsString(), nsnull, 0);
-                ReplaceFontEntry(aProxyEntry, fe);
+                StoreUserFontData(fe, aProxyEntry, nsString(), nullptr, 0);
+                ReplaceFontEntry(aFamily, aProxyEntry, fe);
                 return STATUS_LOADED;
             } else {
                 LOG(("userfonts (%p) [src %d] failed local: (%s) for (%s)\n",
                      this, aProxyEntry->mSrcIndex,
                      NS_ConvertUTF16toUTF8(currSrc.mLocalName).get(),
-                     NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
+                     NS_ConvertUTF16toUTF8(aFamily->Name()).get()));
             }
         }
 
@@ -552,48 +500,67 @@ gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
             if (gfxPlatform::GetPlatform()->IsFontFormatSupported(currSrc.mURI,
                     currSrc.mFormatFlags)) {
 
-                nsresult rv;
-                bool loadDoesntSpin = false;
-                rv = NS_URIChainHasFlags(currSrc.mURI,
-                       nsIProtocolHandler::URI_SYNC_LOAD_IS_OK,
-                       &loadDoesntSpin);
+                nsIPrincipal *principal = nullptr;
+                nsresult rv = CheckFontLoad(&currSrc, &principal);
 
-                if (NS_SUCCEEDED(rv) && loadDoesntSpin) {
-                    PRUint8 *buffer = nsnull;
-                    PRUint32 bufferLength = 0;
-
-                    // sync load font immediately
-                    rv = SyncLoadFontData(aProxyEntry, &currSrc, buffer,
-                                          bufferLength);
-
-                    if (NS_SUCCEEDED(rv) &&
-                        LoadFont(aProxyEntry, buffer, bufferLength)) {
+                if (NS_SUCCEEDED(rv) && principal != nullptr) {
+                    // see if we have an existing entry for this source
+                    gfxFontEntry *fe =
+                        UserFontCache::GetFont(currSrc.mURI, principal,
+                                               aProxyEntry);
+                    if (fe) {
+                        ReplaceFontEntry(aFamily, aProxyEntry, fe);
                         return STATUS_LOADED;
-                    } else {
-                        LogMessage(aProxyEntry, "font load failed",
-                                   nsIScriptError::errorFlag, rv);
                     }
 
-                } else {
-                    // otherwise load font async
-                    rv = StartLoad(aProxyEntry, &currSrc);
-                    bool loadOK = NS_SUCCEEDED(rv);
+                    // record the principal returned by CheckFontLoad,
+                    // for use when creating a channel
+                    // and when caching the loaded entry
+                    aProxyEntry->mPrincipal = principal;
 
-                    if (loadOK) {
-#ifdef PR_LOGGING
-                        if (LOG_ENABLED()) {
-                            nsCAutoString fontURI;
-                            currSrc.mURI->GetSpec(fontURI);
-                            LOG(("userfonts (%p) [src %d] loading uri: (%s) for (%s)\n",
-                                 this, aProxyEntry->mSrcIndex, fontURI.get(),
-                                 NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
+                    bool loadDoesntSpin = false;
+                    rv = NS_URIChainHasFlags(currSrc.mURI,
+                           nsIProtocolHandler::URI_SYNC_LOAD_IS_OK,
+                           &loadDoesntSpin);
+                    if (NS_SUCCEEDED(rv) && loadDoesntSpin) {
+                        uint8_t *buffer = nullptr;
+                        uint32_t bufferLength = 0;
+
+                        // sync load font immediately
+                        rv = SyncLoadFontData(aProxyEntry, &currSrc,
+                                              buffer, bufferLength);
+                        if (NS_SUCCEEDED(rv) &&
+                            (fe = LoadFont(aFamily, aProxyEntry,
+                                           buffer, bufferLength))) {
+                            return STATUS_LOADED;
+                        } else {
+                            LogMessage(aFamily, aProxyEntry,
+                                       "font load failed",
+                                       nsIScriptError::errorFlag, rv);
                         }
-#endif
-                        return STATUS_LOADING;
                     } else {
-                        LogMessage(aProxyEntry, "download failed",
-                                   nsIScriptError::errorFlag, rv);
+                        // otherwise load font async
+                        rv = StartLoad(aFamily, aProxyEntry, &currSrc);
+                        if (NS_SUCCEEDED(rv)) {
+#ifdef PR_LOGGING
+                            if (LOG_ENABLED()) {
+                                nsAutoCString fontURI;
+                                currSrc.mURI->GetSpec(fontURI);
+                                LOG(("userfonts (%p) [src %d] loading uri: (%s) for (%s)\n",
+                                     this, aProxyEntry->mSrcIndex, fontURI.get(),
+                                     NS_ConvertUTF16toUTF8(aFamily->Name()).get()));
+                            }
+#endif
+                            return STATUS_LOADING;
+                        } else {
+                            LogMessage(aFamily, aProxyEntry,
+                                       "download failed",
+                                       nsIScriptError::errorFlag, rv);
+                        }
                     }
+                } else {
+                    LogMessage(aFamily, aProxyEntry, "download not allowed",
+                               nsIScriptError::errorFlag, rv);
                 }
             } else {
                 // We don't log a warning to the web console yet,
@@ -606,13 +573,13 @@ gfxUserFontSet::LoadNext(gfxProxyFontEntry *aProxyEntry)
     }
 
     if (aProxyEntry->mUnsupportedFormat) {
-        LogMessage(aProxyEntry, "no supported format found",
+        LogMessage(aFamily, aProxyEntry, "no supported format found",
                    nsIScriptError::warningFlag);
     }
 
     // all src's failed; mark this entry as unusable (so fallback will occur)
     LOG(("userfonts (%p) failed all src for (%s)\n",
-        this, NS_ConvertUTF16toUTF8(aProxyEntry->mFamily->Name()).get()));
+        this, NS_ConvertUTF16toUTF8(aFamily->Name()).get()));
     aProxyEntry->mLoadingState = gfxProxyFontEntry::LOADING_FAILED;
 
     return STATUS_END_OF_LIST;
@@ -622,42 +589,23 @@ void
 gfxUserFontSet::IncrementGeneration()
 {
     // add one, increment again if zero
-    LL_ADD(sFontSetGeneration, sFontSetGeneration, 1);
-    if (LL_IS_ZERO(sFontSetGeneration))
-        LL_ADD(sFontSetGeneration, sFontSetGeneration, 1);
+    ++sFontSetGeneration;
+    if (sFontSetGeneration == 0)
+       ++sFontSetGeneration;
     mGeneration = sFontSetGeneration;
 }
 
 
 gfxFontEntry*
-gfxUserFontSet::LoadFont(gfxProxyFontEntry *aProxy,
-                         const PRUint8 *aFontData, PRUint32 &aLength)
+gfxUserFontSet::LoadFont(gfxMixedFontFamily *aFamily,
+                         gfxProxyFontEntry *aProxy,
+                         const uint8_t *aFontData, uint32_t &aLength)
 {
-    // if the proxy doesn't belong to a family, we just bail as it won't be
-    // accessible/usable anyhow (maybe the font set got modified right as
-    // the load was completing?)
-    if (!aProxy->Family()) {
-        NS_Free(const_cast<PRUint8*>(aFontData));
-        return nsnull;
-    }
-
-    gfxFontEntry *fe = nsnull;
+    gfxFontEntry *fe = nullptr;
 
     gfxUserFontType fontType =
         gfxFontUtils::DetermineFontDataType(aFontData, aLength);
 
-    // Save a copy of the metadata block (if present) for nsIDOMFontFace
-    // to use if required. Ownership of the metadata block will be passed
-    // to the gfxUserFontData record below.
-    // NOTE: after the non-OTS codepath using PrepareOpenTypeData is
-    // removed, we should defer this until after we've created the new
-    // fontEntry.
-    nsTArray<PRUint8> metadata;
-    PRUint32 metaOrigLen = 0;
-    if (fontType == GFX_USERFONT_WOFF) {
-        CopyWOFFMetadata(aFontData, aLength, &metadata, &metaOrigLen);
-    }
-
     // Unwrap/decompress/sanitize or otherwise munge the downloaded data
     // to make a usable sfnt structure.
 
@@ -666,67 +614,42 @@ gfxUserFontSet::LoadFont(gfxProxyFontEntry *aProxy,
     // it can be reported via the nsIDOMFontFace API.
     nsAutoString originalFullName;
 
-    if (gfxPlatform::GetPlatform()->SanitizeDownloadedFonts()) {
-       // Call the OTS sanitizer; this will also decode WOFF to sfnt
-        // if necessary. The original data in aFontData is left unchanged.
-        PRUint32 saneLen;
-        const PRUint8* saneData =
-            SanitizeOpenTypeData(aProxy, aFontData, aLength, saneLen,
-                                 fontType == GFX_USERFONT_WOFF);
-        if (!saneData) {
-            LogMessage(aProxy, "rejected by sanitizer");
-        }
-        if (saneData) {
-            // The sanitizer ensures that we have a valid sfnt and a usable
-            // name table, so this should never fail unless we're out of
-            // memory, and GetFullNameFromSFNT is not directly exposed to
-            // arbitrary/malicious data from the web.
-            gfxFontUtils::GetFullNameFromSFNT(saneData, saneLen,
-                                              originalFullName);
-            // Here ownership of saneData is passed to the platform,
-            // which will delete it when no longer required
-            fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
-                                                              saneData,
-                                                              saneLen);
-            if (!fe) {
-                LogMessage(aProxy, "not usable by platform");
-            }
-        }
-    } else {
-        // FIXME: this code can be removed once we remove the pref to
-        // disable the sanitizer; the PrepareOpenTypeData and
-        // ValidateSFNTHeaders functions will then be obsolete.
-        aFontData = PrepareOpenTypeData(aFontData, &aLength);
-
-        if (aFontData) {
-            if (gfxFontUtils::ValidateSFNTHeaders(aFontData, aLength)) {
-                // ValidateSFNTHeaders has checked that we have a valid
-                // sfnt structure and a usable 'name' table
-                gfxFontUtils::GetFullNameFromSFNT(aFontData, aLength,
-                                                  originalFullName);
-                // Here ownership of aFontData is passed to the platform,
-                // which will delete it when no longer required
-                fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
-                                                                  aFontData,
-                                                                  aLength);
-                if (!fe) {
-                    LogMessage(aProxy, "not usable by platform");
-                }
-                aFontData = nsnull; // we must NOT free this!
-            } else {
-                // the data was unusable, so just discard it
-                // (error will be reported below, if logging is enabled)
-                LogMessage(aProxy, "SFNT header or tables invalid");
-            }
-        }
+    // Call the OTS sanitizer; this will also decode WOFF to sfnt
+    // if necessary. The original data in aFontData is left unchanged.
+    uint32_t saneLen;
+    const uint8_t* saneData =
+        SanitizeOpenTypeData(aFamily, aProxy, aFontData, aLength, saneLen,
+                             fontType == GFX_USERFONT_WOFF);
+    if (!saneData) {
+        LogMessage(aFamily, aProxy, "rejected by sanitizer");
     }
-
-    if (aFontData) {
-        NS_Free((void*)aFontData);
-        aFontData = nsnull;
+    if (saneData) {
+        // The sanitizer ensures that we have a valid sfnt and a usable
+        // name table, so this should never fail unless we're out of
+        // memory, and GetFullNameFromSFNT is not directly exposed to
+        // arbitrary/malicious data from the web.
+        gfxFontUtils::GetFullNameFromSFNT(saneData, saneLen,
+                                          originalFullName);
+        // Here ownership of saneData is passed to the platform,
+        // which will delete it when no longer required
+        fe = gfxPlatform::GetPlatform()->MakePlatformFont(aProxy,
+                                                          saneData,
+                                                          saneLen);
+        if (!fe) {
+            LogMessage(aFamily, aProxy, "not usable by platform");
+        }
     }
 
     if (fe) {
+        // Save a copy of the metadata block (if present) for nsIDOMFontFace
+        // to use if required. Ownership of the metadata block will be passed
+        // to the gfxUserFontData record below.
+        nsTArray<uint8_t> metadata;
+        uint32_t metaOrigLen = 0;
+        if (fontType == GFX_USERFONT_WOFF) {
+            CopyWOFFMetadata(aFontData, aLength, &metadata, &metaOrigLen);
+        }
+
         // copy OpenType feature/language settings from the proxy to the
         // newly-created font entry
         fe->mFeatureSettings.AppendElements(aProxy->mFeatureSettings);
@@ -734,35 +657,38 @@ gfxUserFontSet::LoadFont(gfxProxyFontEntry *aProxy,
         StoreUserFontData(fe, aProxy, originalFullName,
                           &metadata, metaOrigLen);
 #ifdef PR_LOGGING
-        // must do this before ReplaceFontEntry() because that will
-        // clear the proxy's mFamily pointer!
         if (LOG_ENABLED()) {
-            nsCAutoString fontURI;
+            nsAutoCString fontURI;
             aProxy->mSrcList[aProxy->mSrcIndex].mURI->GetSpec(fontURI);
             LOG(("userfonts (%p) [src %d] loaded uri: (%s) for (%s) gen: %8.8x\n",
                  this, aProxy->mSrcIndex, fontURI.get(),
-                 NS_ConvertUTF16toUTF8(aProxy->mFamily->Name()).get(),
-                 PRUint32(mGeneration)));
+                 NS_ConvertUTF16toUTF8(aFamily->Name()).get(),
+                 uint32_t(mGeneration)));
         }
 #endif
-        ReplaceFontEntry(aProxy, fe);
+        ReplaceFontEntry(aFamily, aProxy, fe);
+        UserFontCache::CacheFont(fe);
     } else {
 #ifdef PR_LOGGING
         if (LOG_ENABLED()) {
-            nsCAutoString fontURI;
+            nsAutoCString fontURI;
             aProxy->mSrcList[aProxy->mSrcIndex].mURI->GetSpec(fontURI);
             LOG(("userfonts (%p) [src %d] failed uri: (%s) for (%s)"
                  " error making platform font\n",
                  this, aProxy->mSrcIndex, fontURI.get(),
-                 NS_ConvertUTF16toUTF8(aProxy->mFamily->Name()).get()));
+                 NS_ConvertUTF16toUTF8(aFamily->Name()).get()));
         }
 #endif
     }
 
+    // The downloaded data can now be discarded; the font entry is using the
+    // sanitized copy
+    NS_Free((void*)aFontData);
+
     return fe;
 }
 
-gfxMixedFontFamily*
+gfxFontFamily*
 gfxUserFontSet::GetFamily(const nsAString& aFamilyName) const
 {
     nsAutoString key(aFamilyName);
@@ -771,3 +697,126 @@ gfxUserFontSet::GetFamily(const nsAString& aFamilyName) const
     return mFontFamilies.GetWeak(key);
 }
 
+struct FindFamilyCallbackData {
+    gfxFontEntry  *mFontEntry;
+    gfxFontFamily *mFamily;
+};
+
+static PLDHashOperator
+FindFamilyCallback(const nsAString&    aName,
+                   gfxMixedFontFamily* aFamily,
+                   void*               aUserArg)
+{
+    FindFamilyCallbackData *d = static_cast<FindFamilyCallbackData*>(aUserArg);
+    if (aFamily->ContainsFace(d->mFontEntry)) {
+        d->mFamily = aFamily;
+        return PL_DHASH_STOP;
+    }
+
+    return PL_DHASH_NEXT;
+}
+
+gfxFontFamily*
+gfxUserFontSet::FindFamilyFor(gfxFontEntry* aFontEntry) const
+{
+    FindFamilyCallbackData d = { aFontEntry, nullptr };
+    mFontFamilies.EnumerateRead(FindFamilyCallback, &d);
+    return d.mFamily;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+// gfxUserFontSet::UserFontCache - re-use platform font entries for user fonts
+// across pages/fontsets rather than instantiating new platform fonts.
+//
+// Entries are added to this cache when a platform font is instantiated from
+// downloaded data, and removed when the platform font entry is destroyed.
+// We don't need to use a timed expiration scheme here because the gfxFontEntry
+// for a downloaded font will be kept alive by its corresponding gfxFont
+// instance(s) until they are deleted, and *that* happens using an expiration
+// tracker (gfxFontCache). The result is that the downloaded font instances
+// recorded here will persist between pages and can get reused (provided the
+// source URI and principal match, of course).
+///////////////////////////////////////////////////////////////////////////////
+
+nsTHashtable<gfxUserFontSet::UserFontCache::Entry>*
+    gfxUserFontSet::UserFontCache::sUserFonts = nullptr;
+
+bool
+gfxUserFontSet::UserFontCache::Entry::KeyEquals(const KeyTypePointer aKey) const
+{
+    bool equal;
+    if (NS_FAILED(mURI->Equals(aKey->mURI, &equal)) || !equal) {
+        return false;
+    }
+
+    if (NS_FAILED(mPrincipal->Equals(aKey->mPrincipal, &equal)) || !equal) {
+        return false;
+    }
+
+    const gfxFontEntry *fe = aKey->mFontEntry;
+    if (mFontEntry->mItalic           != fe->mItalic          ||
+        mFontEntry->mWeight           != fe->mWeight          ||
+        mFontEntry->mStretch          != fe->mStretch         ||
+        mFontEntry->mFeatureSettings  != fe->mFeatureSettings ||
+        mFontEntry->mLanguageOverride != fe->mLanguageOverride ||
+        mFontEntry->mFamilyName       != fe->mFamilyName) {
+        return false;
+    }
+
+    return true;
+}
+
+void
+gfxUserFontSet::UserFontCache::CacheFont(gfxFontEntry *aFontEntry)
+{
+    NS_ASSERTION(aFontEntry->mFamilyName.Length() != 0,
+                 "caching a font associated with no family yet");
+    if (!sUserFonts) {
+        sUserFonts = new nsTHashtable<Entry>;
+        sUserFonts->Init();
+    }
+
+    gfxUserFontData *data = aFontEntry->mUserFontData;
+    sUserFonts->PutEntry(Key(data->mURI, data->mPrincipal, aFontEntry));
+}
+
+void
+gfxUserFontSet::UserFontCache::ForgetFont(gfxFontEntry *aFontEntry)
+{
+    if (!sUserFonts) {
+        // if we've already deleted the cache (i.e. during shutdown),
+        // just ignore this
+        return;
+    }
+
+    gfxUserFontData *data = aFontEntry->mUserFontData;
+    if (data) {
+        sUserFonts->RemoveEntry(Key(data->mURI, data->mPrincipal, aFontEntry));
+    }
+}
+
+gfxFontEntry*
+gfxUserFontSet::UserFontCache::GetFont(nsIURI            *aSrcURI,
+                                       nsIPrincipal      *aPrincipal,
+                                       gfxProxyFontEntry *aProxy)
+{
+    if (!sUserFonts) {
+        return nullptr;
+    }
+
+    Entry* entry = sUserFonts->GetEntry(Key(aSrcURI, aPrincipal, aProxy));
+    if (entry) {
+        return entry->GetFontEntry();
+    }
+
+    return nullptr;
+}
+
+void
+gfxUserFontSet::UserFontCache::Shutdown()
+{
+    if (sUserFonts) {
+        delete sUserFonts;
+        sUserFonts = nullptr;
+    }
+}
