@@ -1,48 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
  *
- * ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is mozilla.org code.
- *
- * The Initial Developer of the Original Code is
- * Netscape Communications Corporation.
- * Portions created by the Initial Developer are Copyright (C) 2001
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Stuart Parmenter <stuart@mozilla.com>
- *   Andrew Smith
- *   Federico Mena-Quintero <federico@novell.com>
- *   Bobby Holley <bobbyholley@gmail.com>
- *   Glenn Randers-Pehrson <glennrp@gmail.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "nsPNGDecoder.h"
 #include "ImageLogging.h"
+#include "nsPNGDecoder.h"
 
 #include "nsMemory.h"
 #include "nsRect.h"
@@ -50,7 +13,6 @@
 #include "nsIInputStream.h"
 
 #include "RasterImage.h"
-#include "imgIContainerObserver.h"
 
 #include "gfxColor.h"
 #include "nsColor.h"
@@ -59,14 +21,29 @@
 #include "png.h"
 
 #include "gfxPlatform.h"
+#include <algorithm>
 
 namespace mozilla {
 namespace image {
 
 #ifdef PR_LOGGING
-static PRLogModuleInfo *gPNGLog = PR_NewLogModule("PNGDecoder");
-static PRLogModuleInfo *gPNGDecoderAccountingLog =
-                        PR_NewLogModule("PNGDecoderAccounting");
+static PRLogModuleInfo *
+GetPNGLog()
+{
+  static PRLogModuleInfo *sPNGLog;
+  if (!sPNGLog)
+    sPNGLog = PR_NewLogModule("PNGDecoder");
+  return sPNGLog;
+}
+
+static PRLogModuleInfo *
+GetPNGDecoderAccountingLog()
+{
+  static PRLogModuleInfo *sPNGDecoderAccountingLog;
+  if (!sPNGDecoderAccountingLog)
+    sPNGDecoderAccountingLog = PR_NewLogModule("PNGDecoderAccounting");
+  return sPNGDecoderAccountingLog;
+}
 #endif
 
 /* limit image dimensions (bug #251381) */
@@ -77,18 +54,67 @@ static PRLogModuleInfo *gPNGDecoderAccountingLog =
 #define HEIGHT_OFFSET (WIDTH_OFFSET + 4)
 #define BYTES_NEEDED_FOR_DIMENSIONS (HEIGHT_OFFSET + 4)
 
+nsPNGDecoder::AnimFrameInfo::AnimFrameInfo()
+ : mDispose(RasterImage::kDisposeKeep)
+ , mBlend(RasterImage::kBlendOver)
+ , mTimeout(0)
+{}
+
+#ifdef PNG_APNG_SUPPORTED
+nsPNGDecoder::AnimFrameInfo::AnimFrameInfo(png_structp aPNG, png_infop aInfo)
+ : mDispose(RasterImage::kDisposeKeep)
+ , mBlend(RasterImage::kBlendOver)
+ , mTimeout(0)
+{
+  png_uint_16 delay_num, delay_den;
+  /* delay, in seconds is delay_num/delay_den */
+  png_byte dispose_op;
+  png_byte blend_op;
+  delay_num = png_get_next_frame_delay_num(aPNG, aInfo);
+  delay_den = png_get_next_frame_delay_den(aPNG, aInfo);
+  dispose_op = png_get_next_frame_dispose_op(aPNG, aInfo);
+  blend_op = png_get_next_frame_blend_op(aPNG, aInfo);
+
+  if (delay_num == 0) {
+    mTimeout = 0; // SetFrameTimeout() will set to a minimum
+  } else {
+    if (delay_den == 0)
+      delay_den = 100; // so says the APNG spec
+
+    // Need to cast delay_num to float to have a proper division and
+    // the result to int to avoid compiler warning
+    mTimeout = static_cast<int32_t>(static_cast<double>(delay_num) * 1000 / delay_den);
+  }
+
+  if (dispose_op == PNG_DISPOSE_OP_PREVIOUS) {
+    mDispose = RasterImage::kDisposeRestorePrevious;
+  } else if (dispose_op == PNG_DISPOSE_OP_BACKGROUND) {
+    mDispose = RasterImage::kDisposeClear;
+  } else {
+    mDispose = RasterImage::kDisposeKeep;
+  }
+
+  if (blend_op == PNG_BLEND_OP_SOURCE) {
+    mBlend = RasterImage::kBlendSource;
+  } else {
+    mBlend = RasterImage::kBlendOver;
+  }
+}
+#endif
+
 // First 8 bytes of a PNG file
-const PRUint8 
+const uint8_t
 nsPNGDecoder::pngSignatureBytes[] = { 137, 80, 78, 71, 13, 10, 26, 10 };
 
-nsPNGDecoder::nsPNGDecoder(RasterImage &aImage, imgIDecoderObserver* aObserver)
- : Decoder(aImage, aObserver),
-   mPNG(nsnull), mInfo(nsnull),
-   mCMSLine(nsnull), interlacebuf(nsnull),
-   mInProfile(nsnull), mTransform(nsnull),
-   mHeaderBuf(nsnull), mHeaderBytesRead(0),
+nsPNGDecoder::nsPNGDecoder(RasterImage &aImage)
+ : Decoder(aImage),
+   mPNG(nullptr), mInfo(nullptr),
+   mCMSLine(nullptr), interlacebuf(nullptr),
+   mInProfile(nullptr), mTransform(nullptr),
+   mHeaderBytesRead(0), mCMSMode(0),
    mChannels(0), mFrameIsHidden(false),
-   mCMSMode(0), mDisablePremultipliedAlpha(false)
+   mDisablePremultipliedAlpha(false),
+   mNumFrames(0)
 {
 }
 
@@ -107,91 +133,45 @@ nsPNGDecoder::~nsPNGDecoder()
     if (mTransform)
       qcms_transform_release(mTransform);
   }
-  if (mHeaderBuf)
-    nsMemory::Free(mHeaderBuf);
 }
 
 // CreateFrame() is used for both simple and animated images
 void nsPNGDecoder::CreateFrame(png_uint_32 x_offset, png_uint_32 y_offset,
-                               PRInt32 width, PRInt32 height,
+                               int32_t width, int32_t height,
                                gfxASurface::gfxImageFormat format)
 {
-  PRUint32 imageDataLength;
-  nsresult rv = mImage.EnsureFrame(GetFrameCount(), x_offset, y_offset,
-                                   width, height, format,
-                                   &mImageData, &imageDataLength);
-  if (NS_FAILED(rv))
-    longjmp(png_jmpbuf(mPNG), 5); // NS_ERROR_OUT_OF_MEMORY
+  // Our first full frame is automatically created by the image decoding
+  // infrastructure. Just use it as long as it matches up.
+  MOZ_ASSERT(HasSize());
+  if (mNumFrames != 0 ||
+      !GetCurrentFrame()->GetRect().IsEqualEdges(nsIntRect(x_offset, y_offset, width, height))) {
+    NeedNewFrame(mNumFrames, x_offset, y_offset, width, height, format);
+  } else if (mNumFrames == 0) {
+    // Our preallocated frame matches up, with the possible exception of alpha.
+    if (format == gfxASurface::ImageFormatRGB24) {
+      GetCurrentFrame()->SetHasNoAlpha();
+    }
+  }
 
   mFrameRect.x = x_offset;
   mFrameRect.y = y_offset;
   mFrameRect.width = width;
   mFrameRect.height = height;
 
-#ifdef PNG_APNG_SUPPORTED
-  if (png_get_valid(mPNG, mInfo, PNG_INFO_acTL))
-    SetAnimFrameInfo();
-#endif
-
-  // Tell the superclass we're starting a frame
-  PostFrameStart();
-
-  PR_LOG(gPNGDecoderAccountingLog, PR_LOG_DEBUG,
+  PR_LOG(GetPNGDecoderAccountingLog(), PR_LOG_DEBUG,
          ("PNGDecoderAccounting: nsPNGDecoder::CreateFrame -- created "
           "image frame with %dx%d pixels in container %p",
           width, height,
           &mImage));
 
   mFrameHasNoAlpha = true;
-}
 
 #ifdef PNG_APNG_SUPPORTED
-// set timeout and frame disposal method for the current frame
-void nsPNGDecoder::SetAnimFrameInfo()
-{
-  png_uint_16 delay_num, delay_den;
-  /* delay, in seconds is delay_num/delay_den */
-  png_byte dispose_op;
-  png_byte blend_op;
-  PRInt32 timeout; /* in milliseconds */
-
-  delay_num = png_get_next_frame_delay_num(mPNG, mInfo);
-  delay_den = png_get_next_frame_delay_den(mPNG, mInfo);
-  dispose_op = png_get_next_frame_dispose_op(mPNG, mInfo);
-  blend_op = png_get_next_frame_blend_op(mPNG, mInfo);
-
-  if (delay_num == 0) {
-    timeout = 0; // SetFrameTimeout() will set to a minimum
-  } else {
-    if (delay_den == 0)
-      delay_den = 100; // so says the APNG spec
-
-    // Need to cast delay_num to float to have a proper division and
-    // the result to int to avoid compiler warning
-    timeout = static_cast<PRInt32>
-              (static_cast<PRFloat64>(delay_num) * 1000 / delay_den);
+  if (png_get_valid(mPNG, mInfo, PNG_INFO_acTL)) {
+    mAnimInfo = AnimFrameInfo(mPNG, mInfo);
   }
-
-  PRUint32 numFrames = mImage.GetNumFrames();
-
-  mImage.SetFrameTimeout(numFrames - 1, timeout);
-
-  if (dispose_op == PNG_DISPOSE_OP_PREVIOUS)
-      mImage.SetFrameDisposalMethod(numFrames - 1,
-                                    RasterImage::kDisposeRestorePrevious);
-  else if (dispose_op == PNG_DISPOSE_OP_BACKGROUND)
-      mImage.SetFrameDisposalMethod(numFrames - 1,
-                                    RasterImage::kDisposeClear);
-  else
-      mImage.SetFrameDisposalMethod(numFrames - 1,
-                                    RasterImage::kDisposeKeep);
-
-  if (blend_op == PNG_BLEND_OP_SOURCE)
-      mImage.SetFrameBlendMethod(numFrames - 1, RasterImage::kBlendSource);
-  /*else // 'over' is the default
-      mImage.SetFrameBlendMethod(numFrames - 1, RasterImage::kBlendOver); */
-}
 #endif
+}
 
 // set timeout and frame disposal method for the current frame
 void nsPNGDecoder::EndImageFrame()
@@ -199,26 +179,34 @@ void nsPNGDecoder::EndImageFrame()
   if (mFrameIsHidden)
     return;
 
-  PRUint32 numFrames = 1;
+  mNumFrames++;
+
+  RasterImage::FrameAlpha alpha;
+  if (mFrameHasNoAlpha)
+    alpha = RasterImage::kFrameOpaque;
+  else
+    alpha = RasterImage::kFrameHasAlpha;
+
 #ifdef PNG_APNG_SUPPORTED
-  numFrames = mImage.GetNumFrames();
+  uint32_t numFrames = GetFrameCount();
 
   // We can't use mPNG->num_frames_read as it may be one ahead.
   if (numFrames > 1) {
-    // Tell the image renderer that the frame is complete
-    if (mFrameHasNoAlpha)
-      mImage.SetFrameHasNoAlpha(numFrames - 1);
-
     PostInvalidation(mFrameRect);
   }
 #endif
 
-  PostFrameStop();
+  PostFrameStop(alpha, mAnimInfo.mDispose, mAnimInfo.mTimeout, mAnimInfo.mBlend);
 }
 
 void
 nsPNGDecoder::InitInternal()
 {
+  // For size decodes, we don't need to initialize the png decoder
+  if (IsSizeDecode()) {
+    return;
+  }
+
   mCMSMode = gfxPlatform::GetCMSMode();
   if ((mDecodeFlags & DECODER_NO_COLORSPACE_CONVERSION) != 0)
     mCMSMode = eCMSMode_Off;
@@ -243,12 +231,6 @@ nsPNGDecoder::InitInternal()
         122,  84,  88, 116, '\0'};  /* zTXt */
 #endif
 
-  // For size decodes, we only need a small buffer
-  if (IsSizeDecode()) {
-    mHeaderBuf = (PRUint8 *)moz_xmalloc(BYTES_NEEDED_FOR_DIMENSIONS);
-    return;
-  }
-
   /* For full decodes, do png init stuff */
 
   /* Initialize the container's source image header. */
@@ -275,7 +257,7 @@ nsPNGDecoder::InitInternal()
     png_set_keep_unknown_chunks(mPNG, 1, color_chunks, 2);
 
   png_set_keep_unknown_chunks(mPNG, 1, unused_chunks,
-                              (int)sizeof(unused_chunks)/5);   
+                              (int)sizeof(unused_chunks)/5);
 #endif
 
 #ifdef PNG_SET_CHUNK_MALLOC_LIMIT_SUPPORTED
@@ -283,6 +265,18 @@ nsPNGDecoder::InitInternal()
     png_set_chunk_malloc_max(mPNG, 4000000L);
 #endif
 
+#ifdef PNG_READ_CHECK_FOR_INVALID_INDEX_SUPPORTED
+#ifndef PR_LOGGING
+  /* Disallow palette-index checking, for speed; we would ignore the warning
+   * anyhow unless we have defined PR_LOGGING.  This feature was added at
+   * libpng version 1.5.10 and is disabled in the embedded libpng but enabled
+   * by default in the system libpng.  This call also disables it in the
+   * system libpng, for decoding speed.  Bug #745202.
+   */
+    png_set_check_for_invalid_index(mPNG, 0);
+#endif
+#endif
+
   /* use this as libpng "progressive pointer" (retrieve in callbacks) */
   png_set_progressive_read_fn(mPNG, static_cast<png_voidp>(this),
                               nsPNGDecoder::info_callback,
@@ -292,12 +286,8 @@ nsPNGDecoder::InitInternal()
 }
 
 void
-nsPNGDecoder::WriteInternal(const char *aBuffer, PRUint32 aCount)
+nsPNGDecoder::WriteInternal(const char *aBuffer, uint32_t aCount)
 {
-  // We use gotos, so we need to declare variables here
-  PRUint32 width = 0;
-  PRUint32 height = 0;
-
   NS_ABORT_IF_FALSE(!HasError(), "Shouldn't call WriteInternal after error!");
 
   // If we only want width/height, we don't need to go through libpng
@@ -307,25 +297,34 @@ nsPNGDecoder::WriteInternal(const char *aBuffer, PRUint32 aCount)
     if (mHeaderBytesRead == BYTES_NEEDED_FOR_DIMENSIONS)
       return;
 
-    // Read data into our header buffer
-    PRUint32 bytesToRead = NS_MIN(aCount, BYTES_NEEDED_FOR_DIMENSIONS -
-                                  mHeaderBytesRead);
-    memcpy(mHeaderBuf + mHeaderBytesRead, aBuffer, bytesToRead);
-    mHeaderBytesRead += bytesToRead;
+    // Scan the header for the width and height bytes
+    uint32_t pos = 0;
+    const uint8_t *bptr = (uint8_t *)aBuffer;
+
+    while (pos < aCount && mHeaderBytesRead < BYTES_NEEDED_FOR_DIMENSIONS) {
+      // Verify the signature bytes
+      if (mHeaderBytesRead < sizeof(pngSignatureBytes)) {
+        if (bptr[pos] != nsPNGDecoder::pngSignatureBytes[mHeaderBytesRead]) {
+          PostDataError();
+          return;
+        }
+      }
+
+      // Get width and height bytes into the buffer
+      if ((mHeaderBytesRead >= WIDTH_OFFSET) &&
+          (mHeaderBytesRead < BYTES_NEEDED_FOR_DIMENSIONS)) {
+        mSizeBytes[mHeaderBytesRead - WIDTH_OFFSET] = bptr[pos];
+      }
+      pos ++;
+      mHeaderBytesRead ++;
+    }
 
     // If we're done now, verify the data and set up the container
     if (mHeaderBytesRead == BYTES_NEEDED_FOR_DIMENSIONS) {
 
-      // Check that the signature bytes are right
-      if (memcmp(mHeaderBuf, nsPNGDecoder::pngSignatureBytes, 
-                 sizeof(pngSignatureBytes))) {
-        PostDataError();
-        return;
-      }
-
       // Grab the width and height, accounting for endianness (thanks libpng!)
-      width = png_get_uint_32(mHeaderBuf + WIDTH_OFFSET);
-      height = png_get_uint_32(mHeaderBuf + HEIGHT_OFFSET);
+      uint32_t width = png_get_uint_32(mSizeBytes);
+      uint32_t height = png_get_uint_32(mSizeBytes + 4);
 
       // Too big?
       if ((width > MOZ_PNG_MAX_DIMENSION) || (height > MOZ_PNG_MAX_DIMENSION)) {
@@ -381,9 +380,9 @@ PNGDoGammaCorrection(png_structp png_ptr, png_infop info_ptr)
 // Adapted from http://www.littlecms.com/pngchrm.c example code
 static qcms_profile *
 PNGGetColorProfile(png_structp png_ptr, png_infop info_ptr,
-                   int color_type, qcms_data_type *inType, PRUint32 *intent)
+                   int color_type, qcms_data_type *inType, uint32_t *intent)
 {
-  qcms_profile *profile = nsnull;
+  qcms_profile *profile = nullptr;
   *intent = QCMS_INTENT_PERCEPTUAL; // Our default
 
   // First try to see if iCCP chunk is present
@@ -408,7 +407,7 @@ PNGGetColorProfile(png_structp png_ptr, png_infop info_ptr,
 #endif
                                        profileLen);
     if (profile) {
-      PRUint32 profileSpace = qcms_profile_get_color_space(profile);
+      uint32_t profileSpace = qcms_profile_get_color_space(profile);
 
       bool mismatch = false;
       if (color_type & PNG_COLOR_MASK_COLOR) {
@@ -423,7 +422,7 @@ PNGGetColorProfile(png_structp png_ptr, png_infop info_ptr,
 
       if (mismatch) {
         qcms_profile_release(profile);
-        profile = nsnull;
+        profile = nullptr;
       } else {
         *intent = qcms_profile_get_rendering_intent(profile);
       }
@@ -438,7 +437,7 @@ PNGGetColorProfile(png_structp png_ptr, png_infop info_ptr,
       int fileIntent;
       png_set_gray_to_rgb(png_ptr);
       png_get_sRGB(png_ptr, info_ptr, &fileIntent);
-      PRUint32 map[] = { QCMS_INTENT_PERCEPTUAL,
+      uint32_t map[] = { QCMS_INTENT_PERCEPTUAL,
                          QCMS_INTENT_RELATIVE_COLORIMETRIC,
                          QCMS_INTENT_SATURATION,
                          QCMS_INTENT_ABSOLUTE_COLORIMETRIC };
@@ -473,7 +472,7 @@ PNGGetColorProfile(png_structp png_ptr, png_infop info_ptr,
   }
 
   if (profile) {
-    PRUint32 profileSpace = qcms_profile_get_color_space(profile);
+    uint32_t profileSpace = qcms_profile_get_color_space(profile);
     if (profileSpace == icSigGrayData) {
       if (color_type & PNG_COLOR_MASK_ALPHA)
         *inType = QCMS_DATA_GRAYA_8;
@@ -516,8 +515,7 @@ nsPNGDecoder::info_callback(png_structp png_ptr, png_infop info_ptr)
   // Post our size to the superclass
   decoder->PostSize(width, height);
   if (decoder->HasError()) {
-    // Setting the size lead to an error; this can happen when for example
-    // a multipart channel sends an image of a different size.
+    // Setting the size led to an error.
     longjmp(png_jmpbuf(decoder->mPNG), 1);
   }
 
@@ -553,14 +551,14 @@ nsPNGDecoder::info_callback(png_structp png_ptr, png_infop info_ptr)
     png_set_scale_16(png_ptr);
 
   qcms_data_type inType;
-  PRUint32 intent = -1;
-  PRUint32 pIntent;
+  uint32_t intent = -1;
+  uint32_t pIntent;
   if (decoder->mCMSMode != eCMSMode_Off) {
     intent = gfxPlatform::GetRenderingIntent();
     decoder->mInProfile = PNGGetColorProfile(png_ptr, info_ptr,
                                              color_type, &inType, &pIntent);
     /* If we're not mandating an intent, use the one from the image. */
-    if (intent == PRUint32(-1))
+    if (intent == uint32_t(-1))
       intent = pIntent;
   }
   if (decoder->mInProfile && gfxPlatform::GetCMSOutputProfile()) {
@@ -606,7 +604,9 @@ nsPNGDecoder::info_callback(png_structp png_ptr, png_infop info_ptr)
   /* copy PNG info into imagelib structs (formerly png_set_dims()) */
   /*---------------------------------------------------------------*/
 
-  PRInt32 alpha_bits = 1;
+  // This code is currently unused, but it will be needed for bug 517713.
+#if 0
+  int32_t alpha_bits = 1;
 
   if (channels == 2 || channels == 4) {
     /* check if alpha is coming from a tRNS chunk and is binary */
@@ -624,6 +624,7 @@ nsPNGDecoder::info_callback(png_structp png_ptr, png_infop info_ptr)
       alpha_bits = 8;
     }
   }
+#endif
 
   if (channels == 1 || channels == 3)
     decoder->format = gfxASurface::ImageFormatRGB24;
@@ -645,30 +646,28 @@ nsPNGDecoder::info_callback(png_structp png_ptr, png_infop info_ptr)
 
   if (decoder->mTransform &&
       (channels <= 2 || interlace_type == PNG_INTERLACE_ADAM7)) {
-    PRUint32 bpp[] = { 0, 3, 4, 3, 4 };
+    uint32_t bpp[] = { 0, 3, 4, 3, 4 };
     decoder->mCMSLine =
-      (PRUint8 *)moz_malloc(bpp[channels] * width);
+      (uint8_t *)moz_malloc(bpp[channels] * width);
     if (!decoder->mCMSLine) {
       longjmp(png_jmpbuf(decoder->mPNG), 5); // NS_ERROR_OUT_OF_MEMORY
     }
   }
 
   if (interlace_type == PNG_INTERLACE_ADAM7) {
-    if (height < PR_INT32_MAX / (width * channels))
-      decoder->interlacebuf = (PRUint8 *)moz_malloc(channels * width * height);
+    if (height < INT32_MAX / (width * channels))
+      decoder->interlacebuf = (uint8_t *)moz_malloc(channels * width * height);
     if (!decoder->interlacebuf) {
       longjmp(png_jmpbuf(decoder->mPNG), 5); // NS_ERROR_OUT_OF_MEMORY
     }
   }
 
-  /* Reject any ancillary chunk after IDAT with a bad CRC (bug #397593).
-   * It would be better to show the default frame (if one has already been
-   * successfully decoded) before bailing, but it's simpler to just bail
-   * out with an error message.
-   */
-  png_set_crc_action(png_ptr, PNG_CRC_NO_CHANGE, PNG_CRC_ERROR_QUIT);
-
-  return;
+  if (decoder->NeedsNewFrame()) {
+    /* We know that we need a new frame, so pause input so the decoder
+     * infrastructure can give it to us.
+     */
+    png_process_data_pause(png_ptr, /* save = */ 1);
+  }
 }
 
 void
@@ -713,8 +712,8 @@ nsPNGDecoder::row_callback(png_structp png_ptr, png_bytep new_row,
     return;
 
   if (new_row) {
-    PRInt32 width = decoder->mFrameRect.width;
-    PRUint32 iwidth = decoder->mFrameRect.width;
+    int32_t width = decoder->mFrameRect.width;
+    uint32_t iwidth = decoder->mFrameRect.width;
 
     png_bytep line = new_row;
     if (decoder->interlacebuf) {
@@ -722,8 +721,8 @@ nsPNGDecoder::row_callback(png_structp png_ptr, png_bytep new_row,
       png_progressive_combine_row(png_ptr, line, new_row);
     }
 
-    PRUint32 bpr = width * sizeof(PRUint32);
-    PRUint32 *cptr32 = (PRUint32*)(decoder->mImageData + (row_num*bpr));
+    uint32_t bpr = width * sizeof(uint32_t);
+    uint32_t *cptr32 = (uint32_t*)(decoder->mImageData + (row_num*bpr));
     bool rowHasNoAlpha = true;
 
     if (decoder->mTransform) {
@@ -731,9 +730,9 @@ nsPNGDecoder::row_callback(png_structp png_ptr, png_bytep new_row,
         qcms_transform_data(decoder->mTransform, line, decoder->mCMSLine,
                             iwidth);
         /* copy alpha over */
-        PRUint32 channels = decoder->mChannels;
+        uint32_t channels = decoder->mChannels;
         if (channels == 2 || channels == 4) {
-          for (PRUint32 i = 0; i < iwidth; i++)
+          for (uint32_t i = 0; i < iwidth; i++)
             decoder->mCMSLine[4 * i + 3] = line[channels * i + channels - 1];
         }
         line = decoder->mCMSLine;
@@ -746,11 +745,11 @@ nsPNGDecoder::row_callback(png_structp png_ptr, png_bytep new_row,
       case gfxASurface::ImageFormatRGB24:
       {
         // counter for while() loops below
-        PRUint32 idx = iwidth;
+        uint32_t idx = iwidth;
 
         // copy as bytes until source pointer is 32-bit-aligned
         for (; (NS_PTR_TO_UINT32(line) & 0x3) && idx; --idx) {
-          *cptr32++ = GFX_PACKED_PIXEL(0xFF, line[0], line[1], line[2]);
+          *cptr32++ = gfxPackedPixel(0xFF, line[0], line[1], line[2]);
           line += 3;
         }
 
@@ -765,7 +764,7 @@ nsPNGDecoder::row_callback(png_structp png_ptr, png_bytep new_row,
         // copy remaining pixel(s)
         while (idx--) {
           // 32-bit read of final pixel will exceed buffer, so read bytes
-          *cptr32++ = GFX_PACKED_PIXEL(0xFF, line[0], line[1], line[2]);
+          *cptr32++ = gfxPackedPixel(0xFF, line[0], line[1], line[2]);
           line += 3;
         }
       }
@@ -773,15 +772,15 @@ nsPNGDecoder::row_callback(png_structp png_ptr, png_bytep new_row,
       case gfxASurface::ImageFormatARGB32:
       {
         if (!decoder->mDisablePremultipliedAlpha) {
-          for (PRUint32 x=width; x>0; --x) {
-            *cptr32++ = GFX_PACKED_PIXEL(line[3], line[0], line[1], line[2]);
+          for (uint32_t x=width; x>0; --x) {
+            *cptr32++ = gfxPackedPixel(line[3], line[0], line[1], line[2]);
             if (line[3] != 0xff)
               rowHasNoAlpha = false;
             line += 4;
           }
         } else {
-          for (PRUint32 x=width; x>0; --x) {
-            *cptr32++ = GFX_PACKED_PIXEL_NO_PREMULTIPLY(line[3], line[0], line[1], line[2]);
+          for (uint32_t x=width; x>0; --x) {
+            *cptr32++ = gfxPackedPixelNoPreMultiply(line[3], line[0], line[1], line[2]);
             if (line[3] != 0xff)
               rowHasNoAlpha = false;
             line += 4;
@@ -796,8 +795,7 @@ nsPNGDecoder::row_callback(png_structp png_ptr, png_bytep new_row,
     if (!rowHasNoAlpha)
       decoder->mFrameHasNoAlpha = false;
 
-    PRUint32 numFrames = decoder->mImage.GetNumFrames();
-    if (numFrames <= 1) {
+    if (decoder->mNumFrames <= 1) {
       // Only do incremental image display for the first frame
       // XXXbholley - this check should be handled in the superclass
       nsIntRect r(0, row_num, width, 1);
@@ -812,7 +810,7 @@ nsPNGDecoder::frame_info_callback(png_structp png_ptr, png_uint_32 frame_num)
 {
 #ifdef PNG_APNG_SUPPORTED
   png_uint_32 x_offset, y_offset;
-  PRInt32 width, height;
+  int32_t width, height;
 
   nsPNGDecoder *decoder =
                static_cast<nsPNGDecoder*>(png_get_progressive_ptr(png_ptr));
@@ -829,6 +827,13 @@ nsPNGDecoder::frame_info_callback(png_structp png_ptr, png_uint_32 frame_num)
   height = png_get_next_frame_height(png_ptr, decoder->mInfo);
 
   decoder->CreateFrame(x_offset, y_offset, width, height, decoder->format);
+
+  if (decoder->NeedsNewFrame()) {
+    /* We know that we need a new frame, so pause input so the decoder
+     * infrastructure can give it to us.
+     */
+    png_process_data_pause(png_ptr, /* save = */ 1);
+  }
 #endif
 }
 
@@ -853,23 +858,24 @@ nsPNGDecoder::end_callback(png_structp png_ptr, png_infop info_ptr)
   // We shouldn't get here if we've hit an error
   NS_ABORT_IF_FALSE(!decoder->HasError(), "Finishing up PNG but hit error!");
 
+  int32_t loop_count = 0;
 #ifdef PNG_APNG_SUPPORTED
   if (png_get_valid(png_ptr, info_ptr, PNG_INFO_acTL)) {
-    PRInt32 num_plays = png_get_num_plays(png_ptr, info_ptr);
-    decoder->mImage.SetLoopCount(num_plays - 1);
+    int32_t num_plays = png_get_num_plays(png_ptr, info_ptr);
+    loop_count = num_plays - 1;
   }
 #endif
 
   // Send final notifications
   decoder->EndImageFrame();
-  decoder->PostDecodeDone();
+  decoder->PostDecodeDone(loop_count);
 }
 
 
 void
 nsPNGDecoder::error_callback(png_structp png_ptr, png_const_charp error_msg)
 {
-  PR_LOG(gPNGLog, PR_LOG_ERROR, ("libpng error: %s\n", error_msg));
+  PR_LOG(GetPNGLog(), PR_LOG_ERROR, ("libpng error: %s\n", error_msg));
   longjmp(png_jmpbuf(png_ptr), 1);
 }
 
@@ -877,7 +883,7 @@ nsPNGDecoder::error_callback(png_structp png_ptr, png_const_charp error_msg)
 void
 nsPNGDecoder::warning_callback(png_structp png_ptr, png_const_charp warning_msg)
 {
-  PR_LOG(gPNGLog, PR_LOG_WARNING, ("libpng warning: %s\n", warning_msg));
+  PR_LOG(GetPNGLog(), PR_LOG_WARNING, ("libpng warning: %s\n", warning_msg));
 }
 
 Telemetry::ID
