@@ -1,6 +1,5 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sw=4 et tw=99 ft=cpp:
- *
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -20,6 +19,9 @@
 
 using namespace js;
 
+using mozilla::DoubleIsInt32;
+using mozilla::IsNaN;
+
 
 /*** OrderedHashTable ****************************************************************************/
 
@@ -229,18 +231,8 @@ class OrderedHashTable
      * The effect on live Ranges is the same as removing all entries; in
      * particular, those Ranges are still live and will see any entries added
      * after a successful clear().
-     *
-     * The DoNotCallDestructors specialization is for use during a GC when the
-     * OrderedHashTable contains pointers to GC things in other arenas. Since
-     * it is invalid to touch objects in other arenas during sweeping, we need
-     * to not trigger destructors on the pointers contained in the table in
-     * this case.
      */
-    enum CallDestructors {
-        DoNotCallDtors = false,
-        DoCallDtors = true
-    };
-    bool clear(CallDestructors callDestructors = DoCallDtors) {
+    bool clear() {
         if (dataLength != 0) {
             Data **oldHashTable = hashTable;
             Data *oldData = data;
@@ -254,9 +246,7 @@ class OrderedHashTable
             }
 
             alloc.free_(oldHashTable);
-            if (callDestructors)
-                destroyData(oldData, oldDataLength);
-            alloc.free_(oldData);
+            freeData(oldData, oldDataLength);
             for (Range *r = ranges; r; r = r->next)
                 r->onClear();
         }
@@ -500,6 +490,49 @@ class OrderedHashTable
 
     Range all() { return Range(*this); }
 
+    /*
+     * Change the value of the given key.
+     *
+     * This calls Ops::hash on both the current key and the new key.
+     * Ops::hash on the current key must return the same hash code as
+     * when the entry was added to the table.
+     */
+    void rekeyOneEntry(const Key &current, const Key &newKey, const T &element) {
+        if (current == newKey)
+            return;
+
+        Data *entry = lookup(current, prepareHash(current));
+        if (!entry)
+            return;
+
+        HashNumber oldHash = prepareHash(current) >> hashShift;
+        HashNumber newHash = prepareHash(newKey) >> hashShift;
+
+        entry->element = element;
+
+        // Remove this entry from its old hash chain. (If this crashes
+        // reading NULL, it would mean we did not find this entry on
+        // the hash chain where we expected it. That probably means the
+        // key's hash code changed since it was inserted, breaking the
+        // hash code invariant.)
+        Data **ep = &hashTable[oldHash];
+        while (*ep != entry)
+            ep = &(*ep)->chain;
+        *ep = entry->chain;
+
+        // Add it to the new hash chain. We could just insert it at the
+        // beginning of the chain. Instead, we do a bit of work to
+        // preserve the invariant that hash chains always go in reverse
+        // insertion order (descending memory order). No code currently
+        // depends on this invariant, so it's fine to kill it if
+        // needed.
+        ep = &hashTable[newHash];
+        while (*ep && *ep > entry)
+            ep = &(*ep)->chain;
+        entry->chain = *ep;
+        *ep = entry;
+    }
+
   private:
     /* Logarithm base 2 of the number of buckets in the hash table initially. */
     static uint32_t initialBucketsLog2() { return 1; }
@@ -702,8 +735,14 @@ class OrderedHashMap
     Entry *get(const Key &key)                      { return impl.get(key); }
     bool put(const Key &key, const Value &value)    { return impl.put(Entry(key, value)); }
     bool remove(const Key &key, bool *foundp)       { return impl.remove(key, foundp); }
-    bool clear()                                    { return impl.clear(Impl::DoCallDtors); }
-    bool clearWithoutCallingDestructors()           { return impl.clear(Impl::DoNotCallDtors); }
+    bool clear()                                    { return impl.clear(); }
+
+    void rekeyOneEntry(const Key &current, const Key &newKey) {
+        const Entry *e = get(current);
+        if (!e)
+            return;
+        return impl.rekeyOneEntry(current, newKey, Entry(newKey, e->value));
+    }
 };
 
 template <class T, class OrderedHashPolicy, class AllocPolicy>
@@ -730,8 +769,11 @@ class OrderedHashSet
     Range all()                                     { return impl.all(); }
     bool put(const T &value)                        { return impl.put(value); }
     bool remove(const T &value, bool *foundp)       { return impl.remove(value, foundp); }
-    bool clear()                                    { return impl.clear(Impl::DoCallDtors); }
-    bool clearWithoutCallingDestructors()           { return impl.clear(Impl::DoNotCallDtors); }
+    bool clear()                                    { return impl.clear(); }
+
+    void rekeyOneEntry(const T &current, const T &newKey) {
+        return impl.rekeyOneEntry(current, newKey, newKey);
+    }
 };
 
 }  // namespace js
@@ -743,7 +785,7 @@ bool
 HashableValue::setValue(JSContext *cx, const Value &v)
 {
     if (v.isString()) {
-        // Atomize so that hash() and equals() are fast and infallible.
+        // Atomize so that hash() and operator==() are fast and infallible.
         JSString *str = AtomizeString<CanGC>(cx, v.toString(), DoNotInternAtom);
         if (!str)
             return false;
@@ -751,10 +793,10 @@ HashableValue::setValue(JSContext *cx, const Value &v)
     } else if (v.isDouble()) {
         double d = v.toDouble();
         int32_t i;
-        if (MOZ_DOUBLE_IS_INT32(d, &i)) {
+        if (DoubleIsInt32(d, &i)) {
             // Normalize int32_t-valued doubles to int32_t for faster hashing and testing.
             value = Int32Value(i);
-        } else if (MOZ_DOUBLE_IS_NaN(d)) {
+        } else if (IsNaN(d)) {
             // NaNs with different bits must hash and test identically.
             value = DoubleValue(js_NaN);
         } else {
@@ -779,7 +821,7 @@ HashableValue::hash() const
 }
 
 bool
-HashableValue::equals(const HashableValue &other) const
+HashableValue::operator==(const HashableValue &other) const
 {
     // Two HashableValues are equal if they have equal bits.
     bool b = (value.asRawBits() == other.value.asRawBits());
@@ -808,10 +850,10 @@ class js::MapIteratorObject : public JSObject
 {
   public:
     enum { TargetSlot, KindSlot, RangeSlot, SlotCount };
-    static JSFunctionSpec methods[];
+    static const JSFunctionSpec methods[];
     static MapIteratorObject *create(JSContext *cx, HandleObject mapobj, ValueMap *data,
                                      MapObject::IteratorKind kind);
-    static void finalize(FreeOp *fop, RawObject obj);
+    static void finalize(FreeOp *fop, JSObject *obj);
 
   private:
     static inline bool is(const Value &v);
@@ -833,7 +875,7 @@ Class js::MapIteratorClass = {
     JSCLASS_IMPLEMENTS_BARRIERS |
     JSCLASS_HAS_RESERVED_SLOTS(MapIteratorObject::SlotCount),
     JS_PropertyStub,         /* addProperty */
-    JS_PropertyStub,         /* delProperty */
+    JS_DeletePropertyStub,   /* delProperty */
     JS_PropertyStub,         /* getProperty */
     JS_StrictPropertyStub,   /* setProperty */
     JS_EnumerateStub,
@@ -842,7 +884,7 @@ Class js::MapIteratorClass = {
     MapIteratorObject::finalize
 };
 
-JSFunctionSpec MapIteratorObject::methods[] = {
+const JSFunctionSpec MapIteratorObject::methods[] = {
     JS_FN("next", next, 0, 0),
     JS_FS_END
 };
@@ -903,7 +945,7 @@ MapIteratorObject::create(JSContext *cx, HandleObject mapobj, ValueMap *data,
 }
 
 void
-MapIteratorObject::finalize(FreeOp *fop, RawObject obj)
+MapIteratorObject::finalize(FreeOp *fop, JSObject *obj)
 {
     fop->delete_(obj->asMapIterator().range());
 }
@@ -973,7 +1015,7 @@ Class MapObject::class_ = {
     JSCLASS_HAS_PRIVATE | JSCLASS_IMPLEMENTS_BARRIERS |
     JSCLASS_HAS_CACHED_PROTO(JSProto_Map),
     JS_PropertyStub,         // addProperty
-    JS_PropertyStub,         // delProperty
+    JS_DeletePropertyStub,   // delProperty
     JS_PropertyStub,         // getProperty
     JS_StrictPropertyStub,   // setProperty
     JS_EnumerateStub,
@@ -987,12 +1029,12 @@ Class MapObject::class_ = {
     mark
 };
 
-JSPropertySpec MapObject::properties[] = {
+const JSPropertySpec MapObject::properties[] = {
     JS_PSG("size", size, 0),
     JS_PS_END
 };
 
-JSFunctionSpec MapObject::methods[] = {
+const JSFunctionSpec MapObject::methods[] = {
     JS_FN("get", get, 1, 0),
     JS_FN("has", has, 1, 0),
     JS_FN("set", set, 2, 0),
@@ -1007,7 +1049,7 @@ JSFunctionSpec MapObject::methods[] = {
 
 static JSObject *
 InitClass(JSContext *cx, Handle<GlobalObject*> global, Class *clasp, JSProtoKey key, Native construct,
-          JSPropertySpec *properties, JSFunctionSpec *methods)
+          const JSPropertySpec *properties, const JSFunctionSpec *methods)
 {
     Rooted<JSObject*> proto(cx, global->createBlankPrototype(cx, clasp));
     if (!proto)
@@ -1060,7 +1102,7 @@ MarkKey(Range &r, const HashableValue &key, JSTracer *trc)
 }
 
 void
-MapObject::mark(JSTracer *trc, RawObject obj)
+MapObject::mark(JSTracer *trc, JSObject *obj)
 {
     if (ValueMap *map = obj->asMap().getData()) {
         for (ValueMap::Range r = map->all(); !r.empty(); r.popFront()) {
@@ -1070,13 +1112,48 @@ MapObject::mark(JSTracer *trc, RawObject obj)
     }
 }
 
-void
-MapObject::finalize(FreeOp *fop, RawObject obj)
+#ifdef JSGC_GENERATIONAL
+template <typename TableType>
+class OrderedHashTableRef : public gc::BufferableRef
 {
-    if (ValueMap *map = obj->asMap().getData()) {
-        map->clearWithoutCallingDestructors();
-        fop->delete_(map);
+    TableType *table;
+    HashableValue key;
+
+  public:
+    OrderedHashTableRef(TableType *t, const HashableValue &k) : table(t), key(k) {}
+
+    bool match(void *location) {
+        if (!table->has(key))
+            return false;
+        for (typename TableType::Range r = table->all(); !r.empty(); r.popFront()) {
+            if ((void*)&r.front() == location)
+                return true;
+        }
+        return false;
     }
+
+    void mark(JSTracer *trc) {
+        HashableValue prior = key;
+        key = key.mark(trc);
+        table->rekeyOneEntry(prior, key);
+    }
+};
+#endif
+
+template <typename TableType>
+static void
+WriteBarrierPost(JSRuntime *rt, TableType *table, const HashableValue &key)
+{
+#ifdef JSGC_GENERATIONAL
+    rt->gcStoreBuffer.putGeneric(OrderedHashTableRef<TableType>(table, key));
+#endif
+}
+
+void
+MapObject::finalize(FreeOp *fop, JSObject *obj)
+{
+    if (ValueMap *map = obj->asMap().getData())
+        fop->delete_(map);
 }
 
 JSBool
@@ -1107,8 +1184,7 @@ MapObject::construct(JSContext *cx, unsigned argc, Value *vp)
             if (!JSObject::getElement(cx, pairobj, pairobj, 0, &key))
                 return false;
 
-            HashableValue hkey;
-            HashableValue::AutoRooter hkeyRoot(cx, &hkey);
+            AutoHashableValueRooter hkey(cx);
             if (!hkey.setValue(cx, key))
                 return false;
 
@@ -1121,6 +1197,7 @@ MapObject::construct(JSContext *cx, unsigned argc, Value *vp)
                 js_ReportOutOfMemory(cx);
                 return false;
             }
+            WriteBarrierPost(cx->runtime, map, hkey);
         }
         if (!iter.close())
             return false;
@@ -1137,8 +1214,7 @@ MapObject::is(const Value &v)
 }
 
 #define ARG0_KEY(cx, args, key)                                               \
-    HashableValue key;                                                        \
-    HashableValue::AutoRooter keyRoot(cx, &key);                              \
+    AutoHashableValueRooter key(cx);                                          \
     if (args.length() > 0 && !key.setValue(cx, args[0]))                      \
         return false
 
@@ -1215,11 +1291,12 @@ MapObject::set_impl(JSContext *cx, CallArgs args)
 
     ValueMap &map = extract(args);
     ARG0_KEY(cx, args, key);
-    RelocatableValue rval(args.length() > 1 ? args[1] : UndefinedValue());
+    RelocatableValue rval(args.get(1));
     if (!map.put(key, rval)) {
         js_ReportOutOfMemory(cx);
         return false;
     }
+    WriteBarrierPost(cx->runtime, &map, key);
     args.rval().setUndefined();
     return true;
 }
@@ -1346,9 +1423,9 @@ class js::SetIteratorObject : public JSObject
 {
   public:
     enum { TargetSlot, RangeSlot, SlotCount };
-    static JSFunctionSpec methods[];
+    static const JSFunctionSpec methods[];
     static SetIteratorObject *create(JSContext *cx, HandleObject setobj, ValueSet *data);
-    static void finalize(FreeOp *fop, RawObject obj);
+    static void finalize(FreeOp *fop, JSObject *obj);
 
   private:
     static inline bool is(const Value &v);
@@ -1369,7 +1446,7 @@ Class js::SetIteratorClass = {
     JSCLASS_IMPLEMENTS_BARRIERS |
     JSCLASS_HAS_RESERVED_SLOTS(SetIteratorObject::SlotCount),
     JS_PropertyStub,         /* addProperty */
-    JS_PropertyStub,         /* delProperty */
+    JS_DeletePropertyStub,   /* delProperty */
     JS_PropertyStub,         /* getProperty */
     JS_StrictPropertyStub,   /* setProperty */
     JS_EnumerateStub,
@@ -1378,7 +1455,7 @@ Class js::SetIteratorClass = {
     SetIteratorObject::finalize
 };
 
-JSFunctionSpec SetIteratorObject::methods[] = {
+const JSFunctionSpec SetIteratorObject::methods[] = {
     JS_FN("next", next, 0, 0),
     JS_FS_END
 };
@@ -1428,7 +1505,7 @@ SetIteratorObject::create(JSContext *cx, HandleObject setobj, ValueSet *data)
 }
 
 void
-SetIteratorObject::finalize(FreeOp *fop, RawObject obj)
+SetIteratorObject::finalize(FreeOp *fop, JSObject *obj)
 {
     fop->delete_(obj->asSetIterator().range());
 }
@@ -1479,7 +1556,7 @@ Class SetObject::class_ = {
     JSCLASS_HAS_PRIVATE | JSCLASS_IMPLEMENTS_BARRIERS |
     JSCLASS_HAS_CACHED_PROTO(JSProto_Set),
     JS_PropertyStub,         // addProperty
-    JS_PropertyStub,         // delProperty
+    JS_DeletePropertyStub,   // delProperty
     JS_PropertyStub,         // getProperty
     JS_StrictPropertyStub,   // setProperty
     JS_EnumerateStub,
@@ -1493,12 +1570,12 @@ Class SetObject::class_ = {
     mark
 };
 
-JSPropertySpec SetObject::properties[] = {
+const JSPropertySpec SetObject::properties[] = {
     JS_PSG("size", size, 0),
     JS_PS_END
 };
 
-JSFunctionSpec SetObject::methods[] = {
+const JSFunctionSpec SetObject::methods[] = {
     JS_FN("has", has, 1, 0),
     JS_FN("add", add, 1, 0),
     JS_FN("delete", delete_, 1, 0),
@@ -1515,7 +1592,7 @@ SetObject::initClass(JSContext *cx, JSObject *obj)
 }
 
 void
-SetObject::mark(JSTracer *trc, RawObject obj)
+SetObject::mark(JSTracer *trc, JSObject *obj)
 {
     SetObject *setobj = static_cast<SetObject *>(obj);
     if (ValueSet *set = setobj->getData()) {
@@ -1525,13 +1602,11 @@ SetObject::mark(JSTracer *trc, RawObject obj)
 }
 
 void
-SetObject::finalize(FreeOp *fop, RawObject obj)
+SetObject::finalize(FreeOp *fop, JSObject *obj)
 {
     SetObject *setobj = static_cast<SetObject *>(obj);
-    if (ValueSet *set = setobj->getData()) {
-        set->clearWithoutCallingDestructors();
+    if (ValueSet *set = setobj->getData())
         fop->delete_(set);
-    }
 }
 
 JSBool
@@ -1554,14 +1629,14 @@ SetObject::construct(JSContext *cx, unsigned argc, Value *vp)
     if (args.hasDefined(0)) {
         ForOfIterator iter(cx, args[0]);
         while (iter.next()) {
-            HashableValue key;
-            HashableValue::AutoRooter hkeyRoot(cx, &key);
+            AutoHashableValueRooter key(cx);
             if (!key.setValue(cx, iter.value()))
                 return false;
             if (!set->put(key)) {
                 js_ReportOutOfMemory(cx);
                 return false;
             }
+            WriteBarrierPost(cx->runtime, set, key);
         }
         if (!iter.close())
             return false;
@@ -1632,6 +1707,7 @@ SetObject::add_impl(JSContext *cx, CallArgs args)
         js_ReportOutOfMemory(cx);
         return false;
     }
+    WriteBarrierPost(cx->runtime, &set, key);
     args.rval().setUndefined();
     return true;
 }
