@@ -1,11 +1,11 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=4 sw=4 et tw=99:
- *
+/* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "Ion.h"
+#include "IonBuilder.h"
 #include "IonSpewer.h"
 #include "CompileInfo.h"
 #include "ValueNumbering.h"
@@ -13,13 +13,14 @@
 using namespace js;
 using namespace js::ion;
 
-ValueNumberer::ValueNumberer(MIRGraph &graph, bool optimistic)
-  : graph_(graph),
+ValueNumberer::ValueNumberer(MIRGenerator *mir, MIRGraph &graph, bool optimistic)
+  : mir(mir),
+    graph_(graph),
     pessimisticPass_(!optimistic),
     count_(0)
 { }
 
-uint32
+uint32_t
 ValueNumberer::lookupValue(MDefinition *ins)
 {
 
@@ -70,6 +71,34 @@ ValueNumberer::simplify(MDefinition *def, bool useValueNumbers)
     return ins;
 }
 
+MControlInstruction *
+ValueNumberer::simplifyControlInstruction(MControlInstruction *def)
+{
+    if (def->isEffectful())
+        return def;
+
+    MDefinition *repl = def->foldsTo(false);
+    if (repl == def || !repl->updateForFolding(def))
+        return def;
+
+    // Ensure this instruction has a value number.
+    if (!repl->valueNumberData())
+        repl->setValueNumberData(new ValueNumberData);
+
+    MBasicBlock *block = def->block();
+
+    // MControlInstructions should not have consumers.
+    JS_ASSERT(repl->isControlInstruction());
+    JS_ASSERT(def->useCount() == 0);
+
+    if (def->isInWorklist())
+        repl->setInWorklist();
+
+    block->discardLastIns();
+    block->end((MControlInstruction *)repl);
+    return (MControlInstruction *)repl;
+}
+
 void
 ValueNumberer::markDefinition(MDefinition *def)
 {
@@ -145,6 +174,8 @@ ValueNumberer::computeValueNumbers()
         return false;
     // Stick a VN object onto every mdefinition
     for (ReversePostorderIterator block(graph_.rpoBegin()); block != graph_.rpoEnd(); block++) {
+        if (mir->shouldCancel("Value Numbering (preparation loop"))
+            return false;
         for (MDefinitionIterator iter(*block); iter; iter++)
             iter->setValueNumberData(new ValueNumberData);
         MControlInstruction *jump = block->lastIns();
@@ -189,6 +220,8 @@ ValueNumberer::computeValueNumbers()
         }
 #endif
         for (ReversePostorderIterator block(graph_.rpoBegin()); block != graph_.rpoEnd(); block++) {
+            if (mir->shouldCancel("Value Numbering (main loop)"))
+                return false;
             for (MDefinitionIterator iter(*block); iter; ) {
 
                 if (!isMarked(*iter)) {
@@ -206,7 +239,7 @@ ValueNumberer::computeValueNumbers()
                     continue;
                 }
 
-                uint32 value = lookupValue(ins);
+                uint32_t value = lookupValue(ins);
 
                 if (!value)
                     return false; // Hashtable insertion failed
@@ -223,6 +256,7 @@ ValueNumberer::computeValueNumbers()
             }
             // Process control flow instruction:
             MControlInstruction *jump = block->lastIns();
+            jump = simplifyControlInstruction(jump);
 
             // If we are pessimistic, then this will never get set.
             if (!jump->isInWorklist())
@@ -325,6 +359,8 @@ ValueNumberer::eliminateRedundancies()
 
     // Starting from each self-dominating block, traverse the CFG in pre-order.
     while (!worklist.empty()) {
+        if (mir->shouldCancel("Value Numbering (eliminate loop)"))
+            return false;
         MBasicBlock *block = worklist.popCopy();
 
         IonSpew(IonSpew_GVN, "Looking at block %d", block->id());
@@ -386,7 +422,7 @@ ValueNumberer::analyze()
     return computeValueNumbers() && eliminateRedundancies();
 }
 
-uint32
+uint32_t
 MDefinition::valueNumber() const
 {
     JS_ASSERT(block_);
@@ -395,7 +431,7 @@ MDefinition::valueNumber() const
     return valueNumber_->valueNumber();
 }
 void
-MDefinition::setValueNumber(uint32 vn)
+MDefinition::setValueNumber(uint32_t vn)
 {
     valueNumber_->setValueNumber(vn);
 }
