@@ -1,23 +1,51 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-#include "mozilla/FloatingPoint.h"
+#include "mozilla/Endian.h"
+/*
+ * This file implements the structured clone algorithm of
+ * http://www.whatwg.org/specs/web-apps/current-work/multipage/common-dom-interfaces.html#safe-passing-of-structured-data
+ *
+ * The implementation differs slightly in that it uses an explicit stack, and
+ * the "memory" maps source objects to sequential integer indexes rather than
+ * directly pointing to destination objects. As a result, the order in which
+ * things are added to the memory must exactly match the order in which they
+ * are placed into 'allObjs', an analogous array of back-referenceable
+ * destination objects constructed while reading.
+ *
+ * For the most part, this is easy: simply add objects to the memory when first
+ * encountering them. But reading in a typed array requires an ArrayBuffer for
+ * construction, so objects cannot just be added to 'allObjs' in the order they
+ * are created. If they were, ArrayBuffers would come before typed arrays when
+ * in fact the typed array was added to 'memory' first.
+ *
+ * So during writing, we add objects to the memory when first encountering
+ * them. When reading a typed array, a placeholder is pushed onto allObjs until
+ * the ArrayBuffer has been read, then it is updated with the actual typed
+ * array object.
+ */
 
 #include "jsclone.h"
+
+#include "mozilla/FloatingPoint.h"
+
 #include "jsdate.h"
 #include "jstypedarray.h"
 
 #include "jstypedarrayinlines.h"
 
 #include "vm/BooleanObject-inl.h"
-#include "vm/NumberObject-inl.h"
 #include "vm/RegExpObject-inl.h"
-#include "vm/StringObject-inl.h"
 
 using namespace js;
 
+using mozilla::IsNaN;
+using mozilla::LittleEndian;
+using mozilla::NativeEndian;
+
 enum StructuredDataType {
     /* Structured data types provided by the engine */
     SCTAG_FLOAT_MAX = 0xFFF00000,
@@ -37,17 +65,18 @@ enum StructuredDataType {
     SCTAG_BACK_REFERENCE_OBJECT,
     SCTAG_TRANSFER_MAP_HEADER,
     SCTAG_TRANSFER_MAP,
-    SCTAG_TYPED_ARRAY_MIN = 0xFFFF0100,
-    SCTAG_TYPED_ARRAY_INT8 = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_INT8,
-    SCTAG_TYPED_ARRAY_UINT8 = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_UINT8,
-    SCTAG_TYPED_ARRAY_INT16 = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_INT16,
-    SCTAG_TYPED_ARRAY_UINT16 = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_UINT16,
-    SCTAG_TYPED_ARRAY_INT32 = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_INT32,
-    SCTAG_TYPED_ARRAY_UINT32 = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_UINT32,
-    SCTAG_TYPED_ARRAY_FLOAT32 = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_FLOAT32,
-    SCTAG_TYPED_ARRAY_FLOAT64 = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_FLOAT64,
-    SCTAG_TYPED_ARRAY_UINT8_CLAMPED = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_UINT8_CLAMPED,
-    SCTAG_TYPED_ARRAY_MAX = SCTAG_TYPED_ARRAY_MIN + TypedArray::TYPE_MAX - 1,
+    SCTAG_TYPED_ARRAY_OBJECT,
+    SCTAG_TYPED_ARRAY_V1_MIN = 0xFFFF0100,
+    SCTAG_TYPED_ARRAY_V1_INT8 = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_INT8,
+    SCTAG_TYPED_ARRAY_V1_UINT8 = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_UINT8,
+    SCTAG_TYPED_ARRAY_V1_INT16 = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_INT16,
+    SCTAG_TYPED_ARRAY_V1_UINT16 = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_UINT16,
+    SCTAG_TYPED_ARRAY_V1_INT32 = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_INT32,
+    SCTAG_TYPED_ARRAY_V1_UINT32 = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_UINT32,
+    SCTAG_TYPED_ARRAY_V1_FLOAT32 = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_FLOAT32,
+    SCTAG_TYPED_ARRAY_V1_FLOAT64 = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_FLOAT64,
+    SCTAG_TYPED_ARRAY_V1_UINT8_CLAMPED = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_UINT8_CLAMPED,
+    SCTAG_TYPED_ARRAY_V1_MAX = SCTAG_TYPED_ARRAY_V1_MIN + TypedArray::TYPE_MAX - 1,
     SCTAG_END_OF_BUILTIN_TYPES
 };
 
@@ -63,69 +92,14 @@ js_GetSCOffset(JSStructuredCloneWriter* writer)
     return writer->output().count() * sizeof(uint64_t);
 }
 
-static StructuredDataType
-ArrayTypeToTag(uint32_t type)
-{
-    JS_ASSERT(type < TypedArray::TYPE_MAX);
-    return static_cast<StructuredDataType>(uint32_t(SCTAG_TYPED_ARRAY_MIN) + type);
-}
-
 JS_STATIC_ASSERT(SCTAG_END_OF_BUILTIN_TYPES <= JS_SCTAG_USER_MIN);
 JS_STATIC_ASSERT(JS_SCTAG_USER_MIN <= JS_SCTAG_USER_MAX);
 JS_STATIC_ASSERT(TypedArray::TYPE_INT8 == 0);
 
-static uint8_t
-SwapBytes(uint8_t u)
-{
-    return u;
-}
-
-static uint16_t
-SwapBytes(uint16_t u)
-{
-#ifdef IS_BIG_ENDIAN
-    return ((u & 0x00ff) << 8) | ((u & 0xff00) >> 8);
-#else
-    return u;
-#endif
-}
-
-static uint32_t
-SwapBytes(uint32_t u)
-{
-#ifdef IS_BIG_ENDIAN
-    return ((u & 0x000000ffU) << 24) |
-           ((u & 0x0000ff00U) << 8) |
-           ((u & 0x00ff0000U) >> 8) |
-           ((u & 0xff000000U) >> 24);
-#else
-    return u;
-#endif
-}
-
-static uint64_t
-SwapBytes(uint64_t u)
-{
-#ifdef IS_BIG_ENDIAN
-    return ((u & 0x00000000000000ffLLU) << 56) |
-           ((u & 0x000000000000ff00LLU) << 40) |
-           ((u & 0x0000000000ff0000LLU) << 24) |
-           ((u & 0x00000000ff000000LLU) << 8) |
-           ((u & 0x000000ff00000000LLU) >> 8) |
-           ((u & 0x0000ff0000000000LLU) >> 24) |
-           ((u & 0x00ff000000000000LLU) >> 40) |
-           ((u & 0xff00000000000000LLU) >> 56);
-#else
-    return u;
-#endif
-}
-
-namespace js {
-
 bool
-WriteStructuredClone(JSContext *cx, const Value &v, uint64_t **bufp, size_t *nbytesp,
-                     const JSStructuredCloneCallbacks *cb, void *cbClosure,
-                     jsval transferable)
+js::WriteStructuredClone(JSContext *cx, HandleValue v, uint64_t **bufp, size_t *nbytesp,
+                         const JSStructuredCloneCallbacks *cb, void *cbClosure,
+                         jsval transferable)
 {
     SCOutput out(cx);
     JSStructuredCloneWriter w(out, cb, cbClosure, transferable);
@@ -133,8 +107,8 @@ WriteStructuredClone(JSContext *cx, const Value &v, uint64_t **bufp, size_t *nby
 }
 
 bool
-ReadStructuredClone(JSContext *cx, uint64_t *data, size_t nbytes, Value *vp,
-                    const JSStructuredCloneCallbacks *cb, void *cbClosure)
+js::ReadStructuredClone(JSContext *cx, uint64_t *data, size_t nbytes, Value *vp,
+                        const JSStructuredCloneCallbacks *cb, void *cbClosure)
 {
     SCInput in(cx, data, nbytes);
 
@@ -146,21 +120,27 @@ ReadStructuredClone(JSContext *cx, uint64_t *data, size_t nbytes, Value *vp,
 }
 
 bool
-ClearStructuredClone(const uint64_t *data, size_t nbytes)
+js::ClearStructuredClone(const uint64_t *data, size_t nbytes)
 {
     const uint64_t *point = data;
     const uint64_t *end = data + nbytes / 8;
 
-    uint64_t u = SwapBytes(*point++);
+    uint64_t u = LittleEndian::readUint64(point++);
     uint32_t tag = uint32_t(u >> 32);
     if (tag == SCTAG_TRANSFER_MAP_HEADER) {
         if ((TransferableMapHeader)uint32_t(u) == SCTAG_TM_NOT_MARKED) {
             while (point != end) {
-                uint64_t u = SwapBytes(*point++);
+                uint64_t u = LittleEndian::readUint64(point++);
                 uint32_t tag = uint32_t(u >> 32);
                 if (tag == SCTAG_TRANSFER_MAP) {
-                    u = SwapBytes(*point++);
+                    u = LittleEndian::readUint64(point++);
                     js_free(reinterpret_cast<void*>(u));
+                } else {
+                    // The only things in the transfer map should be
+                    // SCTAG_TRANSFER_MAP tags paired with pointers. If we find
+                    // any other tag, we've walked off the end of the transfer
+                    // map.
+                    break;
                 }
             }
         }
@@ -171,13 +151,12 @@ ClearStructuredClone(const uint64_t *data, size_t nbytes)
 }
 
 bool
-StructuredCloneHasTransferObjects(const uint64_t *data, size_t nbytes,
-                                  bool *hasTransferable)
+js::StructuredCloneHasTransferObjects(const uint64_t *data, size_t nbytes, bool *hasTransferable)
 {
     *hasTransferable = false;
 
     if (data) {
-        uint64_t u = SwapBytes(*data);
+        uint64_t u = LittleEndian::readUint64(data);
         uint32_t tag = uint32_t(u >> 32);
         if (tag == SCTAG_TRANSFER_MAP_HEADER) {
             *hasTransferable = true;
@@ -187,8 +166,6 @@ StructuredCloneHasTransferObjects(const uint64_t *data, size_t nbytes,
     return true;
 }
 
-} /* namespace js */
-
 static inline uint64_t
 PairToUInt64(uint32_t tag, uint32_t data)
 {
@@ -212,16 +189,18 @@ SCInput::SCInput(JSContext *cx, uint64_t *data, size_t nbytes)
 bool
 SCInput::read(uint64_t *p)
 {
-    if (point == end)
+    if (point == end) {
+        *p = 0;  /* initialize to shut GCC up */
         return eof();
-    *p = SwapBytes(*point++);
+    }
+    *p = LittleEndian::readUint64(point++);
     return true;
 }
 
 bool
 SCInput::readPair(uint32_t *tagp, uint32_t *datap)
 {
-    uint64_t u = 0;     /* initialize to shut GCC up */
+    uint64_t u;
     bool ok = read(&u);
     if (ok) {
         *tagp = uint32_t(u >> 32);
@@ -235,7 +214,7 @@ SCInput::get(uint64_t *p)
 {
     if (point == end)
         return eof();
-    *p = SwapBytes(*point);
+    *p = LittleEndian::readUint64(point);
     return true;
 }
 
@@ -256,7 +235,7 @@ SCInput::replace(uint64_t u)
 {
     if (point == end)
        return eof();
-    *point = SwapBytes(u);
+    LittleEndian::writeUint64(point, u);
     return true;
 }
 
@@ -289,6 +268,20 @@ SCInput::readDouble(double *p)
     return true;
 }
 
+template <typename T>
+static void
+copyAndSwapFromLittleEndian(T *dest, const void *src, size_t nelems)
+{
+    NativeEndian::copyAndSwapFromLittleEndian(dest, src, nelems);
+}
+
+template <>
+void
+copyAndSwapFromLittleEndian(uint8_t *dest, const void *src, size_t nelems)
+{
+    memcpy(dest, src, nelems);
+}
+
 template <class T>
 bool
 SCInput::readArray(T *p, size_t nelems)
@@ -303,14 +296,7 @@ SCInput::readArray(T *p, size_t nelems)
     if (nelems + sizeof(uint64_t) / sizeof(T) - 1 < nelems || nwords > size_t(end - point))
         return eof();
 
-    if (sizeof(T) == 1) {
-        js_memcpy(p, point, nelems);
-    } else {
-        const T *q = (const T *) point;
-        const T *qend = q + nelems;
-        while (q != qend)
-            *p++ = ::SwapBytes(*q++);
-    }
+    copyAndSwapFromLittleEndian(p, point, nelems);
     point += nwords;
     return true;
 }
@@ -331,7 +317,12 @@ SCInput::readChars(jschar *p, size_t nchars)
 bool
 SCInput::readPtr(void **p)
 {
-    return read((uint64_t *)p);
+    // On a 32 bit system the void* variable we have to write to is only
+    // 32 bits, so we create a 64 temporary and discard the unused bits.
+    uint64_t tmp;
+    bool ret = read(&tmp);
+    *p = reinterpret_cast<void*>(tmp);
+    return ret;
 }
 
 SCOutput::SCOutput(JSContext *cx) : cx(cx), buf(cx) {}
@@ -339,7 +330,7 @@ SCOutput::SCOutput(JSContext *cx) : cx(cx), buf(cx) {}
 bool
 SCOutput::write(uint64_t u)
 {
-    return buf.append(SwapBytes(u));
+    return buf.append(NativeEndian::swapToLittleEndian(u));
 }
 
 bool
@@ -391,6 +382,20 @@ SCOutput::writeDouble(double d)
     return write(ReinterpretDoubleAsUInt64(CanonicalizeNan(d)));
 }
 
+template <typename T>
+static void
+copyAndSwapToLittleEndian(void *dest, const T *src, size_t nelems)
+{
+    NativeEndian::copyAndSwapToLittleEndian(dest, src, nelems);
+}
+
+template <>
+void
+copyAndSwapToLittleEndian(void *dest, const uint8_t *src, size_t nelems)
+{
+    memcpy(dest, src, nelems);
+}
+
 template <class T>
 bool
 SCOutput::writeArray(const T *p, size_t nelems)
@@ -413,13 +418,7 @@ SCOutput::writeArray(const T *p, size_t nelems)
     buf.back() = 0;  /* zero-pad to an 8-byte boundary */
 
     T *q = (T *) &buf[start];
-    if (sizeof(T) == 1) {
-        js_memcpy(q, p, nelems);
-    } else {
-        const T *pend = p + nelems;
-        while (p != pend)
-            *q++ = ::SwapBytes(*p++);
-    }
+    copyAndSwapToLittleEndian(q, p, nelems);
     return true;
 }
 
@@ -464,7 +463,7 @@ JSStructuredCloneWriter::parseTransferable()
         return false;
     }
 
-    JSObject* array = &transferable.toObject();
+    RootedObject array(context(), &transferable.toObject());
     if (!JS_IsArrayObject(context(), array)) {
         reportErrorTransferable();
         return false;
@@ -475,9 +474,10 @@ JSStructuredCloneWriter::parseTransferable()
         return false;
     }
 
+    RootedValue v(context());
+
     for (uint32_t i = 0; i < length; ++i) {
-        Value v;
-        if (!JS_GetElement(context(), array, i, &v)) {
+        if (!JS_GetElement(context(), array, i, v.address())) {
             return false;
         }
 
@@ -486,7 +486,11 @@ JSStructuredCloneWriter::parseTransferable()
             return false;
         }
 
-        JSObject* tObj = &v.toObject();
+        JSObject* tObj = CheckedUnwrap(&v.toObject());
+        if (!tObj) {
+            JS_ReportError(context(), "Permission denied to access object");
+            return false;
+        }
         if (!tObj->isArrayBuffer()) {
             reportErrorTransferable();
             return false;
@@ -560,43 +564,43 @@ JS_PUBLIC_API(JSBool)
 JS_WriteTypedArray(JSStructuredCloneWriter *w, jsval v)
 {
     JS_ASSERT(v.isObject());
+    assertSameCompartment(w->context(), v);
     RootedObject obj(w->context(), &v.toObject());
 
-    // If the object is a security wrapper, try puncturing it. This may throw
-    // if the access is not allowed.
-    if (obj->isWrapper()) {
-        JSObject *unwrapped = UnwrapObjectChecked(w->context(), obj);
-        if (!unwrapped)
-            return false;
-        obj = unwrapped;
+    // If the object is a security wrapper, see if we're allowed to unwrap it.
+    // If we aren't, throw.
+    if (obj->isWrapper())
+        obj = CheckedUnwrap(obj);
+    if (!obj) {
+        JS_ReportError(w->context(), "Permission denied to access object");
+        return false;
     }
     return w->writeTypedArray(obj);
 }
 
+/*
+ * Write out a typed array. Note that post-v1 structured clone buffers do not
+ * perform endianness conversion on stored data, so multibyte typed arrays
+ * cannot be deserialized into a different endianness machine. Endianness
+ * conversion would prevent sharing ArrayBuffers: if you have Int8Array and
+ * Int16Array views of the same ArrayBuffer, should the data bytes be
+ * byte-swapped when writing or not? The Int8Array requires them to not be
+ * swapped; the Int16Array requires that they are.
+ */
 bool
 JSStructuredCloneWriter::writeTypedArray(HandleObject arr)
 {
-    if (!out.writePair(ArrayTypeToTag(TypedArray::type(arr)), TypedArray::length(arr)))
+    if (!out.writePair(SCTAG_TYPED_ARRAY_OBJECT, TypedArray::length(arr)))
+        return false;
+    uint64_t type = TypedArray::type(arr);
+    if (!out.write(type))
         return false;
 
-    switch (TypedArray::type(arr)) {
-    case TypedArray::TYPE_INT8:
-    case TypedArray::TYPE_UINT8:
-    case TypedArray::TYPE_UINT8_CLAMPED:
-        return out.writeArray((const uint8_t *) TypedArray::viewData(arr), TypedArray::length(arr));
-    case TypedArray::TYPE_INT16:
-    case TypedArray::TYPE_UINT16:
-        return out.writeArray((const uint16_t *) TypedArray::viewData(arr), TypedArray::length(arr));
-    case TypedArray::TYPE_INT32:
-    case TypedArray::TYPE_UINT32:
-    case TypedArray::TYPE_FLOAT32:
-        return out.writeArray((const uint32_t *) TypedArray::viewData(arr), TypedArray::length(arr));
-    case TypedArray::TYPE_FLOAT64:
-        return out.writeArray((const uint64_t *) TypedArray::viewData(arr), TypedArray::length(arr));
-    default:
-        JS_NOT_REACHED("unknown TypedArray type");
+    // Write out the ArrayBuffer tag and contents
+    if (!startWrite(TypedArray::bufferValue(arr)))
         return false;
-    }
+
+    return out.write(TypedArray::byteOffset(arr));
 }
 
 bool
@@ -669,9 +673,11 @@ JSStructuredCloneWriter::startWrite(const Value &v)
 
         // The object might be a security wrapper. See if we can clone what's
         // behind it. If we can, unwrap the object.
-        obj = UnwrapObjectChecked(context(), obj);
-        if (!obj)
+        obj = CheckedUnwrap(obj);
+        if (!obj) {
+            JS_ReportError(context(), "Permission denied to access object");
             return false;
+        }
 
         AutoCompartment ac(context(), obj);
 
@@ -761,8 +767,8 @@ JSStructuredCloneWriter::write(const Value &v)
                  */
                 RootedObject obj2(context());
                 RootedShape prop(context());
-                if (!js_HasOwnProperty(context(), obj->getOps()->lookupGeneric, obj, id,
-                                       &obj2, &prop)) {
+                if (!HasOwnProperty<CanGC>(context(), obj->getOps()->lookupGeneric, obj, id,
+                                           &obj2, &prop)) {
                     return false;
                 }
 
@@ -831,54 +837,100 @@ JSStructuredCloneReader::readString(uint32_t nchars)
     Chars chars(context());
     if (!chars.allocate(nchars) || !in.readChars(chars.get(), nchars))
         return NULL;
-    JSString *str = js_NewString(context(), chars.get(), nchars);
+    JSString *str = js_NewString<CanGC>(context(), chars.get(), nchars);
     if (str)
         chars.forget();
     return str;
 }
 
+static uint32_t
+TagToV1ArrayType(uint32_t tag)
+{
+    JS_ASSERT(tag >= SCTAG_TYPED_ARRAY_V1_MIN && tag <= SCTAG_TYPED_ARRAY_V1_MAX);
+    return tag - SCTAG_TYPED_ARRAY_V1_MIN;
+}
+
 JS_PUBLIC_API(JSBool)
 JS_ReadTypedArray(JSStructuredCloneReader *r, jsval *vp)
 {
     uint32_t tag, nelems;
     if (!r->input().readPair(&tag, &nelems))
         return false;
-    JS_ASSERT(tag >= SCTAG_TYPED_ARRAY_MIN && tag <= SCTAG_TYPED_ARRAY_MAX);
-    return r->readTypedArray(tag, nelems, vp);
+    if (tag >= SCTAG_TYPED_ARRAY_V1_MIN && tag <= SCTAG_TYPED_ARRAY_V1_MAX) {
+        return r->readTypedArray(TagToV1ArrayType(tag), nelems, vp, true);
+    } else if (tag == SCTAG_TYPED_ARRAY_OBJECT) {
+        uint64_t arrayType;
+        if (!r->input().read(&arrayType))
+            return false;
+        return r->readTypedArray(arrayType, nelems, vp);
+    } else {
+        JS_ReportErrorNumber(r->context(), js_GetErrorMessage, NULL,
+                             JSMSG_SC_BAD_SERIALIZED_DATA, "expected type array");
+        return false;
+    }
 }
 
 bool
-JSStructuredCloneReader::readTypedArray(uint32_t tag, uint32_t nelems, Value *vp)
+JSStructuredCloneReader::readTypedArray(uint32_t arrayType, uint32_t nelems, Value *vp,
+                                        bool v1Read)
 {
+    if (arrayType > TypedArray::TYPE_UINT8_CLAMPED) {
+        JS_ReportErrorNumber(context(), js_GetErrorMessage, NULL,
+                             JSMSG_SC_BAD_SERIALIZED_DATA, "unhandled typed array element type");
+        return false;
+    }
+
+    // Push a placeholder onto the allObjs list to stand in for the typed array
+    uint32_t placeholderIndex = allObjs.length();
+    Value dummy = JSVAL_NULL;
+    if (!allObjs.append(dummy))
+        return false;
+
+    // Read the ArrayBuffer object and its contents (but no properties)
+    RootedValue v(context());
+    uint32_t byteOffset;
+    if (v1Read) {
+        if (!readV1ArrayBuffer(arrayType, nelems, v.address()))
+            return false;
+        byteOffset = 0;
+    } else {
+        if (!startRead(v.address()))
+            return false;
+        uint64_t n;
+        if (!in.read(&n))
+            return false;
+        byteOffset = n;
+    }
+    RootedObject buffer(context(), &v.toObject());
     RootedObject obj(context(), NULL);
 
-    switch (tag) {
-      case SCTAG_TYPED_ARRAY_INT8:
-        obj = JS_NewInt8Array(context(), nelems);
+    switch (arrayType) {
+      case TypedArray::TYPE_INT8:
+        obj = JS_NewInt8ArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
-      case SCTAG_TYPED_ARRAY_UINT8:
-        obj = JS_NewUint8Array(context(), nelems);
+      case TypedArray::TYPE_UINT8:
+        obj = JS_NewUint8ArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
-      case SCTAG_TYPED_ARRAY_INT16:
-        obj = JS_NewInt16Array(context(), nelems);
+      case TypedArray::TYPE_INT16:
+        obj = JS_NewInt16ArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
-      case SCTAG_TYPED_ARRAY_UINT16:
-        obj = JS_NewUint16Array(context(), nelems);
+      case TypedArray::TYPE_UINT16:
+        obj = JS_NewUint16ArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
-      case SCTAG_TYPED_ARRAY_INT32:
-        obj = JS_NewInt32Array(context(), nelems);
+      case TypedArray::TYPE_INT32:
+        obj = JS_NewInt32ArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
-      case SCTAG_TYPED_ARRAY_UINT32:
-        obj = JS_NewUint32Array(context(), nelems);
+      case TypedArray::TYPE_UINT32:
+        obj = JS_NewUint32ArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
-      case SCTAG_TYPED_ARRAY_FLOAT32:
-        obj = JS_NewFloat32Array(context(), nelems);
+      case TypedArray::TYPE_FLOAT32:
+        obj = JS_NewFloat32ArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
-      case SCTAG_TYPED_ARRAY_FLOAT64:
-        obj = JS_NewFloat64Array(context(), nelems);
+      case TypedArray::TYPE_FLOAT64:
+        obj = JS_NewFloat64ArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
-      case SCTAG_TYPED_ARRAY_UINT8_CLAMPED:
-        obj = JS_NewUint8ClampedArray(context(), nelems);
+      case TypedArray::TYPE_UINT8_CLAMPED:
+        obj = JS_NewUint8ClampedArrayWithBuffer(context(), buffer, byteOffset, nelems);
         break;
       default:
         JS_NOT_REACHED("unknown TypedArray type");
@@ -889,30 +941,9 @@ JSStructuredCloneReader::readTypedArray(uint32_t tag, uint32_t nelems, Value *vp
         return false;
     vp->setObject(*obj);
 
-    JS_ASSERT(TypedArray::length(obj) == nelems);
-    switch (tag) {
-      case SCTAG_TYPED_ARRAY_INT8:
-        return in.readArray((uint8_t*) JS_GetInt8ArrayData(obj, context()), nelems);
-      case SCTAG_TYPED_ARRAY_UINT8:
-        return in.readArray(JS_GetUint8ArrayData(obj, context()), nelems);
-      case SCTAG_TYPED_ARRAY_INT16:
-        return in.readArray((uint16_t*) JS_GetInt16ArrayData(obj, context()), nelems);
-      case SCTAG_TYPED_ARRAY_UINT16:
-        return in.readArray(JS_GetUint16ArrayData(obj, context()), nelems);
-      case SCTAG_TYPED_ARRAY_INT32:
-        return in.readArray((uint32_t*) JS_GetInt32ArrayData(obj, context()), nelems);
-      case SCTAG_TYPED_ARRAY_UINT32:
-        return in.readArray(JS_GetUint32ArrayData(obj, context()), nelems);
-      case SCTAG_TYPED_ARRAY_FLOAT32:
-        return in.readArray((uint32_t*) JS_GetFloat32ArrayData(obj, context()), nelems);
-      case SCTAG_TYPED_ARRAY_FLOAT64:
-        return in.readArray((uint64_t*) JS_GetFloat64ArrayData(obj, context()), nelems);
-      case SCTAG_TYPED_ARRAY_UINT8_CLAMPED:
-        return in.readArray(JS_GetUint8ClampedArrayData(obj, context()), nelems);
-      default:
-        JS_NOT_REACHED("unknown TypedArray type");
-        return false;
-    }
+    allObjs[placeholderIndex] = *vp;
+
+    return true;
 }
 
 bool
@@ -927,6 +958,66 @@ JSStructuredCloneReader::readArrayBuffer(uint32_t nbytes, Value *vp)
     return in.readArray(buffer.dataPointer(), nbytes);
 }
 
+static size_t
+bytesPerTypedArrayElement(uint32_t arrayType)
+{
+    switch (arrayType) {
+      case TypedArray::TYPE_INT8:
+      case TypedArray::TYPE_UINT8:
+      case TypedArray::TYPE_UINT8_CLAMPED:
+        return sizeof(uint8_t);
+      case TypedArray::TYPE_INT16:
+      case TypedArray::TYPE_UINT16:
+        return sizeof(uint16_t);
+      case TypedArray::TYPE_INT32:
+      case TypedArray::TYPE_UINT32:
+      case TypedArray::TYPE_FLOAT32:
+        return sizeof(uint32_t);
+      case TypedArray::TYPE_FLOAT64:
+        return sizeof(uint64_t);
+      default:
+        JS_NOT_REACHED("unknown TypedArray type");
+        return 0;
+    }
+}
+
+/*
+ * Read in the data for a structured clone version 1 ArrayBuffer, performing
+ * endianness-conversion while reading.
+ */
+bool
+JSStructuredCloneReader::readV1ArrayBuffer(uint32_t arrayType, uint32_t nelems, Value *vp)
+{
+    JS_ASSERT(arrayType <= TypedArray::TYPE_UINT8_CLAMPED);
+
+    uint32_t nbytes = nelems * bytesPerTypedArrayElement(arrayType);
+    JSObject *obj = ArrayBufferObject::create(context(), nbytes);
+    if (!obj)
+        return false;
+    vp->setObject(*obj);
+    ArrayBufferObject &buffer = obj->asArrayBuffer();
+    JS_ASSERT(buffer.byteLength() == nbytes);
+
+    switch (arrayType) {
+      case TypedArray::TYPE_INT8:
+      case TypedArray::TYPE_UINT8:
+      case TypedArray::TYPE_UINT8_CLAMPED:
+        return in.readArray((uint8_t*) buffer.dataPointer(), nelems);
+      case TypedArray::TYPE_INT16:
+      case TypedArray::TYPE_UINT16:
+        return in.readArray((uint16_t*) buffer.dataPointer(), nelems);
+      case TypedArray::TYPE_INT32:
+      case TypedArray::TYPE_UINT32:
+      case TypedArray::TYPE_FLOAT32:
+        return in.readArray((uint32_t*) buffer.dataPointer(), nelems);
+      case TypedArray::TYPE_FLOAT64:
+        return in.readArray((uint64_t*) buffer.dataPointer(), nelems);
+      default:
+        JS_NOT_REACHED("unknown TypedArray type");
+        return false;
+    }
+}
+
 bool
 JSStructuredCloneReader::startRead(Value *vp)
 {
@@ -975,7 +1066,7 @@ JSStructuredCloneReader::startRead(Value *vp)
         double d;
         if (!in.readDouble(&d) || !checkDouble(d))
             return false;
-        if (!MOZ_DOUBLE_IS_NaN(d) && d != TimeClip(d)) {
+        if (!IsNaN(d) && d != TimeClip(d)) {
             JS_ReportErrorNumber(context(), js_GetErrorMessage, NULL, JSMSG_SC_BAD_SERIALIZED_DATA,
                                  "date");
             return false;
@@ -1005,8 +1096,8 @@ JSStructuredCloneReader::startRead(Value *vp)
             return false;
 
         size_t length = stable->length();
-        const jschar *chars = stable->chars();
-        RegExpObject *reobj = RegExpObject::createNoStatics(context(), chars, length, flags, NULL);
+        const StableCharPtr chars = stable->chars();
+        RegExpObject *reobj = RegExpObject::createNoStatics(context(), chars.get(), length, flags, NULL);
         if (!reobj)
             return false;
         vp->setObject(*reobj);
@@ -1054,6 +1145,14 @@ JSStructuredCloneReader::startRead(Value *vp)
             return false;
         break;
 
+      case SCTAG_TYPED_ARRAY_OBJECT:
+        // readTypedArray adds the array to allObjs
+        uint64_t arrayType;
+        if (!in.read(&arrayType))
+            return false;
+        return readTypedArray(arrayType, data, vp);
+        break;
+
       default: {
         if (tag <= SCTAG_FLOAT_MAX) {
             double d = ReinterpretPairAsDouble(tag, data);
@@ -1063,10 +1162,10 @@ JSStructuredCloneReader::startRead(Value *vp)
             break;
         }
 
-        if (SCTAG_TYPED_ARRAY_MIN <= tag && tag <= SCTAG_TYPED_ARRAY_MAX) {
-            if (!readTypedArray(tag, data, vp))
-                return false;
-            break;
+        if (SCTAG_TYPED_ARRAY_V1_MIN <= tag && tag <= SCTAG_TYPED_ARRAY_V1_MAX) {
+            // A v1-format typed array
+            // readTypedArray adds the array to allObjs
+            return readTypedArray(TagToV1ArrayType(tag), data, vp, true);
         }
 
         if (!callbacks || !callbacks->read) {
@@ -1102,7 +1201,7 @@ JSStructuredCloneReader::readId(jsid *idp)
         JSString *str = readString(data);
         if (!str)
             return false;
-        JSAtom *atom = AtomizeString(context(), str);
+        JSAtom *atom = AtomizeString<CanGC>(context(), str);
         if (!atom)
             return false;
         *idp = NON_INTEGER_ATOM_TO_JSID(atom);
