@@ -1,6 +1,5 @@
 /* -*- Mode: C++; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*-
- * vim: set ts=8 sw=4 et tw=78:
- *
+ * vim: set ts=8 sts=4 et sw=4 tw=99:
  * This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -25,13 +24,8 @@ class StringObject : public JSObject
      * Creates a new String object boxing the given string.  The object's
      * [[Prototype]] is determined from context.
      */
-    static inline StringObject *create(JSContext *cx, HandleString str);
-
-    /*
-     * Identical to create(), but uses |proto| as [[Prototype]].  This method
-     * must not be used to create |String.prototype|.
-     */
-    static inline StringObject *createWithProto(JSContext *cx, HandleString str, JSObject &proto);
+    static inline StringObject *create(JSContext *cx, HandleString str,
+                                       NewObjectKind newKind = GenericObject);
 
     JSString *unbox() const {
         return getFixedSlot(PRIMITIVE_VALUE_SLOT).toString();
