@@ -6,21 +6,22 @@
 #include "mozilla/Util.h"
 
 #include "jsapi.h"
-#include "jsatom.h"
 #include "jsfriendapi.h"
 #include "nsCOMPtr.h"
 #include "xpcprivate.h"
 #include "XPCInlines.h"
 #include "XPCQuickStubs.h"
-#include "XPCWrapper.h"
 #include "mozilla/dom/BindingUtils.h"
 
 using namespace mozilla;
+using namespace JS;
+
+extern const char* xpc_qsStringTable;
 
 static inline QITableEntry *
 GetOffsets(nsISupports *identity, XPCWrappedNativeProto* proto)
 {
-    QITableEntry* offsets = proto ? proto->GetOffsets() : nsnull;
+    QITableEntry* offsets = proto ? proto->GetOffsets() : nullptr;
     if (!offsets) {
         static NS_DEFINE_IID(kThisPtrOffsetsSID, NS_THISPTROFFSETS_SID);
         identity->QueryInterface(kThisPtrOffsetsSID, (void**)&offsets);
@@ -37,7 +38,7 @@ GetOffsetsFromSlimWrapper(JSObject *obj)
 }
 
 static const xpc_qsHashEntry *
-LookupEntry(PRUint32 tableSize, const xpc_qsHashEntry *table, const nsID &iid)
+LookupEntry(uint32_t tableSize, const xpc_qsHashEntry *table, const nsID &iid)
 {
     size_t i;
     const xpc_qsHashEntry *p;
@@ -50,11 +51,11 @@ LookupEntry(PRUint32 tableSize, const xpc_qsHashEntry *table, const nsID &iid)
             return p;
         i = p->chain;
     } while (i != XPC_QS_NULL_INDEX);
-    return nsnull;
+    return nullptr;
 }
 
 static const xpc_qsHashEntry *
-LookupInterfaceOrAncestor(PRUint32 tableSize, const xpc_qsHashEntry *table,
+LookupInterfaceOrAncestor(uint32_t tableSize, const xpc_qsHashEntry *table,
                           const nsID &iid)
 {
     const xpc_qsHashEntry *entry = LookupEntry(tableSize, table, iid);
@@ -65,7 +66,7 @@ LookupInterfaceOrAncestor(PRUint32 tableSize, const xpc_qsHashEntry *table,
          */
         nsCOMPtr<nsIInterfaceInfo> info;
         if (NS_FAILED(nsXPConnect::GetXPConnect()->GetInfoForIID(&iid, getter_AddRefs(info))))
-            return nsnull;
+            return nullptr;
 
         const nsIID *piid;
         for (;;) {
@@ -94,190 +95,14 @@ PointerFinalize(JSFreeOp *fop, JSObject *obj)
 JSClass
 PointerHolderClass = {
     "Pointer", JSCLASS_HAS_PRIVATE,
-    JS_PropertyStub, JS_PropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
+    JS_PropertyStub, JS_DeletePropertyStub, JS_PropertyStub, JS_StrictPropertyStub,
     JS_EnumerateStub, JS_ResolveStub, JS_ConvertStub, PointerFinalize
 };
 
-static JSBool
-ReifyPropertyOps(JSContext *cx, JSObject *obj, jsid id, unsigned orig_attrs,
-                 JSPropertyOp getter, JSStrictPropertyOp setter,
-                 JSObject **getterobjp, JSObject **setterobjp)
-{
-    // Generate both getter and setter and stash them in the prototype.
-    jsval roots[2] = { JSVAL_NULL, JSVAL_NULL };
-    JS::AutoArrayRooter tvr(cx, ArrayLength(roots), roots);
-
-    unsigned attrs = JSPROP_SHARED | (orig_attrs & JSPROP_ENUMERATE);
-    JSObject *getterobj;
-    if (getter) {
-        getterobj = GeneratePropertyOp(cx, obj, id, 0, getter);
-        if (!getterobj)
-            return false;
-        roots[0] = OBJECT_TO_JSVAL(getterobj);
-        attrs |= JSPROP_GETTER;
-    } else
-        getterobj = nsnull;
-
-    JSObject *setterobj;
-    if (setter) {
-        setterobj = GeneratePropertyOp(cx, obj, id, 1, setter);
-        if (!setterobj)
-            return false;
-        roots[1] = OBJECT_TO_JSVAL(setterobj);
-        attrs |= JSPROP_SETTER;
-    } else
-        setterobj = nsnull;
-
-    if (getterobjp)
-        *getterobjp = getterobj;
-    if (setterobjp)
-        *setterobjp = setterobj;
-    return JS_DefinePropertyById(cx, obj, id, JSVAL_VOID,
-                                 JS_DATA_TO_FUNC_PTR(JSPropertyOp, getterobj),
-                                 JS_DATA_TO_FUNC_PTR(JSStrictPropertyOp, setterobj),
-                                 attrs);
-}
-
-static JSBool
-LookupGetterOrSetter(JSContext *cx, JSBool wantGetter, unsigned argc, jsval *vp)
-{
-    XPC_QS_ASSERT_CONTEXT_OK(cx);
-
-    if (argc == 0) {
-        JS_SET_RVAL(cx, vp, JSVAL_VOID);
-        return true;
-    }
-
-    JSObject *obj = JS_THIS_OBJECT(cx, vp);
-    if (!obj)
-        return false;
-
-    jsval idval = JS_ARGV(cx, vp)[0];
-    jsid id;
-    JSPropertyDescriptor desc;
-    if (!JS_ValueToId(cx, idval, &id) ||
-        !JS_GetPropertyDescriptorById(cx, obj, id, JSRESOLVE_QUALIFIED, &desc))
-        return false;
-
-    // No property at all means no getters or setters possible.
-    if (!desc.obj) {
-        JS_SET_RVAL(cx, vp, JSVAL_VOID);
-        return true;
-    }
-
-    // Inline obj_lookup[GS]etter here.
-    if (wantGetter) {
-        if (desc.attrs & JSPROP_GETTER) {
-            JS_SET_RVAL(cx, vp,
-                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, desc.getter)));
-            return true;
-        }
-    } else {
-        if (desc.attrs & JSPROP_SETTER) {
-            JS_SET_RVAL(cx, vp,
-                        OBJECT_TO_JSVAL(JS_FUNC_TO_DATA_PTR(JSObject *, desc.setter)));
-            return true;
-        }
-    }
-
-    // Since XPConnect doesn't use JSPropertyOps in any other contexts,
-    // ensuring that we have an XPConnect prototype object ensures that
-    // we are only going to expose quickstubbed properties to script.
-    // Also be careful not to overwrite existing properties!
-
-    if (!JSID_IS_STRING(id) ||
-        !IS_PROTO_CLASS(js::GetObjectClass(desc.obj)) ||
-        (desc.attrs & (JSPROP_GETTER | JSPROP_SETTER)) ||
-        !(desc.getter || desc.setter) ||
-        desc.setter == js::GetObjectJSClass(desc.obj)->setProperty) {
-        JS_SET_RVAL(cx, vp, JSVAL_VOID);
-        return true;
-    }
-
-    JSObject *getterobj, *setterobj;
-    if (!ReifyPropertyOps(cx, desc.obj, id, desc.attrs, desc.getter, desc.setter,
-                          &getterobj, &setterobj)) {
-        return false;
-    }
-
-    JSObject *wantedobj = wantGetter ? getterobj : setterobj;
-    jsval v = wantedobj ? OBJECT_TO_JSVAL(wantedobj) : JSVAL_VOID;
-    JS_SET_RVAL(cx, vp, v);
-    return true;
-}
-
-static JSBool
-SharedLookupGetter(JSContext *cx, unsigned argc, jsval *vp)
-{
-    return LookupGetterOrSetter(cx, true, argc, vp);
-}
-
-static JSBool
-SharedLookupSetter(JSContext *cx, unsigned argc, jsval *vp)
-{
-    return LookupGetterOrSetter(cx, false, argc, vp);
-}
-
-static JSBool
-DefineGetterOrSetter(JSContext *cx, unsigned argc, JSBool wantGetter, jsval *vp)
-{
-    unsigned attrs;
-    JSBool found;
-    JSPropertyOp getter;
-    JSStrictPropertyOp setter;
-    JSObject *obj2;
-    jsval v;
-    jsid id;
-
-    XPC_QS_ASSERT_CONTEXT_OK(cx);
-    JSObject *obj = JS_THIS_OBJECT(cx, vp);
-    if (!obj)
-        return false;
-    JSNative forward = wantGetter ? js::obj_defineGetter : js::obj_defineSetter;
-    jsval idval = (argc >= 1) ? JS_ARGV(cx, vp)[0] : JSVAL_VOID;
-    if (!JSVAL_IS_STRING(idval))
-        return forward(cx, argc, vp);
-
-    if (!JS_ValueToId(cx, idval, &id) ||
-        !JS_LookupPropertyWithFlagsById(cx, obj, id,
-                                        JSRESOLVE_QUALIFIED, &obj2, &v) ||
-        (obj2 &&
-         !JS_GetPropertyAttrsGetterAndSetterById(cx, obj2, id, &attrs,
-                                                 &found, &getter, &setter)))
-        return false;
-
-    // The property didn't exist, already has a getter or setter, or is not
-    // our property, then just forward now.
-    if (!obj2 ||
-        (attrs & (JSPROP_GETTER | JSPROP_SETTER)) ||
-        !(getter || setter) ||
-        !IS_PROTO_CLASS(js::GetObjectClass(obj2)))
-        return forward(cx, argc, vp);
-
-    // Reify the getter and setter...
-    if (!ReifyPropertyOps(cx, obj2, id, attrs, getter, setter, nsnull, nsnull))
-        return false;
-
-    return forward(cx, argc, vp);
-}
-
-static JSBool
-SharedDefineGetter(JSContext *cx, unsigned argc, jsval *vp)
-{
-    return DefineGetterOrSetter(cx, argc, true, vp);
-}
-
-static JSBool
-SharedDefineSetter(JSContext *cx, unsigned argc, jsval *vp)
-{
-    return DefineGetterOrSetter(cx, argc, false, vp);
-}
-
-
 JSBool
-xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, unsigned flags,
-                       PRUint32 ifacec, const nsIID **interfaces,
-                       PRUint32 tableSize, const xpc_qsHashEntry *table,
+xpc_qsDefineQuickStubs(JSContext *cx, JSObject *protoArg, unsigned flags,
+                       uint32_t ifacec, const nsIID **interfaces,
+                       uint32_t tableSize, const xpc_qsHashEntry *table,
                        const xpc_qsPropertySpec *propspecs,
                        const xpc_qsFunctionSpec *funcspecs,
                        const char *stringTable)
@@ -290,7 +115,7 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, unsigned flags,
      * searching the interfaces forward.  Here, definitions toward the
      * front of 'interfaces' overwrite those toward the back.
      */
-    bool definedProperty = false;
+    RootedObject proto(cx, protoArg);
     for (uint32_t i = ifacec; i-- != 0;) {
         const nsID &iid = *interfaces[i];
         const xpc_qsHashEntry *entry =
@@ -302,11 +127,12 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, unsigned flags,
                 const xpc_qsPropertySpec *ps = propspecs + entry->prop_index;
                 const xpc_qsPropertySpec *ps_end = ps + entry->n_props;
                 for ( ; ps < ps_end; ++ps) {
-                    definedProperty = true;
                     if (!JS_DefineProperty(cx, proto,
                                            stringTable + ps->name_index,
-                                           JSVAL_VOID, ps->getter, ps->setter,
-                                           flags | JSPROP_SHARED)) 
+                                           JSVAL_VOID,
+                                           (JSPropertyOp)ps->getter,
+                                           (JSStrictPropertyOp)ps->setter,
+                                           flags | JSPROP_SHARED | JSPROP_NATIVE_ACCESSORS))
                         return false;
                 }
 
@@ -321,6 +147,9 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, unsigned flags,
                         return false;
                 }
 
+                if (entry->newBindingProperties) {
+                    mozilla::dom::DefineWebIDLBindingPropertiesOnXPCProto(cx, proto, entry->newBindingProperties);
+                }
                 // Next.
                 size_t j = entry->parentInterface;
                 if (j == XPC_QS_NULL_INDEX)
@@ -330,17 +159,6 @@ xpc_qsDefineQuickStubs(JSContext *cx, JSObject *proto, unsigned flags,
         }
     }
 
-    static JSFunctionSpec getterfns[] = {
-        JS_FN("__lookupGetter__", SharedLookupGetter, 1, 0),
-        JS_FN("__lookupSetter__", SharedLookupSetter, 1, 0),
-        JS_FN("__defineGetter__", SharedDefineGetter, 2, 0),
-        JS_FN("__defineSetter__", SharedDefineSetter, 2, 0),
-        JS_FS_END
-    };
-
-    if (definedProperty && !JS_DefineFunctions(cx, proto, getterfns))
-        return false;
-
     return true;
 }
 
@@ -395,21 +213,21 @@ GetMemberInfo(JSObject *obj, jsid memberId, const char **ifaceName)
 static void
 GetMethodInfo(JSContext *cx, jsval *vp, const char **ifaceNamep, jsid *memberIdp)
 {
-    JSObject *funobj = JSVAL_TO_OBJECT(JS_CALLEE(cx, vp));
+    RootedObject funobj(cx, JSVAL_TO_OBJECT(JS_CALLEE(cx, vp)));
     NS_ASSERTION(JS_ObjectIsFunction(cx, funobj),
                  "JSNative callee should be Function object");
-    JSString *str = JS_GetFunctionId(JS_GetObjectFunction(funobj));
-    jsid methodId = str ? INTERNED_STRING_TO_JSID(cx, str) : JSID_VOID;
+    RootedString str(cx, JS_GetFunctionId(JS_GetObjectFunction(funobj)));
+    RootedId methodId(cx, str ? INTERNED_STRING_TO_JSID(cx, str) : JSID_VOID);
     GetMemberInfo(JSVAL_TO_OBJECT(vp[1]), methodId, ifaceNamep);
     *memberIdp = methodId;
 }
 
 static bool
 ThrowCallFailed(JSContext *cx, nsresult rv,
-                const char *ifaceName, jsid memberId, const char *memberName)
+                const char *ifaceName, HandleId memberId, const char *memberName)
 {
     /* Only one of memberId or memberName should be given. */
-    JS_ASSERT(JSID_IS_VOID(memberId) != !memberName);
+    MOZ_ASSERT(JSID_IS_VOID(memberId) != !memberName);
 
     // From XPCThrower::ThrowBadResult.
     char* sz;
@@ -427,7 +245,7 @@ ThrowCallFailed(JSContext *cx, nsresult rv,
 
     // else...
 
-    if (!nsXPCException::NameAndFormatForNSResult(NS_ERROR_XPC_NATIVE_RETURNED_FAILURE, nsnull, &format) ||
+    if (!nsXPCException::NameAndFormatForNSResult(NS_ERROR_XPC_NATIVE_RETURNED_FAILURE, nullptr, &format) ||
         !format) {
         format = "";
     }
@@ -435,10 +253,10 @@ ThrowCallFailed(JSContext *cx, nsresult rv,
     JSAutoByteString memberNameBytes;
     if (!memberName) {
         memberName = JSID_IS_STRING(memberId)
-                     ? memberNameBytes.encode(cx, JSID_TO_STRING(memberId))
+                     ? memberNameBytes.encodeLatin1(cx, JSID_TO_STRING(memberId))
                      : "unknown";
     }
-    if (nsXPCException::NameAndFormatForNSResult(rv, &name, nsnull)
+    if (nsXPCException::NameAndFormatForNSResult(rv, &name, nullptr)
         && name) {
         sz = JS_smprintf("%s 0x%x (%s) [%s.%s]",
                          format, rv, name, ifaceName, memberName);
@@ -457,19 +275,41 @@ ThrowCallFailed(JSContext *cx, nsresult rv,
 
 JSBool
 xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv, JSObject *obj,
-                              jsid memberId)
+                              jsid memberIdArg)
 {
+    RootedId memberId(cx, memberIdArg);
     const char *ifaceName;
     GetMemberInfo(obj, memberId, &ifaceName);
     return ThrowCallFailed(cx, rv, ifaceName, memberId, NULL);
 }
 
+JSBool
+xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv, JSObject *objArg,
+                              const char* memberName)
+{
+    RootedObject obj(cx, objArg);
+    JSString *str = JS_InternString(cx, memberName);
+    if (!str) {
+        return false;
+    }
+    return xpc_qsThrowGetterSetterFailed(cx, rv, obj,
+                                         INTERNED_STRING_TO_JSID(cx, str));
+}
+
+JSBool
+xpc_qsThrowGetterSetterFailed(JSContext *cx, nsresult rv, JSObject *obj,
+                              uint16_t memberIndex)
+{
+    return xpc_qsThrowGetterSetterFailed(cx, rv, obj,
+                                         xpc_qsStringTable + memberIndex);
+}
+
 JSBool
 xpc_qsThrowMethodFailed(JSContext *cx, nsresult rv, jsval *vp)
 {
     const char *ifaceName;
-    jsid memberId;
-    GetMethodInfo(cx, vp, &ifaceName, &memberId);
+    RootedId memberId(cx);
+    GetMethodInfo(cx, vp, &ifaceName, memberId.address());
     return ThrowCallFailed(cx, rv, ifaceName, memberId, NULL);
 }
 
@@ -485,7 +325,7 @@ xpc_qsThrowMethodFailedWithDetails(JSContext *cx, nsresult rv,
                                    const char *ifaceName,
                                    const char *memberName)
 {
-    return ThrowCallFailed(cx, rv, ifaceName, JSID_VOID, memberName);
+    return ThrowCallFailed(cx, rv, ifaceName, JSID_VOIDHANDLE, memberName);
 }
 
 static void
@@ -493,19 +333,19 @@ ThrowBadArg(JSContext *cx, nsresult rv, const char *ifaceName,
             jsid memberId, const char *memberName, unsigned paramnum)
 {
     /* Only one memberId or memberName should be given. */
-    JS_ASSERT(JSID_IS_VOID(memberId) != !memberName);
+    MOZ_ASSERT(JSID_IS_VOID(memberId) != !memberName);
 
     // From XPCThrower::ThrowBadParam.
     char* sz;
     const char* format;
 
-    if (!nsXPCException::NameAndFormatForNSResult(rv, nsnull, &format))
+    if (!nsXPCException::NameAndFormatForNSResult(rv, nullptr, &format))
         format = "";
 
     JSAutoByteString memberNameBytes;
     if (!memberName) {
         memberName = JSID_IS_STRING(memberId)
-                     ? memberNameBytes.encode(cx, JSID_TO_STRING(memberId))
+                     ? memberNameBytes.encodeLatin1(cx, JSID_TO_STRING(memberId))
                      : "unknown";
     }
     sz = JS_smprintf("%s arg %u [%s.%s]",
@@ -521,8 +361,8 @@ void
 xpc_qsThrowBadArg(JSContext *cx, nsresult rv, jsval *vp, unsigned paramnum)
 {
     const char *ifaceName;
-    jsid memberId;
-    GetMethodInfo(cx, vp, &ifaceName, &memberId);
+    RootedId memberId(cx);
+    GetMethodInfo(cx, vp, &ifaceName, memberId.address());
     ThrowBadArg(cx, rv, ifaceName, memberId, NULL, paramnum);
 }
 
@@ -541,15 +381,46 @@ xpc_qsThrowBadArgWithDetails(JSContext *cx, nsresult rv, unsigned paramnum,
 
 void
 xpc_qsThrowBadSetterValue(JSContext *cx, nsresult rv,
-                          JSObject *obj, jsid propId)
+                          JSObject *obj, jsid propIdArg)
 {
+    RootedId propId(cx, propIdArg);
     const char *ifaceName;
     GetMemberInfo(obj, propId, &ifaceName);
     ThrowBadArg(cx, rv, ifaceName, propId, NULL, 0);
 }
 
+void
+xpc_qsThrowBadSetterValue(JSContext *cx, nsresult rv,
+                          JSObject *objArg, const char* propName)
+{
+    RootedObject obj(cx, objArg);
+    JSString *str = JS_InternString(cx, propName);
+    if (!str) {
+        return;
+    }
+    xpc_qsThrowBadSetterValue(cx, rv, obj, INTERNED_STRING_TO_JSID(cx, str));
+}
+
+void
+xpc_qsThrowBadSetterValue(JSContext *cx, nsresult rv, JSObject *obj,
+                          uint16_t name_index)
+{
+    xpc_qsThrowBadSetterValue(cx, rv, obj, xpc_qsStringTable + name_index);
+}
+
 JSBool
-xpc_qsGetterOnlyPropertyStub(JSContext *cx, JSHandleObject obj, JSHandleId id, JSBool strict, jsval *vp)
+xpc_qsGetterOnlyPropertyStub(JSContext *cx, JSHandleObject obj, JSHandleId id, JSBool strict,
+                             JSMutableHandleValue vp)
+{
+    return JS_ReportErrorFlagsAndNumber(cx,
+                                        JSREPORT_WARNING | JSREPORT_STRICT |
+                                        JSREPORT_STRICT_MODE_ERROR,
+                                        js_GetErrorMessage, NULL,
+                                        JSMSG_GETTER_ONLY);
+}
+
+JSBool
+xpc_qsGetterOnlyNativeStub(JSContext *cx, unsigned argc, jsval *vp)
 {
     return JS_ReportErrorFlagsAndNumber(cx,
                                         JSREPORT_WARNING | JSREPORT_STRICT |
@@ -629,7 +500,7 @@ xpc_qsAUTF8String::xpc_qsAUTF8String(JSContext *cx, jsval v, jsval *pval)
 static nsresult
 getNative(nsISupports *idobj,
           QITableEntry* entries,
-          JSObject *obj,
+          HandleObject obj,
           const nsIID &iid,
           void **ppThis,
           nsISupports **pThisRef,
@@ -641,7 +512,7 @@ getNative(nsISupports *idobj,
             if (e->iid->Equals(iid)) {
                 *ppThis = (char*) idobj + e->offset - entries[0].offset;
                 *vp = OBJECT_TO_JSVAL(obj);
-                *pThisRef = nsnull;
+                *pThisRef = nullptr;
                 return NS_OK;
             }
         }
@@ -662,8 +533,9 @@ getNativeFromWrapper(JSContext *cx,
                      nsISupports **pThisRef,
                      jsval *vp)
 {
+    RootedObject obj(cx, wrapper->GetFlatJSObject());
     return getNative(wrapper->GetIdentityObject(), wrapper->GetOffsets(),
-                     wrapper->GetFlatJSObject(), iid, ppThis, pThisRef, vp);
+                     obj, iid, ppThis, pThisRef, vp);
 }
 
 
@@ -680,25 +552,24 @@ getWrapper(JSContext *cx,
     // * A (possible) outer window
     //
     // If we pass stopAtOuter == false, we can handle all three with one call
-    // to XPCWrapper::Unwrap.
+    // to js::CheckedUnwrap.
     if (js::IsWrapper(obj)) {
-        obj = XPCWrapper::Unwrap(cx, obj, false);
+        obj = js::CheckedUnwrap(obj, /* stopAtOuter = */ false);
 
-        // The safe unwrap might have failed for SCRIPT_ACCESS_ONLY objects. If it
-        // didn't fail though, we should be done with wrappers.
+        // The safe unwrap might have failed if we encountered an object that
+        // we're not allowed to unwrap. If it didn't fail though, we should be
+        // done with wrappers.
         if (!obj)
             return NS_ERROR_XPC_SECURITY_MANAGER_VETO;
         MOZ_ASSERT(!js::IsWrapper(obj));
     }
 
     // Start with sane values.
-    *wrapper = nsnull;
-    *cur = nsnull;
-    *tearoff = nsnull;
+    *wrapper = nullptr;
+    *cur = nullptr;
+    *tearoff = nullptr;
 
-    js::Class* clasp = js::GetObjectClass(obj);
-    if (dom::IsDOMClass(clasp) ||
-        dom::binding::instanceIsProxy(obj)) {
+    if (dom::IsDOMObject(obj)) {
         *cur = obj;
 
         return NS_OK;
@@ -710,6 +581,7 @@ getWrapper(JSContext *cx,
     // object reflection of a particular interface (ie, |foo.nsIBar|). These
     // JS objects are parented to their wrapper, so we snag the tearoff object
     // along the way (if desired), and then set |obj| to its parent.
+    js::Class* clasp = js::GetObjectClass(obj);
     if (clasp == &XPC_WN_Tearoff_JSClass) {
         *tearoff = (XPCWrappedNativeTearOff*) js::GetObjectPrivate(obj);
         obj = js::GetObjectParent(obj);
@@ -730,7 +602,7 @@ getWrapper(JSContext *cx,
 nsresult
 castNative(JSContext *cx,
            XPCWrappedNative *wrapper,
-           JSObject *cur,
+           JSObject *curArg,
            XPCWrappedNativeTearOff *tearoff,
            const nsIID &iid,
            void **ppThis,
@@ -738,6 +610,7 @@ castNative(JSContext *cx,
            jsval *vp,
            XPCLazyCallContext *lccx)
 {
+    RootedObject cur(cx, curArg);
     if (wrapper) {
         nsresult rv = getNativeFromWrapper(cx,wrapper, iid, ppThis, pThisRef,
                                            vp);
@@ -750,23 +623,14 @@ castNative(JSContext *cx,
     } else if (cur) {
         nsISupports *native;
         QITableEntry *entries;
-        js::Class* clasp = js::GetObjectClass(cur);
-        if (dom::IsDOMClass(clasp)) {
-            dom::DOMJSClass* domClass = dom::DOMJSClass::FromJSClass(clasp);
-            if (!domClass->mDOMObjectIsISupports) {
-                *pThisRef = nsnull;
-                return NS_ERROR_ILLEGAL_VALUE;
-            }
-            native = dom::UnwrapDOMObject<nsISupports>(cur);
-            entries = nsnull;
-        } else if (dom::binding::instanceIsProxy(cur)) {
-            native = static_cast<nsISupports*>(js::GetProxyPrivate(cur).toPrivate());
-            entries = nsnull;
-        } else if (IS_WRAPPER_CLASS(clasp) && IS_SLIM_WRAPPER_OBJECT(cur)) {
+        if (mozilla::dom::UnwrapDOMObjectToISupports(cur, native)) {
+            entries = nullptr;
+        } else if (IS_SLIM_WRAPPER(cur)) {
             native = static_cast<nsISupports*>(xpc_GetJSPrivate(cur));
             entries = GetOffsetsFromSlimWrapper(cur);
         } else {
-            MOZ_NOT_REACHED("what kind of wrapper is this?");
+            *pThisRef = nullptr;
+            return NS_ERROR_ILLEGAL_VALUE;
         }
 
         if (NS_SUCCEEDED(getNative(native, entries, cur, iid, ppThis, pThisRef, vp))) {
@@ -782,7 +646,7 @@ castNative(JSContext *cx,
         }
     }
 
-    *pThisRef = nsnull;
+    *pThisRef = nullptr;
     return NS_ERROR_XPC_BAD_OP_ON_WN_PROTO;
 }
 
@@ -797,35 +661,14 @@ xpc_qsUnwrapThisFromCcxImpl(XPCCallContext &ccx,
     if (!native)
         return xpc_qsThrow(ccx.GetJSContext(), NS_ERROR_XPC_HAS_BEEN_SHUTDOWN);
 
+    RootedObject obj(ccx, ccx.GetFlattenedJSObject());
     nsresult rv = getNative(native, GetOffsets(native, ccx.GetProto()),
-                            ccx.GetFlattenedJSObject(), iid, ppThis, pThisRef,
-                            vp);
+                            obj, iid, ppThis, pThisRef, vp);
     if (NS_FAILED(rv))
         return xpc_qsThrow(ccx.GetJSContext(), rv);
     return true;
 }
 
-JSObject*
-xpc_qsUnwrapObj(JS::Value v, nsISupports **ppArgRef, nsresult *rv)
-{
-    if (v.isNullOrUndefined()) {
-        *ppArgRef = nsnull;
-        *rv = NS_OK;
-        return nsnull;
-    }
-
-    if (!v.isObject()) {
-        *ppArgRef = nsnull;
-        *rv = ((v.isInt32() && v.toInt32() == 0)
-               ? NS_ERROR_XPC_BAD_CONVERT_JS_ZERO_ISNOT_NULL
-               : NS_ERROR_XPC_BAD_CONVERT_JS);
-        return nsnull;
-    }
-
-    *rv = NS_OK;
-    return &v.toObject();
-}
-
 nsresult
 xpc_qsUnwrapArgImpl(JSContext *cx,
                     jsval v,
@@ -835,9 +678,9 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
                     jsval *vp)
 {
     nsresult rv;
-    JSObject *src = xpc_qsUnwrapObj(v, ppArgRef, &rv);
+    RootedObject src(cx, xpc_qsUnwrapObj(v, ppArgRef, &rv));
     if (!src) {
-        *ppArg = nsnull;
+        *ppArg = nullptr;
 
         return rv;
     }
@@ -850,26 +693,18 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
 
     if (wrapper || obj2) {
         if (NS_FAILED(castNative(cx, wrapper, obj2, tearoff, iid, ppArg,
-                                 ppArgRef, vp, nsnull)))
+                                 ppArgRef, vp, nullptr)))
             return NS_ERROR_XPC_BAD_CONVERT_JS;
         return NS_OK;
     }
     // else...
     // Slow path.
 
-    // XXX E4X breaks the world. Don't try wrapping E4X objects!
-    // This hack can be removed (or changed accordingly) when the
-    // DOM <-> E4X bindings are complete, see bug 270553
-    if (JS_TypeOfValue(cx, OBJECT_TO_JSVAL(src)) == JSTYPE_XML) {
-        *ppArgRef = nsnull;
-        return NS_ERROR_XPC_BAD_CONVERT_JS;
-    }
-
     // Try to unwrap a slim wrapper.
     nsISupports *iface;
     if (XPCConvert::GetISupportsFromJSObject(src, &iface)) {
         if (!iface || NS_FAILED(iface->QueryInterface(iid, ppArg))) {
-            *ppArgRef = nsnull;
+            *ppArgRef = nullptr;
             return NS_ERROR_XPC_BAD_CONVERT_JS;
         }
 
@@ -880,15 +715,15 @@ xpc_qsUnwrapArgImpl(JSContext *cx,
     // Create the ccx needed for quick stubs.
     XPCCallContext ccx(JS_CALLER, cx);
     if (!ccx.IsValid()) {
-        *ppArgRef = nsnull;
+        *ppArgRef = nullptr;
         return NS_ERROR_XPC_BAD_CONVERT_JS;
     }
 
     nsRefPtr<nsXPCWrappedJS> wrappedJS;
-    rv = nsXPCWrappedJS::GetNewOrUsed(ccx, src, iid, nsnull,
+    rv = nsXPCWrappedJS::GetNewOrUsed(ccx, src, iid, nullptr,
                                       getter_AddRefs(wrappedJS));
     if (NS_FAILED(rv) || !wrappedJS) {
-        *ppArgRef = nsnull;
+        *ppArgRef = nullptr;
         return rv;
     }
 
@@ -909,7 +744,7 @@ xpc_qsJsvalToCharStr(JSContext *cx, jsval v, JSAutoByteString *bytes)
 {
     JSString *str;
 
-    JS_ASSERT(!bytes->ptr());
+    MOZ_ASSERT(!bytes->ptr());
     if (JSVAL_IS_STRING(v)) {
         str = JSVAL_TO_STRING(v);
     } else if (JSVAL_IS_VOID(v) || JSVAL_IS_NULL(v)) {
@@ -918,7 +753,7 @@ xpc_qsJsvalToCharStr(JSContext *cx, jsval v, JSAutoByteString *bytes)
         if (!(str = JS_ValueToString(cx, v)))
             return false;
     }
-    return !!bytes->encode(cx, str);
+    return !!bytes->encodeLatin1(cx, str);
 }
 
 JSBool
@@ -947,17 +782,6 @@ xpc_qsJsvalToWcharStr(JSContext *cx, jsval v, jsval *pval, const PRUnichar **pst
 
 namespace xpc {
 
-bool
-StringToJsval(JSContext *cx, nsAString &str, JS::Value *rval)
-{
-    // From the T_DOMSTRING case in XPCConvert::NativeData2JS.
-    if (str.IsVoid()) {
-        *rval = JSVAL_NULL;
-        return true;
-    }
-    return NonVoidStringToJsval(cx, str, rval);
-}
-
 bool
 NonVoidStringToJsval(JSContext *cx, nsAString &str, JS::Value *rval)
 {
@@ -981,7 +805,7 @@ xpc_qsStringToJsstring(JSContext *cx, nsString &str, JSString **rval)
 {
     // From the T_DOMSTRING case in XPCConvert::NativeData2JS.
     if (str.IsVoid()) {
-        *rval = nsnull;
+        *rval = nullptr;
         return true;
     }
 
@@ -1011,7 +835,7 @@ xpc_qsXPCOMObjectToJsval(XPCLazyCallContext &lccx, qsObjectHelper &aHelper,
     JSContext *cx = lccx.GetJSContext();
 
     nsresult rv;
-    if (!XPCConvert::NativeInterface2JSObject(lccx, rval, nsnull,
+    if (!XPCConvert::NativeInterface2JSObject(lccx, rval, nullptr,
                                               aHelper, iid, iface,
                                               true, &rv)) {
         // I can't tell if NativeInterface2JSObject throws JS exceptions
@@ -1061,4 +885,12 @@ xpc_qsAssertContextOK(JSContext *cx)
     // This is what we're actually trying to assert here.
     NS_ASSERTION(cx == topJSContext, "wrong context on XPCJSContextStack!");
 }
+
+void
+xpcObjectHelper::AssertGetClassInfoResult()
+{
+    MOZ_ASSERT(mXPCClassInfo ||
+               static_cast<nsINode*>(GetCanonical())->IsDOMBinding(),
+               "GetClassInfo() should only return null for new DOM bindings!");
+}
 #endif
