@@ -5,14 +5,12 @@
 
 #include "nsXULTooltipListener.h"
 
+#include "nsDOMEvent.h"
 #include "nsIDOMMouseEvent.h"
-#include "nsIDOMEventTarget.h"
-#include "nsIDOMDocument.h"
 #include "nsIDOMXULDocument.h"
 #include "nsIDOMXULElement.h"
 #include "nsIDocument.h"
 #include "nsGkAtoms.h"
-#include "nsIFrame.h"
 #include "nsIPopupBoxObject.h"
 #include "nsMenuPopupFrame.h"
 #include "nsIServiceManager.h"
@@ -22,7 +20,6 @@
 #include "nsITreeView.h"
 #endif
 #include "nsGUIEvent.h"
-#include "nsIPrivateDOMEvent.h"
 #include "nsIScriptContext.h"
 #include "nsPIDOMWindow.h"
 #ifdef MOZ_XUL
@@ -34,10 +31,10 @@
 #include "mozilla/LookAndFeel.h"
 #include "mozilla/dom/Element.h"
 
-
 using namespace mozilla;
+using namespace mozilla::dom;
 
-nsXULTooltipListener* nsXULTooltipListener::mInstance = nsnull;
+nsXULTooltipListener* nsXULTooltipListener::mInstance = nullptr;
 
 //////////////////////////////////////////////////////////////////////////
 //// nsISupports
@@ -58,7 +55,7 @@ nsXULTooltipListener::nsXULTooltipListener()
                                   "browser.chrome.toolbar_tips");
 
     // Call the pref callback to initialize our state.
-    ToolbarTipsPrefChanged("browser.chrome.toolbar_tips", nsnull);
+    ToolbarTipsPrefChanged("browser.chrome.toolbar_tips", nullptr);
   }
 }
 
@@ -90,7 +87,7 @@ nsXULTooltipListener::MouseOut(nsIDOMEvent* aEvent)
   nsCOMPtr<nsIContent> currentTooltip = do_QueryReferent(mCurrentTooltip);
   if (mTooltipTimer && !currentTooltip) {
     mTooltipTimer->Cancel();
-    mTooltipTimer = nsnull;
+    mTooltipTimer = nullptr;
     return;
   }
 
@@ -104,9 +101,8 @@ nsXULTooltipListener::MouseOut(nsIDOMEvent* aEvent)
   // hide the tooltip
   if (currentTooltip) {
     // which node did the mouse leave?
-    nsCOMPtr<nsIDOMEventTarget> eventTarget;
-    aEvent->GetTarget(getter_AddRefs(eventTarget));
-    nsCOMPtr<nsIDOMNode> targetNode(do_QueryInterface(eventTarget));
+    nsCOMPtr<nsIDOMNode> targetNode = do_QueryInterface(
+      aEvent->InternalDOMEvent()->GetTarget());
 
     nsXULPopupManager* pm = nsXULPopupManager::GetInstance();
     if (pm) {
@@ -119,7 +115,7 @@ nsXULTooltipListener::MouseOut(nsIDOMEvent* aEvent)
         // reset special tree tracking
         if (mIsSourceTree) {
           mLastTreeRow = -1;
-          mLastTreeCol = nsnull;
+          mLastTreeCol = nullptr;
         }
       }
     }
@@ -140,7 +136,7 @@ nsXULTooltipListener::MouseMove(nsIDOMEvent* aEvent)
   nsCOMPtr<nsIDOMMouseEvent> mouseEvent(do_QueryInterface(aEvent));
   if (!mouseEvent)
     return;
-  PRInt32 newMouseX, newMouseY;
+  int32_t newMouseX, newMouseY;
   mouseEvent->GetScreenX(&newMouseX);
   mouseEvent->GetScreenY(&newMouseY);
 
@@ -159,10 +155,8 @@ nsXULTooltipListener::MouseMove(nsIDOMEvent* aEvent)
   mMouseScreenX = newMouseX;
   mMouseScreenY = newMouseY;
 
-  nsCOMPtr<nsIDOMEventTarget> currentTarget;
-  aEvent->GetCurrentTarget(getter_AddRefs(currentTarget));
-
-  nsCOMPtr<nsIContent> sourceContent = do_QueryInterface(currentTarget);
+  nsCOMPtr<nsIContent> sourceContent = do_QueryInterface(
+    aEvent->InternalDOMEvent()->GetCurrentTarget());
   mSourceNode = do_GetWeakReference(sourceContent);
 #ifdef MOZ_XUL
   mIsSourceTree = sourceContent->Tag() == nsGkAtoms::treechildren;
@@ -179,8 +173,7 @@ nsXULTooltipListener::MouseMove(nsIDOMEvent* aEvent)
   // showing and the tooltip hasn't been displayed since the mouse entered
   // the node, then start the timer to show the tooltip.
   if (!currentTooltip && !mTooltipShownOnce) {
-    nsCOMPtr<nsIDOMEventTarget> eventTarget;
-    aEvent->GetTarget(getter_AddRefs(eventTarget));
+    nsCOMPtr<EventTarget> eventTarget = aEvent->InternalDOMEvent()->GetTarget();
 
     // don't show tooltips attached to elements outside of a menu popup
     // when hovering over an element inside it. The popupsinherittooltip
@@ -195,7 +188,7 @@ nsXULTooltipListener::MouseMove(nsIDOMEvent* aEvent)
             (tag == nsGkAtoms::menupopup ||
              tag == nsGkAtoms::panel ||
              tag == nsGkAtoms::tooltip)) {
-          mSourceNode = nsnull;
+          mSourceNode = nullptr;
           return;
         }
 
@@ -212,8 +205,8 @@ nsXULTooltipListener::MouseMove(nsIDOMEvent* aEvent)
             LookAndFeel::GetInt(LookAndFeel::eIntID_TooltipDelay, 500),
             nsITimer::TYPE_ONE_SHOT);
         if (NS_FAILED(rv)) {
-          mTargetNode = nsnull;
-          mSourceNode = nsnull;
+          mTargetNode = nullptr;
+          mSourceNode = nullptr;
         }
       }
     }
@@ -294,7 +287,7 @@ nsXULTooltipListener::ToolbarTipsPrefChanged(const char *aPref,
 //// nsXULTooltipListener
 
 bool nsXULTooltipListener::sShowTooltips = false;
-PRUint32 nsXULTooltipListener::sTooltipListenerCount = 0;
+uint32_t nsXULTooltipListener::sTooltipListenerCount = 0;
 
 nsresult
 nsXULTooltipListener::AddTooltipSupport(nsIContent* aNode)
@@ -306,6 +299,10 @@ nsXULTooltipListener::AddTooltipSupport(nsIContent* aNode)
                                 false, false);
   aNode->AddSystemEventListener(NS_LITERAL_STRING("mousemove"), this,
                                 false, false);
+  aNode->AddSystemEventListener(NS_LITERAL_STRING("mousedown"), this,
+                                false, false);
+  aNode->AddSystemEventListener(NS_LITERAL_STRING("mouseup"), this,
+                                false, false);
   aNode->AddSystemEventListener(NS_LITERAL_STRING("dragstart"), this,
                                 true, false);
 
@@ -320,6 +317,8 @@ nsXULTooltipListener::RemoveTooltipSupport(nsIContent* aNode)
 
   aNode->RemoveSystemEventListener(NS_LITERAL_STRING("mouseout"), this, false);
   aNode->RemoveSystemEventListener(NS_LITERAL_STRING("mousemove"), this, false);
+  aNode->RemoveSystemEventListener(NS_LITERAL_STRING("mousedown"), this, false);
+  aNode->RemoveSystemEventListener(NS_LITERAL_STRING("mouseup"), this, false);
   aNode->RemoveSystemEventListener(NS_LITERAL_STRING("dragstart"), this, true);
 
   return NS_OK;
@@ -337,25 +336,23 @@ nsXULTooltipListener::CheckTreeBodyMove(nsIDOMMouseEvent* aMouseEvent)
   nsCOMPtr<nsIBoxObject> bx;
   nsIDocument* doc = sourceNode->GetDocument();
   if (doc) {
-    nsCOMPtr<nsIDOMElement> docElement = do_QueryInterface(doc->GetRootElement());
-    if (docElement) {
-      doc->GetBoxObjectFor(docElement, getter_AddRefs(bx));
-    }
+    ErrorResult ignored;
+    bx = doc->GetBoxObjectFor(doc->GetRootElement(), ignored);
   }
 
   nsCOMPtr<nsITreeBoxObject> obx;
   GetSourceTreeBoxObject(getter_AddRefs(obx));
   if (bx && obx) {
-    PRInt32 x, y;
+    int32_t x, y;
     aMouseEvent->GetScreenX(&x);
     aMouseEvent->GetScreenY(&y);
 
-    PRInt32 row;
+    int32_t row;
     nsCOMPtr<nsITreeColumn> col;
-    nsCAutoString obj;
+    nsAutoCString obj;
 
     // subtract off the documentElement's boxObject
-    PRInt32 boxX, boxY;
+    int32_t boxX, boxY;
     bx->GetScreenX(&boxX);
     bx->GetScreenY(&boxY);
     x -= boxX;
@@ -401,13 +398,13 @@ nsXULTooltipListener::ShowTooltip()
 #ifdef MOZ_XUL
       if (!mIsSourceTree) {
         mLastTreeRow = -1;
-        mLastTreeCol = nsnull;
+        mLastTreeCol = nullptr;
       }
 #endif
 
       mCurrentTooltip = do_GetWeakReference(tooltipNode);
       LaunchTooltip();
-      mTargetNode = nsnull;
+      mTargetNode = nullptr;
 
       nsCOMPtr<nsIContent> currentTooltip = do_QueryReferent(mCurrentTooltip);
       if (!currentTooltip)
@@ -435,7 +432,7 @@ nsXULTooltipListener::ShowTooltip()
         doc->AddSystemEventListener(NS_LITERAL_STRING("keydown"),
                                     this, true);
       }
-      mSourceNode = nsnull;
+      mSourceNode = nullptr;
     }
   }
 
@@ -448,14 +445,14 @@ nsXULTooltipListener::ShowTooltip()
 #ifdef DEBUG_crap
 static void
 GetTreeCellCoords(nsITreeBoxObject* aTreeBox, nsIContent* aSourceNode, 
-                  PRInt32 aRow, nsITreeColumn* aCol, PRInt32* aX, PRInt32* aY)
+                  int32_t aRow, nsITreeColumn* aCol, int32_t* aX, int32_t* aY)
 {
-  PRInt32 junk;
+  int32_t junk;
   aTreeBox->GetCoordsForCellItem(aRow, aCol, EmptyCString(), aX, aY, &junk, &junk);
   nsCOMPtr<nsIDOMXULElement> xulEl(do_QueryInterface(aSourceNode));
   nsCOMPtr<nsIBoxObject> bx;
   xulEl->GetBoxObject(getter_AddRefs(bx));
-  PRInt32 myX, myY;
+  int32_t myX, myY;
   bx->GetX(&myX);
   bx->GetY(&myY);
   *aX += myX;
@@ -465,7 +462,7 @@ GetTreeCellCoords(nsITreeBoxObject* aTreeBox, nsIContent* aSourceNode,
 
 static void
 SetTitletipLabel(nsITreeBoxObject* aTreeBox, nsIContent* aTooltip,
-                 PRInt32 aRow, nsITreeColumn* aCol)
+                 int32_t aRow, nsITreeColumn* aCol)
 {
   nsCOMPtr<nsITreeView> view;
   aTreeBox->GetView(getter_AddRefs(view));
@@ -498,9 +495,9 @@ nsXULTooltipListener::LaunchTooltip()
       // Because of mutation events, currentTooltip can be null.
       return;
     }
-    currentTooltip->SetAttr(nsnull, nsGkAtoms::titletip, NS_LITERAL_STRING("true"), true);
+    currentTooltip->SetAttr(kNameSpaceID_None, nsGkAtoms::titletip, NS_LITERAL_STRING("true"), true);
   } else {
-    currentTooltip->UnsetAttr(nsnull, nsGkAtoms::titletip, true);
+    currentTooltip->UnsetAttr(kNameSpaceID_None, nsGkAtoms::titletip, true);
   }
   if (!(currentTooltip = do_QueryReferent(mCurrentTooltip))) {
     // Because of mutation events, currentTooltip can be null.
@@ -514,7 +511,7 @@ nsXULTooltipListener::LaunchTooltip()
 
     // Clear the current tooltip if the popup was not opened successfully.
     if (!pm->IsPopupOpen(currentTooltip))
-      mCurrentTooltip = nsnull;
+      mCurrentTooltip = nullptr;
   }
 #endif
 
@@ -539,9 +536,9 @@ nsXULTooltipListener::HideTooltip()
 static void
 GetImmediateChild(nsIContent* aContent, nsIAtom *aTag, nsIContent** aResult) 
 {
-  *aResult = nsnull;
-  PRUint32 childCount = aContent->GetChildCount();
-  for (PRUint32 i = 0; i < childCount; i++) {
+  *aResult = nullptr;
+  uint32_t childCount = aContent->GetChildCount();
+  for (uint32_t i = 0; i < childCount; i++) {
     nsIContent *child = aContent->GetChildAt(i);
 
     if (child->Tag() == aTag) {
@@ -630,22 +627,24 @@ nsXULTooltipListener::FindTooltip(nsIContent* aTarget, nsIContent** aTooltip)
 nsresult
 nsXULTooltipListener::GetTooltipFor(nsIContent* aTarget, nsIContent** aTooltip)
 {
-  *aTooltip = nsnull;
+  *aTooltip = nullptr;
   nsCOMPtr<nsIContent> tooltip;
   nsresult rv = FindTooltip(aTarget, getter_AddRefs(tooltip));
   if (NS_FAILED(rv) || !tooltip) {
     return rv;
   }
 
+#ifdef MOZ_XUL
   // Submenus can't be used as tooltips, see bug 288763.
   nsIContent* parent = tooltip->GetParent();
   if (parent) {
-    nsIFrame* frame = parent->GetPrimaryFrame();
-    if (frame && frame->GetType() == nsGkAtoms::menuFrame) {
+    nsMenuFrame* menu = do_QueryFrame(parent->GetPrimaryFrame());
+    if (menu) {
       NS_WARNING("Menu cannot be used as a tooltip");
       return NS_ERROR_FAILURE;
     }
   }
+#endif
 
   tooltip.swap(*aTooltip);
   return rv;
@@ -669,21 +668,19 @@ nsXULTooltipListener::DestroyTooltip()
       doc->RemoveSystemEventListener(NS_LITERAL_STRING("keydown"), this, true);
     }
 
-    // remove the popuphidden listener from tooltip
-    nsCOMPtr<nsIDOMEventTarget> evtTarget(do_QueryInterface(currentTooltip));
-
     // release tooltip before removing listener to prevent our destructor from
     // being called recursively (bug 120863)
-    mCurrentTooltip = nsnull;
+    mCurrentTooltip = nullptr;
 
-    evtTarget->RemoveEventListener(NS_LITERAL_STRING("popuphiding"), this, false);
+    // remove the popuphidden listener from tooltip
+    currentTooltip->RemoveEventListener(NS_LITERAL_STRING("popuphiding"), this, false);
   }
-  
+
   // kill any ongoing timers
   KillTooltipTimer();
-  mSourceNode = nsnull;
+  mSourceNode = nullptr;
 #ifdef MOZ_XUL
-  mLastTreeCol = nsnull;
+  mLastTreeCol = nullptr;
 #endif
 
   return NS_OK;
@@ -694,8 +691,8 @@ nsXULTooltipListener::KillTooltipTimer()
 {
   if (mTooltipTimer) {
     mTooltipTimer->Cancel();
-    mTooltipTimer = nsnull;
-    mTargetNode = nsnull;
+    mTooltipTimer = nullptr;
+    mTargetNode = nullptr;
   }
 }
 
@@ -711,7 +708,7 @@ nsXULTooltipListener::sTooltipCallback(nsITimer *aTimer, void *aListener)
 nsresult
 nsXULTooltipListener::GetSourceTreeBoxObject(nsITreeBoxObject** aBoxObject)
 {
-  *aBoxObject = nsnull;
+  *aBoxObject = nullptr;
 
   nsCOMPtr<nsIContent> sourceNode = do_QueryReferent(mSourceNode);
   if (mIsSourceTree && sourceNode) {
