@@ -19,6 +19,7 @@
 #ifdef MOZILLA_INTERNAL_API
 #include "MediaStreamList.h"
 #include "nsIScriptGlobalObject.h"
+#include "mozilla/Preferences.h"
 #include "jsapi.h"
 #endif
 
@@ -131,7 +132,8 @@ PeerConnectionMedia::PeerConnectionMedia(PeerConnectionImpl *parent)
       mMainThread(mParent->GetMainThread()),
       mSTSThread(mParent->GetSTSThread()) {}
 
-nsresult PeerConnectionMedia::Init(const std::vector<NrIceStunServer>& stun_servers)
+nsresult PeerConnectionMedia::Init(const std::vector<NrIceStunServer>& stun_servers,
+                                   const std::vector<NrIceTurnServer>& turn_servers)
 {
   // TODO(ekr@rtfm.com): need some way to set not offerer later
   // Looks like a bug in the NrIceCtx API.
@@ -145,6 +147,20 @@ nsresult PeerConnectionMedia::Init(const std::vector<NrIceStunServer>& stun_serv
     CSFLogError(logTag, "%s: Failed to set stun servers", __FUNCTION__);
     return rv;
   }
+  // Give us a way to globally turn off TURN support
+#ifdef MOZILLA_INTERNAL_API
+  bool disabled = Preferences::GetBool("media.peerconnection.turn.disable", false);
+#else
+  bool disabled = false;
+#endif
+  if (!disabled) {
+    if (NS_FAILED(rv = mIceCtx->SetTurnServers(turn_servers))) {
+      CSFLogError(logTag, "%s: Failed to set turn servers", __FUNCTION__);
+      return rv;
+    }
+  } else if (turn_servers.size() != 0) {
+    CSFLogError(logTag, "%s: Setting turn servers disabled", __FUNCTION__);
+  }
   if (NS_FAILED(rv = mDNSResolver->Init())) {
     CSFLogError(logTag, "%s: Failed to initialize dns resolver", __FUNCTION__);
     return rv;
