@@ -743,7 +743,11 @@ WebRtc_Word32 AudioDeviceAndroidOpenSLES::InitPlayout() {
   // Setup the data source structure for the buffer queue.
   player_pcm_.formatType = SL_DATAFORMAT_PCM;
   player_pcm_.numChannels = N_PLAY_CHANNELS;
-  player_pcm_.samplesPerSec = speaker_sampling_rate_ * 1000;
+  if (speaker_sampling_rate_ == 44000) {
+      player_pcm_.samplesPerSec = 44100 * 1000;
+  } else {
+    player_pcm_.samplesPerSec = speaker_sampling_rate_ * 1000;
+  }
   player_pcm_.bitsPerSample = SL_PCMSAMPLEFORMAT_FIXED_16;
   player_pcm_.containerSize = SL_PCMSAMPLEFORMAT_FIXED_16;
   if (1 == player_pcm_.numChannels) {
@@ -863,7 +867,11 @@ WebRtc_Word32 AudioDeviceAndroidOpenSLES::InitRecording() {
   // Setup the format of the content in the buffer queue
   record_pcm_.formatType = SL_DATAFORMAT_PCM;
   record_pcm_.numChannels = N_REC_CHANNELS;
-  record_pcm_.samplesPerSec = speaker_sampling_rate_ * 1000;
+  if (speaker_sampling_rate_ == 44000) {
+    record_pcm_.samplesPerSec = 44100 * 1000;
+  } else {
+    record_pcm_.samplesPerSec = speaker_sampling_rate_ * 1000;
+  }
   record_pcm_.bitsPerSample = SL_PCMSAMPLEFORMAT_FIXED_16;
   record_pcm_.containerSize = SL_PCMSAMPLEFORMAT_FIXED_16;
   if (1 == record_pcm_.numChannels) {
