@@ -10,12 +10,15 @@ import org.mozilla.gecko.db.BrowserContract.Bookmarks;
 import org.mozilla.gecko.db.BrowserContract.Combined;
 import org.mozilla.gecko.db.BrowserDB;
 import org.mozilla.gecko.db.BrowserDB.URLColumns;
+import org.mozilla.gecko.gfx.BitmapUtils;
 import org.mozilla.gecko.util.GamepadUtils;
 import org.mozilla.gecko.util.ThreadUtils;
+import org.mozilla.gecko.widget.FaviconView;
 
 import android.app.Activity;
 import android.content.Context;
 import android.database.Cursor;
+import android.graphics.Bitmap;
 import android.os.AsyncTask;
 import android.util.Log;
 import android.util.Pair;
@@ -26,7 +29,6 @@ import android.view.MenuInflater;
 import android.view.View;
 import android.view.ViewGroup;
 import android.widget.AdapterView;
-import android.widget.ImageView;
 import android.widget.ListView;
 import android.widget.SimpleCursorAdapter;
 import android.widget.TextView;
@@ -59,7 +61,7 @@ public class BookmarksTab extends AwesomeBarTab {
     @Override
     public View getView() {
         if (mView == null) {
-            mView = (LayoutInflater.from(mContext).inflate(R.layout.awesomebar_list, null));
+            mView = new ListView(mContext, null);
             ((Activity)mContext).registerForContextMenu(mView);
             mView.setTag(TAG);
             mView.setOnTouchListener(mListListener);
@@ -93,6 +95,7 @@ public class BookmarksTab extends AwesomeBarTab {
 
     @Override
     public void destroy() {
+        super.destroy();
         // Can't use getters for adapter. It will create one if null.
         if (mCursorAdapter != null && mView != null) {
             ListView list = (ListView)mView;
@@ -201,18 +204,13 @@ public class BookmarksTab extends AwesomeBarTab {
         }
 
         // Otherwise, just open the URL
-        AwesomeBarTabs.OnUrlOpenListener listener = getUrlListener();
-        if (listener == null) {
-            return;
-        }
-
         String url = cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.URL));
         String title = cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.TITLE));
         long parentId = cursor.getLong(cursor.getColumnIndexOrThrow(Bookmarks.PARENT));
         if (parentId == Bookmarks.FIXED_READING_LIST_ID) {
             url = ReaderModeUtils.getAboutReaderForUrl(url, true);
         }
-        listener.onUrlOpen(url, title);
+        sendToListener(url, title);
     }
 
     private class BookmarksListAdapter extends SimpleCursorAdapter {
@@ -325,7 +323,7 @@ public class BookmarksTab extends AwesomeBarTab {
 
                 viewHolder = new AwesomeEntryViewHolder();
                 viewHolder.titleView = (TextView) convertView.findViewById(R.id.title);
-                viewHolder.faviconView = (ImageView) convertView.findViewById(R.id.favicon);
+                viewHolder.faviconView = (FaviconView) convertView.findViewById(R.id.favicon);
 
                 if (viewType == VIEW_TYPE_ITEM)
                     viewHolder.urlView = (TextView) convertView.findViewById(R.id.url);
@@ -341,8 +339,18 @@ public class BookmarksTab extends AwesomeBarTab {
 
             if (viewType == VIEW_TYPE_ITEM) {
                 updateTitle(viewHolder.titleView, cursor);
-                updateUrl(viewHolder.urlView, cursor);
-                updateFavicon(viewHolder.faviconView, cursor);
+                updateUrl(viewHolder, cursor);
+
+                byte[] b = cursor.getBlob(cursor.getColumnIndexOrThrow(URLColumns.FAVICON));
+                Bitmap favicon = null;
+                if (b != null && b.length > 0) {
+                    Bitmap bitmap = BitmapUtils.decodeByteArray(b);
+                    if (bitmap != null) {
+                        favicon = Favicons.getInstance().scaleImage(bitmap);
+                    }
+                }
+                String url = cursor.getString(cursor.getColumnIndexOrThrow(URLColumns.URL));
+                updateFavicon(viewHolder.faviconView, favicon, url);
             } else {
                 viewHolder.titleView.setText(getFolderTitle(position));
             }
@@ -453,12 +461,10 @@ public class BookmarksTab extends AwesomeBarTab {
         if (subject == null)
             return subject;
 
-        MenuInflater inflater = new MenuInflater(mContext);
-        inflater.inflate(R.menu.awesomebar_contextmenu, menu);
+        setupMenu(menu, subject);
         
         menu.findItem(R.id.remove_history).setVisible(false);
         menu.findItem(R.id.open_in_reader).setVisible(false);
-        menu.setHeaderTitle(subject.title);
 
         return subject;
     }
