@@ -44,7 +44,6 @@ SessionStore.prototype = {
 
   _windows: {},
   _lastSaveTime: 0,
-  _lastSessionTime: 0,
   _interval: 10000,
   _maxTabsUndo: 1,
   _shouldRestore: false,
@@ -53,43 +52,16 @@ SessionStore.prototype = {
     // Get file references
     this._sessionFile = Services.dirsvc.get("ProfD", Ci.nsILocalFile);
     this._sessionFileBackup = this._sessionFile.clone();
-    this._sessionCache = this._sessionFile.clone();
     this._sessionFile.append("sessionstore.js");
     this._sessionFileBackup.append("sessionstore.bak");
-    this._sessionCache.append("sessionstoreCache");
 
     this._loadState = STATE_STOPPED;
 
-    try {
-      if (this._sessionFile.exists()) {
-        // We move sessionstore.js -> sessionstore.bak on quit, so the
-        // existence of sessionstore.js indicates a crash
-        this._lastSessionTime = this._sessionFile.lastModifiedTime;
-        let delta = Date.now() - this._lastSessionTime;
-        let timeout = Services.prefs.getIntPref("browser.sessionstore.resume_from_crash_timeout");
-
-        // Disable crash recovery if we have exceeded the timeout
-        this._shouldRestore = (delta <= (timeout * 60000));
-        if (!this._shouldRestore) {
-          this._sessionFile.clone().moveTo(null, this._sessionFileBackup.leafName);
-        }
-      }
-
-      if (!this._sessionCache.exists() || !this._sessionCache.isDirectory())
-        this._sessionCache.create(Ci.nsIFile.DIRECTORY_TYPE, 0700);
-    } catch (ex) {
-      Cu.reportError(ex); // file was write-locked?
-    }
-
     this._interval = Services.prefs.getIntPref("browser.sessionstore.interval");
     this._maxTabsUndo = Services.prefs.getIntPref("browser.sessionstore.max_tabs_undo");
 
-    // Disable crash recovery if it has been turned off
-    if (!Services.prefs.getBoolPref("browser.sessionstore.resume_from_crash"))
-      this._shouldRestore = false;
-
     // Do we need to restore session just this once, in case of a restart?
-    if (Services.prefs.getBoolPref("browser.sessionstore.resume_session_once")) {
+    if (this._sessionFileBackup.exists() && Services.prefs.getBoolPref("browser.sessionstore.resume_session_once")) {
       Services.prefs.setBoolPref("browser.sessionstore.resume_session_once", false);
       this._shouldRestore = true;
     }
@@ -107,36 +79,11 @@ SessionStore.prototype = {
       } catch (ex) { dump(ex + '\n'); } // couldn't remove the file - what now?
     }
 
-    this._clearCache();
   },
 
-  _clearCache: function ss_clearCache() {
-    // First, let's get a list of files we think should be active
-    let activeFiles = [];
-    this._forEachBrowserWindow(function(aWindow) {
-      let tabs = aWindow.BrowserApp.tabs;
-      for (let i = 0; i < tabs.length; i++) {
-        let browser = tabs[i].browser;
-        if (browser.__SS_extdata && "thumbnail" in browser.__SS_extdata)
-          activeFiles.push(browser.__SS_extdata.thumbnail);
-      }
-    });
-
-    // Now, let's find the stale files in the cache folder
-    let staleFiles = [];
-    let cacheFiles = this._sessionCache.directoryEntries;
-    while (cacheFiles.hasMoreElements()) {
-      let file = cacheFiles.getNext().QueryInterface(Ci.nsILocalFile);
-      let fileURI = Services.io.newFileURI(file);
-      if (activeFiles.indexOf(fileURI) == -1)
-        staleFiles.push(file);
-    }
-
-    // Remove the stale files in a separate step to keep the enumerator from
-    // messing up if we remove the files as we collect them.
-    staleFiles.forEach(function(aFile) {
-      aFile.remove(false);
-    })
+  _sendMessageToJava: function (aMsg) {
+    let data = Cc["@mozilla.org/android/bridge;1"].getService(Ci.nsIAndroidBridge).handleGeckoMessage(JSON.stringify(aMsg));
+    return JSON.parse(data);
   },
 
   observe: function ss_observe(aSubject, aTopic, aData) {
@@ -152,18 +99,20 @@ SessionStore.prototype = {
         observerService.addObserver(this, "quit-application-requested", true);
         observerService.addObserver(this, "quit-application-granted", true);
         observerService.addObserver(this, "quit-application", true);
+        observerService.addObserver(this, "Session:Restore", true);
         break;
       case "final-ui-startup":
         observerService.removeObserver(this, "final-ui-startup");
         this.init();
         break;
-      case "domwindowopened":
+      case "domwindowopened": {
         let window = aSubject;
         window.addEventListener("load", function() {
           self.onWindowOpen(window);
           window.removeEventListener("load", arguments.callee, false);
         }, false);
         break;
+      }
       case "domwindowclosed": // catch closed windows
         this.onWindowClose(aSubject);
         break;
@@ -217,6 +166,7 @@ SessionStore.prototype = {
         observerService.removeObserver(this, "quit-application-requested");
         observerService.removeObserver(this, "quit-application-granted");
         observerService.removeObserver(this, "quit-application");
+        observerService.removeObserver(this, "Session:Restore");
 
         // If a save has been queued, kill the timer and save state now
         if (this._saveTimer) {
@@ -250,6 +200,44 @@ SessionStore.prototype = {
         this._saveTimer = null;
         this.saveState();
         break;
+      case "Session:Restore": {
+        if (aData) {
+          // Be ready to handle any restore failures by making sure we have a valid tab opened
+          let window = Services.wm.getMostRecentWindow("navigator:browser");
+          let restoreCleanup = {
+            observe: function (aSubject, aTopic, aData) {
+              Services.obs.removeObserver(restoreCleanup, "sessionstore-windows-restored");
+
+              if (window.BrowserApp.tabs.length == 0) {
+                window.BrowserApp.addTab("about:home", {
+                  showProgress: false,
+                  selected: true
+                });
+              }
+
+              // Let Java know we're done restoring tabs so tabs added after this can be animated
+              this._sendMessageToJava({
+                type: "Session:RestoreEnd"
+              });
+            }.bind(this)
+          };
+          Services.obs.addObserver(restoreCleanup, "sessionstore-windows-restored", false);
+
+          // Do a restore, triggered by Java
+          let data = JSON.parse(aData);
+          this.restoreLastSession(data.restoringOOM, data.sessionString);
+        } else if (this._shouldRestore) {
+          // Do a restore triggered by Gecko (e.g., if
+          // browser.sessionstore.resume_session_once is true). In these cases,
+          // our Java front-end doesn't know we're doing a restore, so it has
+          // already opened an about:home tab.
+          this.restoreLastSession(false, null);
+        } else {
+          // Not doing a restore; just send restore message
+          Services.obs.notifyObservers(null, "sessionstore-windows-restored", "");
+        }
+        break;
+      }
     }
   },
 
@@ -297,12 +285,6 @@ SessionStore.prototype = {
     if (this._loadState == STATE_STOPPED) {
       this._loadState = STATE_RUNNING;
       this._lastSaveTime = Date.now();
-
-      // Nothing to restore, notify observers things are complete
-      if (!this._shouldRestore) {
-        this._clearCache();
-        Services.obs.notifyObservers(null, "sessionstore-windows-restored", "");
-      }
     }
 
     // Add tab change listeners to all already existing tabs
@@ -400,11 +382,19 @@ SessionStore.prototype = {
     } else {
       // Serialize the tab data
       let entries = [];
+      let index = history.index + 1;
       for (let i = 0; i < history.count; i++) {
-        let entry = this._serializeHistoryEntry(history.getEntryAtIndex(i, false));
+        let historyEntry = history.getEntryAtIndex(i, false);
+        // Don't try to restore wyciwyg URLs
+        if (historyEntry.URI.schemeIs("wyciwyg")) {
+          // Adjust the index to account for skipped history entries
+          if (i <= history.index)
+            index--;
+          continue;
+        }
+        let entry = this._serializeHistoryEntry(historyEntry);
         entries.push(entry);
       }
-      let index = history.index + 1;
       let data = { entries: entries, index: index };
 
       delete aBrowser.__SS_data;
@@ -463,7 +453,41 @@ SessionStore.prototype = {
 
   saveState: function ss_saveState() {
     let data = this._getCurrentState();
-    this._writeFile(this._sessionFile, JSON.stringify(data));
+    let normalData = { windows: [] };
+    let privateData = { windows: [] };
+
+    for (let winIndex = 0; winIndex < data.windows.length; ++winIndex) {
+      let win = data.windows[winIndex];
+      let normalWin = {};
+      for (let prop in win) {
+        normalWin[prop] = data[prop];
+      }
+      normalWin.tabs = [];
+      normalData.windows.push(normalWin);
+      privateData.windows.push({ tabs: [] });
+
+      // Split the session data into private and non-private data objects.
+      // Non-private session data will be saved to disk, and private session
+      // data will be sent to Java for Android to hold it in memory.
+      for (let i = 0; i < win.tabs.length; ++i) {
+        let tab = win.tabs[i];
+        let savedWin = tab.isPrivate ? privateData.windows[winIndex] : normalData.windows[winIndex];
+        savedWin.tabs.push(tab);
+        if (win.selected == i + 1) {
+          savedWin.selected = savedWin.tabs.length;
+        }
+      }
+    }
+
+    // Write only non-private data to disk
+    this._writeFile(this._sessionFile, JSON.stringify(normalData));
+
+    // If we have private data, send it to Java; otherwise, send null to
+    // indicate that there is no private data
+    this._sendMessageToJava({
+      type: "PrivateBrowsing:Data",
+      session: (privateData.windows[0].tabs.length > 0) ? JSON.stringify(privateData) : null
+    });
 
     this._lastSaveTime = Date.now();
   },
@@ -475,9 +499,10 @@ SessionStore.prototype = {
     });
 
     let data = { windows: [] };
-    let index;
-    for (index in this._windows)
+    for (let index in this._windows) {
       data.windows.push(this._windows[index]);
+    }
+
     return data;
   },
 
@@ -486,13 +511,14 @@ SessionStore.prototype = {
     if (aBrowser.__SS_restore)
       return;
 
-    let aHistory = aHistory || { entries: [{ url: aBrowser.currentURI.spec, title: aBrowser.contentTitle }], index: 1 };
+    aHistory = aHistory || { entries: [{ url: aBrowser.currentURI.spec, title: aBrowser.contentTitle }], index: 1 };
 
     let tabData = {};
     tabData.entries = aHistory.entries;
     tabData.index = aHistory.index;
     tabData.attributes = { image: aBrowser.mIconURL };
     tabData.desktopMode = aWindow.BrowserApp.getTabForBrowser(aBrowser).desktopMode;
+    tabData.isPrivate = aBrowser.docShell.QueryInterface(Ci.nsILoadContext).usePrivateBrowsing;
 
     aBrowser.__SS_data = tabData;
   },
@@ -637,19 +663,21 @@ SessionStore.prototype = {
       return entry;
 
     if (aEntry.childCount > 0) {
-      entry.children = [];
+      let children = [];
       for (let i = 0; i < aEntry.childCount; i++) {
         let child = aEntry.GetChildAt(i);
-        if (child)
-          entry.children.push(this._serializeHistoryEntry(child));
-        else // to maintain the correct frame order, insert a dummy entry
-          entry.children.push({ url: "about:blank" });
 
-        // don't try to restore framesets containing wyciwyg URLs (cf. bug 424689 and bug 450595)
-        if (/^wyciwyg:\/\//.test(entry.children[i].url)) {
-          delete entry.children;
-          break;
+        if (child) {
+          // don't try to restore framesets containing wyciwyg URLs (cf. bug 424689 and bug 450595)
+          if (child.URI.schemeIs("wyciwyg")) {
+            children = [];
+            break;
+          }
+          children.push(this._serializeHistoryEntry(child));
         }
+
+        if (children.length)
+          entry.children = children;
       }
     }
 
@@ -782,8 +810,69 @@ SessionStore.prototype = {
   },
 
   getBrowserState: function ss_getBrowserState() {
-    let data = this._getCurrentState();
-    return JSON.stringify(data);
+    return this._getCurrentState();
+  },
+
+  _restoreWindow: function ss_restoreWindow(aData) {
+    let state;
+    try {
+      state = JSON.parse(aData);
+    } catch (e) {
+      Cu.reportError("SessionStore: invalid session JSON");
+      return false;
+    }
+
+    // To do a restore, we must have at least one window with one tab
+    if (!state || state.windows.length == 0 || !state.windows[0].tabs || state.windows[0].tabs.length == 0) {
+      Cu.reportError("SessionStore: no tabs to restore");
+      return false;
+    }
+
+    let window = Services.wm.getMostRecentWindow("navigator:browser");
+
+    let tabs = state.windows[0].tabs;
+    let selected = state.windows[0].selected;
+    if (selected == null || selected > tabs.length) // Clamp the selected index if it's bogus
+      selected = 1;
+
+    for (let i = 0; i < tabs.length; i++) {
+      let tabData = tabs[i];
+      let entry = tabData.entries[tabData.index - 1];
+
+      // Use stubbed tab if we've already created it; otherwise, make a new tab
+      let tab;
+      if (tabData.tabId == null) {
+        let params = {
+          selected: (selected == i+1),
+          delayLoad: true,
+          title: entry.title,
+          desktopMode: (tabData.desktopMode == true),
+          isPrivate: (tabData.isPrivate == true)
+        };
+        tab = window.BrowserApp.addTab(entry.url, params);
+      } else {
+        tab = window.BrowserApp.getTabForId(tabData.tabId);
+        delete tabData.tabId;
+
+        // Don't restore tab if user has closed it
+        if (tab == null) {
+          continue;
+        }
+      }
+
+      if (window.BrowserApp.selectedTab == tab) {
+        this._restoreHistory(tabData, tab.browser.sessionHistory);
+        delete tab.browser.__SS_restore;
+      } else {
+        // Make sure the browser has its session data for the delay reload
+        tab.browser.__SS_data = tabData;
+        tab.browser.__SS_restore = true;
+      }
+
+      tab.browser.__SS_extdata = tabData.extData;
+    }
+
+    return true;
   },
 
   getClosedTabCount: function ss_getClosedTabCount(aWindow) {
@@ -851,26 +940,6 @@ SessionStore.prototype = {
   setTabValue: function ss_setTabValue(aTab, aKey, aStringValue) {
     let browser = aTab.browser;
 
-    // Thumbnails are actually stored in the cache, so do the save and update the URI
-    if (aKey == "thumbnail") {
-      let file = this._sessionCache.clone();
-      file.append("thumbnail-" + browser.contentWindowId);
-      file.createUnique(Ci.nsIFile.NORMAL_FILE_TYPE, 0600);
-
-      let source = Services.io.newURI(aStringValue, "UTF8", null);
-      let target = Services.io.newFileURI(file)
-
-      let persist = Cc["@mozilla.org/embedding/browser/nsWebBrowserPersist;1"].createInstance(Ci.nsIWebBrowserPersist);
-      persist.persistFlags = Ci.nsIWebBrowserPersist.PERSIST_FLAGS_REPLACE_EXISTING_FILES | Ci.nsIWebBrowserPersist.PERSIST_FLAGS_AUTODETECT_APPLY_CONVERSION;
-      let privacyContext = browser.contentWindow
-                                  .QueryInterface(Ci.nsIInterfaceRequestor)
-                                  .getInterface(Ci.nsIWebNavigation)
-                                  .QueryInterface(Ci.nsILoadContext);
-      persist.saveURI(source, null, null, null, null, file, privacyContext);
-
-      aStringValue = target.spec;
-    }
-
     if (!browser.__SS_extdata)
       browser.__SS_extdata = {};
     browser.__SS_extdata[aKey] = aStringValue;
@@ -889,111 +958,76 @@ SessionStore.prototype = {
     return this._shouldRestore;
   },
 
-  restoreLastSession: function ss_restoreLastSession(aBringToFront, aForceRestore) {
+  restoreLastSession: function ss_restoreLastSession(aRestoringOOM, aSessionString) {
     let self = this;
+
+    function restoreWindow(data) {
+      if (!self._restoreWindow(data)) {
+        throw "Could not restore window";
+      }
+
+      notifyObservers();
+    }
+
     function notifyObservers(aMessage) {
-      self._clearCache();
       Services.obs.notifyObservers(null, "sessionstore-windows-restored", aMessage || "");
     }
 
-    let sessionFile = this._sessionFile;
-
-    // aForceRestore will be true when we are recovering from Android OOM kills
-    if (!aForceRestore) {
-      // If we are not recovering from an OOM kill (i.e., we actually crashed),
-      // move sessionstore.js -> sessionstore.bak. sessionstore.bak is used in
-      // about:home to read the "tabs from last time", so since we've started a
-      // new session after the crash, we need to make sure sessionstore.bak is
-      // current. We do not move sessionstore.js -> sessionstore.bak if we had
-      // an OOM kill since restoring from an OOM kill should look like the same
-      // session as before (so the "tabs from last time" should stay the same).
-      if (sessionFile.exists()) {
-        sessionFile.clone().moveTo(null, this._sessionFileBackup.leafName);
-        sessionFile = this._sessionFileBackup;
-      }
-
-      let maxCrashes = Services.prefs.getIntPref("browser.sessionstore.max_resumed_crashes");
-      let recentCrashes = Services.prefs.getIntPref("browser.sessionstore.recent_crashes") + 1;
-      Services.prefs.setIntPref("browser.sessionstore.recent_crashes", recentCrashes);
-      Services.prefs.savePrefFile(null);
-
-      if (recentCrashes > maxCrashes) {
-        notifyObservers("fail");
-        return;
-      }
-    }
-
-    if (!sessionFile.exists()) {
-      notifyObservers("fail");
-      return;
-    }
-
     try {
-      let channel = NetUtil.newChannel(sessionFile);
-      channel.contentType = "application/json";
-      NetUtil.asyncFetch(channel, function(aStream, aResult) {
-        if (!Components.isSuccessCode(aResult)) {
-          Cu.reportError("SessionStore: Could not read from sessionstore.bak file");
-          notifyObservers("fail");
-          return;
+      if (!aRestoringOOM && !this._shouldRestore) {
+        // If we're here, it means we're restoring from a crash (not an OOM
+        // kill). Check prefs and other conditions to make sure we want to
+        // continue with the restore.
+
+        // Disable crash recovery if it has been turned off.
+        if (!Services.prefs.getBoolPref("browser.sessionstore.resume_from_crash")) {
+          throw "Restore is disabled via prefs";
         }
 
-        // Read session state file into a string and let observers modify the state before it's being used
-        let state = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
-        state.data = NetUtil.readInputStreamToString(aStream, aStream.available(), { charset : "UTF-8" }) || "";
-        aStream.close();
+        // Check to see if we've exceeded the maximum number of crashes to
+        // avoid a crash loop
+        let maxCrashes = Services.prefs.getIntPref("browser.sessionstore.max_resumed_crashes");
+        let recentCrashes = Services.prefs.getIntPref("browser.sessionstore.recent_crashes") + 1;
+        Services.prefs.setIntPref("browser.sessionstore.recent_crashes", recentCrashes);
+        Services.prefs.savePrefFile(null);
 
-        Services.obs.notifyObservers(state, "sessionstore-state-read", "");
+        if (recentCrashes > maxCrashes) {
+          throw "Exceeded maximum number of allowed restores";
+        }
+      }
 
-        let data = null;
-        try {
-          data = JSON.parse(state.data);
-        } catch (ex) {
-          Cu.reportError("SessionStore: Could not parse JSON: " + ex);
+      // Normally, we'll receive the session string from Java, but there are
+      // cases where we may want to restore that Java cannot detect (e.g., if
+      // browser.sessionstore.resume_session_once is true). In these cases, the
+      // session will be read from sessionstore.bak (which is also used for
+      // "tabs from last time").
+      if (aSessionString == null) {
+        if (!this._sessionFileBackup.exists()) {
+          throw "Session file doesn't exist";
         }
 
-        // To do a restore, we must have at least one window with one tab
-        if (!data || data.windows.length == 0 || !data.windows[0].tabs || data.windows[0].tabs.length == 0) {
-          notifyObservers("fail");
-          return;
-        }
+        let channel = NetUtil.newChannel(this._sessionFileBackup);
+        channel.contentType = "application/json";
+        NetUtil.asyncFetch(channel, function(aStream, aResult) {
+          try {
+            if (!Components.isSuccessCode(aResult)) {
+              throw "Could not fetch session file";
+            }
 
-        let window = Services.wm.getMostRecentWindow("navigator:browser");
-
-        let tabs = data.windows[0].tabs;
-        let selected = data.windows[0].selected;
-        if (selected > tabs.length) // Clamp the selected index if it's bogus
-          selected = 1;
-
-        for (let i=0; i<tabs.length; i++) {
-          let tabData = tabs[i];
-          let isSelected = (i + 1 == selected) && aBringToFront;
-          let entry = tabData.entries[tabData.index - 1];
-
-          // Add a tab, but don't load the URL until we need to
-          let params = {
-            selected: isSelected,
-            delayLoad: true,
-            title: entry.title,
-            desktopMode: tabData.desktopMode == true
-          };
-          let tab = window.BrowserApp.addTab(entry.url, params);
-
-          if (isSelected) {
-            self._restoreHistory(tabData, tab.browser.sessionHistory);
-          } else {
-            // Make sure the browser has its session data for the delay reload
-            tab.browser.__SS_data = tabData;
-            tab.browser.__SS_restore = true;
+            let data = NetUtil.readInputStreamToString(aStream, aStream.available(), { charset : "UTF-8" }) || "";
+            aStream.close();
+            
+            restoreWindow(data);
+          } catch (e) {
+            Cu.reportError("SessionStore: " + e.message);
+            notifyObservers("fail");
           }
-
-          tab.browser.__SS_extdata = tabData.extData;
-        }
-
-        notifyObservers();
-      });
-    } catch (ex) {
-      Cu.reportError("SessionStore: Could not read from sessionstore.bak file: " + ex);
+        });
+      } else {
+        restoreWindow(aSessionString);
+      }
+    } catch (e) {
+      Cu.reportError("SessionStore: " + e);
       notifyObservers("fail");
     }
   }
