@@ -65,6 +65,13 @@ MappableFile::finalize()
   fd = -1;
 }
 
+size_t
+MappableFile::GetLength() const
+{
+  struct stat st;
+  return fstat(fd, &st) ? 0 : st.st_size;
+}
+
 Mappable *
 MappableExtractFile::Create(const char *name, Zip *zip, Zip::Stream *stream)
 {
@@ -342,6 +349,12 @@ MappableDeflate::finalize()
   zip = NULL;
 }
 
+size_t
+MappableDeflate::GetLength() const
+{
+  return buffer->GetLength();
+}
+
 Mappable *
 MappableSeekableZStream::Create(const char *name, Zip *zip,
                                 Zip::Stream *stream)
@@ -543,3 +556,9 @@ MappableSeekableZStream::stats(const char *when, const char *name) const
     }
   }
 }
+
+size_t
+MappableSeekableZStream::GetLength() const
+{
+  return buffer->GetLength();
+}
