@@ -6,6 +6,7 @@
 
 #include "nsHtml5TreeOperation.h"
 #include "nsContentUtils.h"
+#include "nsDocElementCreatedNotificationRunner.h"
 #include "nsNodeUtils.h"
 #include "nsAttrName.h"
 #include "nsHtml5TreeBuilder.h"
@@ -29,12 +30,16 @@
 #include "nsIFormProcessor.h"
 #include "nsIServiceManager.h"
 #include "nsEscape.h"
+#include "mozilla/dom/Comment.h"
 #include "mozilla/dom/Element.h"
+#include "mozilla/dom/HTMLTemplateElement.h"
 #include "nsHtml5SVGLoadDispatcher.h"
 #include "nsIURI.h"
 #include "nsIProtocolHandler.h"
 #include "nsNetUtil.h"
 #include "nsIHTMLDocument.h"
+#include "mozilla/Likely.h"
+#include "nsTextNode.h"
 
 namespace dom = mozilla::dom;
 
@@ -44,13 +49,13 @@ static NS_DEFINE_CID(kFormProcessorCID, NS_FORMPROCESSOR_CID);
  * Helper class that opens a notification batch if the current doc
  * is different from the executor doc.
  */
-class NS_STACK_CLASS nsHtml5OtherDocUpdate {
+class MOZ_STACK_CLASS nsHtml5OtherDocUpdate {
   public:
     nsHtml5OtherDocUpdate(nsIDocument* aCurrentDoc, nsIDocument* aExecutorDoc)
     {
       NS_PRECONDITION(aCurrentDoc, "Node has no doc?");
       NS_PRECONDITION(aExecutorDoc, "Executor has no doc?");
-      if (NS_LIKELY(aCurrentDoc == aExecutorDoc)) {
+      if (MOZ_LIKELY(aCurrentDoc == aExecutorDoc)) {
         mDocument = nullptr;
       } else {
         mDocument = aCurrentDoc;
@@ -60,7 +65,7 @@ class NS_STACK_CLASS nsHtml5OtherDocUpdate {
 
     ~nsHtml5OtherDocUpdate()
     {
-      if (NS_UNLIKELY(mDocument)) {
+      if (MOZ_UNLIKELY(mDocument)) {
         mDocument->EndUpdate(UPDATE_CONTENT_MODEL);
       }
     }
@@ -159,8 +164,7 @@ nsHtml5TreeOperation::AppendText(const PRUnichar* aBuffer,
                                 aBuilder);
   }
 
-  nsCOMPtr<nsIContent> text;
-  NS_NewTextNode(getter_AddRefs(text), aBuilder->GetNodeInfoManager());
+  nsRefPtr<nsTextNode> text = new nsTextNode(aBuilder->GetNodeInfoManager());
   NS_ASSERTION(text, "Infallible malloc failed?");
   rv = text->SetText(aBuffer, aLength, false);
   NS_ENSURE_SUCCESS(rv, rv);
@@ -179,7 +183,7 @@ nsHtml5TreeOperation::Append(nsIContent* aNode,
   nsIDocument* parentDoc = aParent->OwnerDoc();
   NS_ASSERTION(parentDoc, "Null owner doc on old node.");
 
-  if (NS_LIKELY(executorDoc == parentDoc)) {
+  if (MOZ_LIKELY(executorDoc == parentDoc)) {
     // the usual case. the parent is in the parser's doc
     rv = aParent->AppendChildTo(aNode, false);
     if (NS_SUCCEEDED(rv)) {
@@ -330,7 +334,7 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       nsHtml5HtmlAttributes* attributes = mThree.attributes;
       
       bool isKeygen = (name == nsHtml5Atoms::keygen && ns == kNameSpaceID_XHTML);
-      if (NS_UNLIKELY(isKeygen)) {
+      if (MOZ_UNLIKELY(isKeygen)) {
         name = nsHtml5Atoms::select;
       }
       
@@ -349,13 +353,13 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
 
       aBuilder->HoldElement(*target = newContent);      
 
-      if (NS_UNLIKELY(name == nsHtml5Atoms::style || name == nsHtml5Atoms::link)) {
+      if (MOZ_UNLIKELY(name == nsHtml5Atoms::style || name == nsHtml5Atoms::link)) {
         nsCOMPtr<nsIStyleSheetLinkingElement> ssle(do_QueryInterface(newContent));
         if (ssle) {
           ssle->InitStyleLinkElement(false);
           ssle->SetEnableUpdates(false);
         }
-      } else if (NS_UNLIKELY(isKeygen)) {
+      } else if (MOZ_UNLIKELY(isKeygen)) {
         // Adapted from CNavDTD
         nsCOMPtr<nsIFormProcessor> theFormProcessor =
           do_GetService(kFormProcessorCID, &rv);
@@ -390,9 +394,8 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
                          : (aBuilder->BelongsToStringParser() ?
                             dom::FROM_PARSER_FRAGMENT :
                             dom::FROM_PARSER_DOCUMENT_WRITE)));
-          nsCOMPtr<nsIContent> optionText;
-          NS_NewTextNode(getter_AddRefs(optionText), 
-                         aBuilder->GetNodeInfoManager());
+          nsRefPtr<nsTextNode> optionText =
+            new nsTextNode(aBuilder->GetNodeInfoManager());
           (void) optionText->SetText(theContent[i], false);
           optionElt->AppendChildTo(optionText, false);
           newContent->AppendChildTo(optionElt, false);
@@ -495,8 +498,8 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
                                       aBuilder);
         }
         
-        nsCOMPtr<nsIContent> text;
-        NS_NewTextNode(getter_AddRefs(text), aBuilder->GetNodeInfoManager());
+        nsRefPtr<nsTextNode> text =
+          new nsTextNode(aBuilder->GetNodeInfoManager());
         NS_ASSERTION(text, "Infallible malloc failed?");
         rv = text->SetText(buffer, length, false);
         NS_ENSURE_SUCCESS(rv, rv);
@@ -514,8 +517,8 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       PRUnichar* buffer = mTwo.unicharPtr;
       int32_t length = mFour.integer;
       
-      nsCOMPtr<nsIContent> comment;
-      NS_NewCommentNode(getter_AddRefs(comment), aBuilder->GetNodeInfoManager());
+      nsRefPtr<dom::Comment> comment =
+        new dom::Comment(aBuilder->GetNodeInfoManager());
       NS_ASSERTION(comment, "Infallible malloc failed?");
       rv = comment->SetText(buffer, length, false);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -526,8 +529,8 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       PRUnichar* buffer = mTwo.unicharPtr;
       int32_t length = mFour.integer;
       
-      nsCOMPtr<nsIContent> comment;
-      NS_NewCommentNode(getter_AddRefs(comment), aBuilder->GetNodeInfoManager());
+      nsRefPtr<dom::Comment> comment =
+        new dom::Comment(aBuilder->GetNodeInfoManager());
       NS_ASSERTION(comment, "Infallible malloc failed?");
       rv = comment->SetText(buffer, length, false);
       NS_ENSURE_SUCCESS(rv, rv);
@@ -556,6 +559,13 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
       nsCOMPtr<nsIContent> asContent = do_QueryInterface(docType);
       return AppendToDocument(asContent, aBuilder);
     }
+    case eTreeOpGetDocumentFragmentForTemplate: {
+      dom::HTMLTemplateElement* tempElem =
+        static_cast<dom::HTMLTemplateElement*>(*mOne.node);
+      nsRefPtr<dom::DocumentFragment> frag = tempElem->Content();
+      *mTwo.node = frag.get();
+      return rv;
+    }
     case eTreeOpMarkAsBroken: {
       aBuilder->MarkAsBroken(NS_ERROR_OUT_OF_MEMORY);
       return rv;
@@ -809,3 +819,4 @@ nsHtml5TreeOperation::Perform(nsHtml5TreeOpExecutor* aBuilder,
   }
   return rv; // keep compiler happy
 }
+
