@@ -27,7 +27,6 @@ FormHistory.prototype = {
 
     debug          : true,
     enabled        : true,
-    saveHttpsForms : true,
 
     // The current database schema.
     dbSchema : {
@@ -73,21 +72,6 @@ FormHistory.prototype = {
         return this._uuidService;
     },
 
-    // Private Browsing Service
-    // If the service is not available, null will be returned.
-    _privBrowsingSvc : undefined,
-    get privBrowsingSvc() {
-        if (this._privBrowsingSvc == undefined) {
-            if ("@mozilla.org/privatebrowsing;1" in Cc)
-                this._privBrowsingSvc = Cc["@mozilla.org/privatebrowsing;1"].
-                                        getService(Ci.nsIPrivateBrowsingService);
-            else
-                this._privBrowsingSvc = null;
-        }
-        return this._privBrowsingSvc;
-    },
-
-
     log : function log(message) {
         if (!this.debug)
             return;
@@ -97,43 +81,14 @@ FormHistory.prototype = {
 
 
     init : function init() {
-        Services.prefs.addObserver("browser.formfill.", this, true);
-
         this.updatePrefs();
 
         this.dbStmts = {};
 
-        this.messageManager = Cc["@mozilla.org/globalmessagemanager;1"].
-                              getService(Ci.nsIMessageListenerManager);
-        this.messageManager.loadFrameScript("chrome://satchel/content/formSubmitListener.js", true);
-        this.messageManager.addMessageListener("FormHistory:FormSubmitEntries", this);
-
-        // Add observers
+        // Add observer
         Services.obs.addObserver(this, "profile-before-change", true);
-        Services.obs.addObserver(this, "idle-daily", true);
-        Services.obs.addObserver(this, "formhistory-expire-now", true);
     },
 
-    /* ---- message listener ---- */
-
-
-    receiveMessage: function receiveMessage(message) {
-        // Open a transaction so multiple adds happen in one commit
-        this.dbConnection.beginTransaction();
-
-        try {
-            let entries = message.json;
-            for (let i = 0; i < entries.length; i++) {
-                this.addEntry(entries[i].name, entries[i].value);
-            }
-        } finally {
-            // Don't need it to be atomic if there was an error.  Commit what
-            // we managed to put in the table.
-            this.dbConnection.commitTransaction();
-        }
-    },
-
-
     /* ---- nsIFormHistory2 interfaces ---- */
 
 
@@ -143,8 +98,7 @@ FormHistory.prototype = {
 
 
     addEntry : function addEntry(name, value) {
-        if (!this.enabled ||
-            this.privBrowsingSvc && this.privBrowsingSvc.privateBrowsingEnabled)
+        if (!this.enabled)
             return;
 
         this.log("addEntry for " + name + "=" + value);
@@ -450,10 +404,6 @@ FormHistory.prototype = {
         case "nsPref:changed":
             this.updatePrefs();
             break;
-        case "idle-daily":
-        case "formhistory-expire-now":
-            this.expireOldEntries();
-            break;
         case "profile-before-change":
             this._dbClose(false);
             break;
@@ -585,56 +535,9 @@ FormHistory.prototype = {
     },
 
 
-    expireOldEntries : function () {
-        this.log("expireOldEntries");
-
-        // Determine how many days of history we're supposed to keep.
-        let expireDays = 180;
-        try {
-            expireDays = Services.prefs.getIntPref("browser.formfill.expire_days");
-        } catch (e) { /* ignore */ }
-
-        let expireTime = Date.now() - expireDays * DAY_IN_MS;
-        expireTime *= 1000; // switch to microseconds
-
-        this.sendIntNotification("before-expireOldEntries", expireTime);
-
-        let beginningCount = this.countAllEntries();
-
-        // Purge the form history...
-        let stmt;
-        let query = "DELETE FROM moz_formhistory WHERE lastUsed <= :expireTime";
-        let params = { expireTime : expireTime };
-
-        try {
-            stmt = this.dbCreateStatement(query, params);
-            stmt.execute();
-        } catch (e) {
-            this.log("expireOldEntries failed: " + e);
-            throw e;
-        } finally {
-            if (stmt) {
-                stmt.reset();
-            }
-        }
-
-        let endingCount = this.countAllEntries();
-
-        // If we expired a large batch of entries, shrink the DB to reclaim wasted
-        // space. This is expected to happen when entries predating timestamps
-        // (added in the v.1 schema) expire in mass, 180 days after the DB was
-        // upgraded -- entries not used since then expire all at once.
-        if (beginningCount - endingCount > 500)
-            this.dbConnection.executeSimpleSQL("VACUUM");
-
-        this.sendIntNotification("expireOldEntries", expireTime);
-    },
-
-
     updatePrefs : function () {
         this.debug          = Services.prefs.getBoolPref("browser.formfill.debug");
         this.enabled        = Services.prefs.getBoolPref("browser.formfill.enable");
-        this.saveHttpsForms = Services.prefs.getBoolPref("browser.formfill.saveHttpsForms");
     },
 
 //**************************************************************************//
@@ -939,7 +842,10 @@ FormHistory.prototype = {
             stmt.finalize();
         }
         this.dbStmts = {};
-        if (this.dbConnection === undefined)
+
+        let connectionDescriptor = Object.getOwnPropertyDescriptor(FormHistory.prototype, "dbConnection");
+        // Return if the database hasn't been opened.
+        if (!connectionDescriptor || connectionDescriptor.value === undefined)
             return;
 
         let completed = false;
