@@ -58,12 +58,13 @@ const RWX_OWNER = 0700;
 //   * MEMORY_JS_GC_HEAP, and
 //   * MEMORY_JS_COMPARTMENTS_SYSTEM.
 //
+// We used to measure "explicit" too, but it could cause hangs, and the data
+// was always really noisy anyway.  See bug 859657.
 const MEM_HISTOGRAMS = {
   "js-gc-heap": "MEMORY_JS_GC_HEAP",
   "js-compartments/system": "MEMORY_JS_COMPARTMENTS_SYSTEM",
   "js-compartments/user": "MEMORY_JS_COMPARTMENTS_USER",
   "js-main-runtime-temporary-peak": "MEMORY_JS_MAIN_RUNTIME_TEMPORARY_PEAK",
-  "explicit": "MEMORY_EXPLICIT",
   "resident-fast": "MEMORY_RESIDENT",
   "vsize": "MEMORY_VSIZE",
   "storage-sqlite": "MEMORY_STORAGE_SQLITE",
@@ -173,7 +174,6 @@ TelemetryPing.prototype = {
   _pendingPings: [],
   _doLoadSaveNotifications: false,
   _startupIO : {},
-  _hashID: Ci.nsICryptoHash.SHA256,
   // The number of outstanding saved pings that we have issued loading
   // requests for.
   _pingsLoaded: 0,
@@ -244,6 +244,10 @@ TelemetryPing.prototype = {
     if (failedProfileLockCount)
       ret.failedProfileLockCount = failedProfileLockCount;
 
+    let maximalNumberOfConcurrentThreads = Telemetry.maximalNumberOfConcurrentThreads;
+    if (maximalNumberOfConcurrentThreads)
+      ret.maximalNumberOfConcurrentThreads = maximalNumberOfConcurrentThreads;
+
     for (let ioCounter in this._startupIO)
       ret[ioCounter] = this._startupIO[ioCounter];
 
@@ -419,7 +423,7 @@ TelemetryPing.prototype = {
                      "adapterDriverDate", "adapterDescription2",
                      "adapterVendorID2", "adapterDeviceID2", "adapterRAM2",
                      "adapterDriver2", "adapterDriverVersion2",
-                     "adapterDriverDate2", "isGPU2Active", "D2DEnabled;",
+                     "adapterDriverDate2", "isGPU2Active", "D2DEnabled",
                      "DWriteEnabled", "DWriteVersion"
                     ];
 
@@ -566,10 +570,10 @@ TelemetryPing.prototype = {
     };
 
     if (Object.keys(this._slowSQLStartup.mainThread).length
-	|| Object.keys(this._slowSQLStartup.otherThreads).length) {
+      || Object.keys(this._slowSQLStartup.otherThreads).length) {
       payloadObj.slowSQLStartup = this._slowSQLStartup;
     }
-    
+
     return payloadObj;
   },
 
@@ -606,17 +610,6 @@ TelemetryPing.prototype = {
     return { __iterator__: payloadIterWithThis };
   },
 
-  hashString: function hashString(s) {
-    let digest = Cc["@mozilla.org/security/hash;1"]
-                 .createInstance(Ci.nsICryptoHash);
-    digest.init(this._hashID);
-    let stream = Cc["@mozilla.org/io/string-input-stream;1"]
-                 .createInstance(Ci.nsIStringInputStream);
-    stream.data = s;
-    digest.updateFromStream(stream, stream.available());
-    return digest.finish(/*base64encode=*/true);
-  },
-
   /**
    * Send data to the server. Record success/send-time in histograms
    */
@@ -823,18 +816,6 @@ TelemetryPing.prototype = {
                                  Ci.nsITimer.TYPE_ONE_SHOT);
   },
 
-  ensurePingChecksum: function ensurePingChecksum(ping) {
-    /* A ping from the current session won't have a checksum.  */
-    if (!ping.checksum) {
-      return;
-    }
-
-    let checksumNow = this.hashString(ping.payload);
-    if (ping.checksum != checksumNow) {
-      throw new Error("Invalid ping checksum")
-    }
-  },
-
   addToPendingPings: function addToPendingPings(file, stream) {
     let success = false;
 
@@ -843,8 +824,6 @@ TelemetryPing.prototype = {
       stream.close();
       let ping = JSON.parse(string);
       this._pingLoadsCompleted++;
-      // This will throw if checksum is invalid.
-      this.ensurePingChecksum(ping);
       this._pendingPings.push(ping);
       if (this._doLoadSaveNotifications &&
           this._pingLoadsCompleted == this._pingsLoaded) {
@@ -852,7 +831,7 @@ TelemetryPing.prototype = {
       }
       success = true;
     } catch (e) {
-      // An error reading the file, or an error parsing/checksumming the contents.
+      // An error reading the file, or an error parsing the contents.
       stream.close();           // close is idempotent.
       file.remove(true);
     }
@@ -981,9 +960,6 @@ TelemetryPing.prototype = {
   },
 
   saveFileForPing: function saveFileForPing(ping) {
-    if (!('checksum' in ping)) {
-      ping.checksum = this.hashString(ping.payload);
-    }
     let file = this.ensurePingDirectory();
     file.append(ping.slug);
     return file;
