@@ -28,7 +28,10 @@
 #include <string.h>
 #include <errno.h>
 
+#include "nsThreadUtils.h"
+
 #include "platform.h"
+#include "TableTicker.h"
 #include "UnwinderThread2.h"  /* uwt__register_thread_for_profiling */
 
 // this port is based off of v8 svn revision 9837
@@ -89,21 +92,13 @@ void OS::Sleep(int milliseconds) {
   usleep(1000 * milliseconds);
 }
 
-class Thread::PlatformData : public Malloced {
- public:
-  PlatformData() : thread_(kNoThread) {}
-  pthread_t thread_;  // Thread handle for pthread.
-};
-
 Thread::Thread(const char* name)
-    : data_(new PlatformData),
-      stack_size_(0) {
+    : stack_size_(0) {
   set_name(name);
 }
 
 
 Thread::~Thread() {
-  delete data_;
 }
 
 
@@ -126,22 +121,10 @@ static void SetThreadName(const char* name) {
 
 static void* ThreadEntry(void* arg) {
   Thread* thread = reinterpret_cast<Thread*>(arg);
-  // This is also initialized by the first argument to pthread_create() but we
-  // don't know which thread will run first (the original thread or the new
-  // one) so we initialize it here too.
 
-  // BEGIN temp hack for SPS v1-vs-v2
-  extern bool sps_version2();
-  if (sps_version2()) {
-    // Register this thread for profiling.
-    int aLocal;
-    uwt__register_thread_for_profiling( &aLocal );
-  }
-  // END temp hack for SPS v1-vs-v2
-
-  thread->data()->thread_ = pthread_self();
+  thread->thread_ = pthread_self();
   SetThreadName(thread->name());
-  ASSERT(thread->data()->thread_ != kNoThread);
+  ASSERT(thread->thread_ != kNoThread);
   thread->Run();
   return NULL;
 }
@@ -161,15 +144,15 @@ void Thread::Start() {
     pthread_attr_setstacksize(&attr, static_cast<size_t>(stack_size_));
     attr_ptr = &attr;
   }
-  pthread_create(&data_->thread_, attr_ptr, ThreadEntry, this);
-  ASSERT(data_->thread_ != kNoThread);
+  pthread_create(&thread_, attr_ptr, ThreadEntry, this);
+  ASSERT(thread_ != kNoThread);
 }
 
 void Thread::Join() {
-  pthread_join(data_->thread_, NULL);
+  pthread_join(thread_, NULL);
 }
 
-class Sampler::PlatformData : public Malloced {
+class PlatformData : public Malloced {
  public:
   PlatformData() : profiled_thread_(mach_thread_self())
   {
@@ -195,15 +178,26 @@ class Sampler::PlatformData : public Malloced {
   pthread_t profiled_pthread_;
 };
 
+/* static */ PlatformData*
+Sampler::AllocPlatformData(int aThreadId)
+{
+  return new PlatformData;
+}
+
+/* static */ void
+Sampler::FreePlatformData(PlatformData* aData)
+{
+  delete aData;
+}
 
 class SamplerThread : public Thread {
  public:
   explicit SamplerThread(int interval)
-      : Thread("SamplerThread"),
-        interval_(interval) {}
+      : Thread("SamplerThread")
+      , interval_(interval) {}
 
   static void AddActiveSampler(Sampler* sampler) {
-    ScopedLock lock(mutex_);
+    mozilla::MutexAutoLock lock(*Sampler::sRegisteredThreadsMutex);
     SamplerRegistry::AddActiveSampler(sampler);
     if (instance_ == NULL) {
       instance_ = new SamplerThread(sampler->interval());
@@ -214,37 +208,45 @@ class SamplerThread : public Thread {
   }
 
   static void RemoveActiveSampler(Sampler* sampler) {
-    ScopedLock lock(mutex_);
+    mozilla::MutexAutoLock lock(*Sampler::sRegisteredThreadsMutex);
     instance_->Join();
     //XXX: unlike v8 we need to remove the active sampler after doing the Join
     // because we drop the sampler immediately
     SamplerRegistry::RemoveActiveSampler(sampler);
     delete instance_;
     instance_ = NULL;
-    /*
-    if (SamplerRegistry::GetState() == SamplerRegistry::HAS_NO_SAMPLERS) {
-      RuntimeProfiler::StopRuntimeProfilerThreadBeforeShutdown(instance_);
-      delete instance_;
-      instance_ = NULL;
-    }
-    */
   }
 
   // Implement Thread::Run().
   virtual void Run() {
     while (SamplerRegistry::sampler->IsActive()) {
-      if (!SamplerRegistry::sampler->IsPaused())
-        SampleContext(SamplerRegistry::sampler);
+      {
+        mozilla::MutexAutoLock lock(*Sampler::sRegisteredThreadsMutex);
+        std::vector<ThreadInfo*> threads =
+          SamplerRegistry::sampler->GetRegisteredThreads();
+        for (uint32_t i = 0; i < threads.size(); i++) {
+          ThreadInfo* info = threads[i];
+
+          // This will be null if we're not interested in profiling this thread.
+          if (!info->Profile())
+            continue;
+
+          ThreadProfile* thread_profile = info->Profile();
+
+          if (!SamplerRegistry::sampler->IsPaused())
+            SampleContext(SamplerRegistry::sampler, thread_profile);
+        }
+      }
       OS::Sleep(interval_);
     }
   }
 
-  void SampleContext(Sampler* sampler) {
-    thread_act_t profiled_thread = sampler->platform_data()->profiled_thread();
+  void SampleContext(Sampler* sampler, ThreadProfile* thread_profile) {
+    thread_act_t profiled_thread =
+      thread_profile->GetPlatformData()->profiled_thread();
+
     TickSample sample_obj;
     TickSample* sample = &sample_obj;
-    //TickSample* sample = CpuProfiler::TickSampleEvent(sampler->isolate());
-    //if (sample == NULL) sample = &sample_obj;
 
     if (KERN_SUCCESS != thread_suspend(profiled_thread)) return;
 
@@ -274,12 +276,11 @@ class SamplerThread : public Thread {
                          flavor,
                          reinterpret_cast<natural_t*>(&state),
                          &count) == KERN_SUCCESS) {
-      //sample->state = sampler->isolate()->current_vm_state();
       sample->pc = reinterpret_cast<Address>(state.REGISTER_FIELD(ip));
       sample->sp = reinterpret_cast<Address>(state.REGISTER_FIELD(sp));
       sample->fp = reinterpret_cast<Address>(state.REGISTER_FIELD(bp));
       sample->timestamp = mozilla::TimeStamp::Now();
-      sampler->SampleStack(sample);
+      sample->threadProfile = thread_profile;
       sampler->Tick(sample);
     }
     thread_resume(profiled_thread);
@@ -297,25 +298,21 @@ class SamplerThread : public Thread {
 
 #undef REGISTER_FIELD
 
-
-Mutex* SamplerThread::mutex_ = OS::CreateMutex();
 SamplerThread* SamplerThread::instance_ = NULL;
 
-
-Sampler::Sampler(int interval, bool profiling)
+Sampler::Sampler(int interval, bool profiling, int entrySize)
     : // isolate_(isolate),
       interval_(interval),
       profiling_(profiling),
       paused_(false),
-      active_(false) /*,
+      active_(false),
+      entrySize_(entrySize) /*,
       samples_taken_(0)*/ {
-  data_ = new PlatformData;
 }
 
 
 Sampler::~Sampler() {
   ASSERT(!IsActive());
-  delete data_;
 }
 
 
@@ -333,7 +330,62 @@ void Sampler::Stop() {
 }
 
 pthread_t
-Sampler::GetProfiledThread(Sampler::PlatformData* aData)
+Sampler::GetProfiledThread(PlatformData* aData)
 {
   return aData->profiled_pthread();
 }
+
+#include <sys/syscall.h>
+pid_t gettid()
+{
+  return (pid_t) syscall(SYS_thread_selfid);
+}
+
+bool Sampler::RegisterCurrentThread(const char* aName, PseudoStack* aPseudoStack, bool aIsMainThread)
+{
+  if (!Sampler::sRegisteredThreadsMutex)
+    return false;
+
+  mozilla::MutexAutoLock lock(*Sampler::sRegisteredThreadsMutex);
+
+  ThreadInfo* info = new ThreadInfo(aName, gettid(),
+    aIsMainThread, aPseudoStack);
+
+  bool profileThread = sActiveSampler &&
+    (aIsMainThread || sActiveSampler->ProfileThreads());
+
+  if (profileThread) {
+    // We need to create the ThreadProfile now
+    info->SetProfile(new ThreadProfile(info->Name(),
+                                       sActiveSampler->EntrySize(),
+                                       info->Stack(),
+                                       info->ThreadId(),
+                                       info->GetPlatformData(),
+                                       aIsMainThread));
+    if (sActiveSampler->ProfileJS()) {
+      info->Profile()->GetPseudoStack()->enableJSSampling();
+    }
+  }
+
+  sRegisteredThreads->push_back(info);
+  return true;
+}
+
+void Sampler::UnregisterCurrentThread()
+{
+  if (!Sampler::sRegisteredThreadsMutex)
+    return;
+
+  mozilla::MutexAutoLock lock(*Sampler::sRegisteredThreadsMutex);
+
+  int id = gettid();
+
+  for (uint32_t i = 0; i < sRegisteredThreads->size(); i++) {
+    ThreadInfo* info = sRegisteredThreads->at(i);
+    if (info->ThreadId() == id) {
+      delete info;
+      sRegisteredThreads->erase(sRegisteredThreads->begin() + i);
+      break;
+    }
+  }
+}
