@@ -7,6 +7,7 @@
 #define AndroidJavaWrappers_h__
 
 #include <jni.h>
+#include <android/input.h>
 #include <android/log.h>
 
 #include "nsGeoPosition.h"
@@ -15,6 +16,8 @@
 #include "nsString.h"
 #include "nsTArray.h"
 #include "mozilla/gfx/Rect.h"
+#include "mozilla/dom/Touch.h"
+#include "InputData.h"
 
 //#define FORCE_ALOG 1
 
@@ -193,6 +196,8 @@ public:
     float GetY(JNIEnv *env);
     float GetScale(JNIEnv *env);
     void GetFixedLayerMargins(JNIEnv *env, gfx::Margin &aFixedLayerMargins);
+    float GetOffsetX(JNIEnv *env);
+    float GetOffsetY(JNIEnv *env);
 
 private:
     static jclass jViewTransformClass;
@@ -203,6 +208,8 @@ private:
     static jfieldID jFixedLayerMarginTop;
     static jfieldID jFixedLayerMarginRight;
     static jfieldID jFixedLayerMarginBottom;
+    static jfieldID jOffsetXField;
+    static jfieldID jOffsetYField;
 };
 
 class AndroidProgressiveUpdateData : public WrappedJavaObject {
@@ -264,7 +271,10 @@ public:
     void SetPageRect(const gfx::Rect& aCssPageRect);
     void SyncViewportInfo(const nsIntRect& aDisplayPort, float aDisplayResolution, bool aLayersUpdated,
                           nsIntPoint& aScrollOffset, float& aScaleX, float& aScaleY,
-                          gfx::Margin& aFixedLayerMargins);
+                          gfx::Margin& aFixedLayerMargins, gfx::Point& aOffset);
+    void SyncFrameMetrics(const gfx::Point& aScrollOffset, float aZoom, const gfx::Rect& aCssPageRect,
+                          bool aLayersUpdated, const gfx::Rect& aDisplayPort, float aDisplayResolution,
+                          bool aIsFirstPaint, gfx::Margin& aFixedLayerMargins, gfx::Point& aOffset);
     bool ProgressiveUpdateCallback(bool aHasPendingNewThebesContent, const gfx::Rect& aDisplayPort, float aDisplayResolution, bool aDrawingCritical, gfx::Rect& aViewport, float& aScaleX, float& aScaleY);
     bool CreateFrame(AutoLocalJNIFrame *jniFrame, AndroidLayerRendererFrame& aFrame);
     bool ActivateProgram(AutoLocalJNIFrame *jniFrame);
@@ -276,6 +286,7 @@ protected:
     static jmethodID jSetFirstPaintViewport;
     static jmethodID jSetPageRect;
     static jmethodID jSyncViewportInfoMethod;
+    static jmethodID jSyncFrameMetricsMethod;
     static jmethodID jCreateFrameMethod;
     static jmethodID jActivateProgramMethod;
     static jmethodID jDeactivateProgramMethod;
@@ -290,262 +301,135 @@ public:
     static jfieldID jDisplayportResolution;
 };
 
-class AndroidKeyEvent
-{
-public:
-    enum {
-        KEYCODE_UNKNOWN            = 0,
-        KEYCODE_SOFT_LEFT          = 1,
-        KEYCODE_SOFT_RIGHT         = 2,
-        KEYCODE_HOME               = 3,
-        KEYCODE_BACK               = 4,
-        KEYCODE_CALL               = 5,
-        KEYCODE_ENDCALL            = 6,
-        KEYCODE_0                  = 7,
-        KEYCODE_1                  = 8,
-        KEYCODE_2                  = 9,
-        KEYCODE_3                  = 10,
-        KEYCODE_4                  = 11,
-        KEYCODE_5                  = 12,
-        KEYCODE_6                  = 13,
-        KEYCODE_7                  = 14,
-        KEYCODE_8                  = 15,
-        KEYCODE_9                  = 16,
-        KEYCODE_STAR               = 17,
-        KEYCODE_POUND              = 18,
-        KEYCODE_DPAD_UP            = 19,
-        KEYCODE_DPAD_DOWN          = 20,
-        KEYCODE_DPAD_LEFT          = 21,
-        KEYCODE_DPAD_RIGHT         = 22,
-        KEYCODE_DPAD_CENTER        = 23,
-        KEYCODE_VOLUME_UP          = 24,
-        KEYCODE_VOLUME_DOWN        = 25,
-        KEYCODE_POWER              = 26,
-        KEYCODE_CAMERA             = 27,
-        KEYCODE_CLEAR              = 28,
-        KEYCODE_A                  = 29,
-        KEYCODE_B                  = 30,
-        KEYCODE_C                  = 31,
-        KEYCODE_D                  = 32,
-        KEYCODE_E                  = 33,
-        KEYCODE_F                  = 34,
-        KEYCODE_G                  = 35,
-        KEYCODE_H                  = 36,
-        KEYCODE_I                  = 37,
-        KEYCODE_J                  = 38,
-        KEYCODE_K                  = 39,
-        KEYCODE_L                  = 40,
-        KEYCODE_M                  = 41,
-        KEYCODE_N                  = 42,
-        KEYCODE_O                  = 43,
-        KEYCODE_P                  = 44,
-        KEYCODE_Q                  = 45,
-        KEYCODE_R                  = 46,
-        KEYCODE_S                  = 47,
-        KEYCODE_T                  = 48,
-        KEYCODE_U                  = 49,
-        KEYCODE_V                  = 50,
-        KEYCODE_W                  = 51,
-        KEYCODE_X                  = 52,
-        KEYCODE_Y                  = 53,
-        KEYCODE_Z                  = 54,
-        KEYCODE_COMMA              = 55,
-        KEYCODE_PERIOD             = 56,
-        KEYCODE_ALT_LEFT           = 57,
-        KEYCODE_ALT_RIGHT          = 58,
-        KEYCODE_SHIFT_LEFT         = 59,
-        KEYCODE_SHIFT_RIGHT        = 60,
-        KEYCODE_TAB                = 61,
-        KEYCODE_SPACE              = 62,
-        KEYCODE_SYM                = 63,
-        KEYCODE_EXPLORER           = 64,
-        KEYCODE_ENVELOPE           = 65,
-        KEYCODE_ENTER              = 66,
-        KEYCODE_DEL                = 67,
-        KEYCODE_GRAVE              = 68,
-        KEYCODE_MINUS              = 69,
-        KEYCODE_EQUALS             = 70,
-        KEYCODE_LEFT_BRACKET       = 71,
-        KEYCODE_RIGHT_BRACKET      = 72,
-        KEYCODE_BACKSLASH          = 73,
-        KEYCODE_SEMICOLON          = 74,
-        KEYCODE_APOSTROPHE         = 75,
-        KEYCODE_SLASH              = 76,
-        KEYCODE_AT                 = 77,
-        KEYCODE_NUM                = 78,
-        KEYCODE_HEADSETHOOK        = 79,
-        KEYCODE_FOCUS              = 80,
-        KEYCODE_PLUS               = 81,
-        KEYCODE_MENU               = 82,
-        KEYCODE_NOTIFICATION       = 83,
-        KEYCODE_SEARCH             = 84,
-        KEYCODE_MEDIA_PLAY_PAUSE   = 85,
-        KEYCODE_MEDIA_STOP         = 86,
-        KEYCODE_MEDIA_NEXT         = 87,
-        KEYCODE_MEDIA_PREVIOUS     = 88,
-        KEYCODE_MEDIA_REWIND       = 89,
-        KEYCODE_MEDIA_FAST_FORWARD = 90,
-        KEYCODE_MUTE               = 91,
-        KEYCODE_PAGE_UP            = 92,
-        KEYCODE_PAGE_DOWN          = 93,
-        KEYCODE_PICTSYMBOLS        = 94,
-        KEYCODE_SWITCH_CHARSET     = 95,
-        KEYCODE_BUTTON_A           = 96,
-        KEYCODE_BUTTON_B           = 97,
-        KEYCODE_BUTTON_C           = 98,
-        KEYCODE_BUTTON_X           = 99,
-        KEYCODE_BUTTON_Y           = 100,
-        KEYCODE_BUTTON_Z           = 101,
-        KEYCODE_BUTTON_L1          = 102,
-        KEYCODE_BUTTON_R1          = 103,
-        KEYCODE_BUTTON_L2          = 104,
-        KEYCODE_BUTTON_R2          = 105,
-        KEYCODE_BUTTON_THUMBL      = 106,
-        KEYCODE_BUTTON_THUMBR      = 107,
-        KEYCODE_BUTTON_START       = 108,
-        KEYCODE_BUTTON_SELECT      = 109,
-        KEYCODE_BUTTON_MODE        = 110,
-        KEYCODE_ESCAPE             = 111,
-        KEYCODE_FORWARD_DEL        = 112,
-        KEYCODE_CTRL_LEFT          = 113,
-        KEYCODE_CTRL_RIGHT         = 114,
-        KEYCODE_CAPS_LOCK          = 115,
-        KEYCODE_SCROLL_LOCK        = 116,
-        KEYCODE_META_LEFT          = 117,
-        KEYCODE_META_RIGHT         = 118,
-        KEYCODE_FUNCTION           = 119,
-        KEYCODE_SYSRQ              = 120,
-        KEYCODE_BREAK              = 121,
-        KEYCODE_MOVE_HOME          = 122,
-        KEYCODE_MOVE_END           = 123,
-        KEYCODE_INSERT             = 124,
-        KEYCODE_FORWARD            = 125,
-        KEYCODE_MEDIA_PLAY         = 126,
-        KEYCODE_MEDIA_PAUSE        = 127,
-        KEYCODE_MEDIA_CLOSE        = 128,
-        KEYCODE_MEDIA_EJECT        = 129,
-        KEYCODE_MEDIA_RECORD       = 130,
-        KEYCODE_F1                 = 131,
-        KEYCODE_F2                 = 132,
-        KEYCODE_F3                 = 133,
-        KEYCODE_F4                 = 134,
-        KEYCODE_F5                 = 135,
-        KEYCODE_F6                 = 136,
-        KEYCODE_F7                 = 137,
-        KEYCODE_F8                 = 138,
-        KEYCODE_F9                 = 139,
-        KEYCODE_F10                = 140,
-        KEYCODE_F11                = 141,
-        KEYCODE_F12                = 142,
-        KEYCODE_NUM_LOCK           = 143,
-        KEYCODE_NUMPAD_0           = 144,
-        KEYCODE_NUMPAD_1           = 145,
-        KEYCODE_NUMPAD_2           = 146,
-        KEYCODE_NUMPAD_3           = 147,
-        KEYCODE_NUMPAD_4           = 148,
-        KEYCODE_NUMPAD_5           = 149,
-        KEYCODE_NUMPAD_6           = 150,
-        KEYCODE_NUMPAD_7           = 151,
-        KEYCODE_NUMPAD_8           = 152,
-        KEYCODE_NUMPAD_9           = 153,
-        KEYCODE_NUMPAD_DIVIDE      = 154,
-        KEYCODE_NUMPAD_MULTIPLY    = 155,
-        KEYCODE_NUMPAD_SUBTRACT    = 156,
-        KEYCODE_NUMPAD_ADD         = 157,
-        KEYCODE_NUMPAD_DOT         = 158,
-        KEYCODE_NUMPAD_COMMA       = 159,
-        KEYCODE_NUMPAD_ENTER       = 160,
-        KEYCODE_NUMPAD_EQUALS      = 161,
-        KEYCODE_NUMPAD_LEFT_PAREN  = 162,
-        KEYCODE_NUMPAD_RIGHT_PAREN = 163,
-        KEYCODE_VOLUME_MUTE        = 164,
-        KEYCODE_INFO               = 165,
-        KEYCODE_CHANNEL_UP         = 166,
-        KEYCODE_CHANNEL_DOWN       = 167,
-        KEYCODE_ZOOM_IN            = 168,
-        KEYCODE_ZOOM_OUT           = 169,
-        KEYCODE_TV                 = 170,
-        KEYCODE_WINDOW             = 171,
-        KEYCODE_GUIDE              = 172,
-        KEYCODE_DVR                = 173,
-        KEYCODE_BOOKMARK           = 174,
-        KEYCODE_CAPTIONS           = 175,
-        KEYCODE_SETTINGS           = 176,
-        KEYCODE_TV_POWER           = 177,
-        KEYCODE_TV_INPUT           = 178,
-        KEYCODE_STB_POWER          = 179,
-        KEYCODE_STB_INPUT          = 180,
-        KEYCODE_AVR_POWER          = 181,
-        KEYCODE_AVR_INPUT          = 182,
-        KEYCODE_PROG_RED           = 183,
-        KEYCODE_PROG_GREEN         = 184,
-        KEYCODE_PROG_YELLOW        = 185,
-        KEYCODE_PROG_BLUE          = 186,
-        KEYCODE_APP_SWITCH         = 187,
-        KEYCODE_BUTTON_1           = 188,
-        KEYCODE_BUTTON_2           = 189,
-        KEYCODE_BUTTON_3           = 190,
-        KEYCODE_BUTTON_4           = 191,
-        KEYCODE_BUTTON_5           = 192,
-        KEYCODE_BUTTON_6           = 193,
-        KEYCODE_BUTTON_7           = 194,
-        KEYCODE_BUTTON_8           = 195,
-        KEYCODE_BUTTON_9           = 196,
-        KEYCODE_BUTTON_10          = 197,
-        KEYCODE_BUTTON_11          = 198,
-        KEYCODE_BUTTON_12          = 199,
-        KEYCODE_BUTTON_13          = 200,
-        KEYCODE_BUTTON_14          = 201,
-        KEYCODE_BUTTON_15          = 202,
-        KEYCODE_BUTTON_16          = 203,
-        KEYCODE_LANGUAGE_SWITCH    = 204,
-        KEYCODE_MANNER_MODE        = 205,
-        KEYCODE_3D_MODE            = 206,
-        KEYCODE_CONTACTS           = 207,
-        KEYCODE_CALENDAR           = 208,
-        KEYCODE_MUSIC              = 209,
-        KEYCODE_CALCULATOR         = 210,
+enum {
+    // These keycode masks are not defined in android/keycodes.h:
+    AKEYCODE_ESCAPE             = 111,
+    AKEYCODE_FORWARD_DEL        = 112,
+    AKEYCODE_CTRL_LEFT          = 113,
+    AKEYCODE_CTRL_RIGHT         = 114,
+    AKEYCODE_CAPS_LOCK          = 115,
+    AKEYCODE_SCROLL_LOCK        = 116,
+    AKEYCODE_META_LEFT          = 117,
+    AKEYCODE_META_RIGHT         = 118,
+    AKEYCODE_FUNCTION           = 119,
+    AKEYCODE_SYSRQ              = 120,
+    AKEYCODE_BREAK              = 121,
+    AKEYCODE_MOVE_HOME          = 122,
+    AKEYCODE_MOVE_END           = 123,
+    AKEYCODE_INSERT             = 124,
+    AKEYCODE_FORWARD            = 125,
+    AKEYCODE_MEDIA_PLAY         = 126,
+    AKEYCODE_MEDIA_PAUSE        = 127,
+    AKEYCODE_MEDIA_CLOSE        = 128,
+    AKEYCODE_MEDIA_EJECT        = 129,
+    AKEYCODE_MEDIA_RECORD       = 130,
+    AKEYCODE_F1                 = 131,
+    AKEYCODE_F2                 = 132,
+    AKEYCODE_F3                 = 133,
+    AKEYCODE_F4                 = 134,
+    AKEYCODE_F5                 = 135,
+    AKEYCODE_F6                 = 136,
+    AKEYCODE_F7                 = 137,
+    AKEYCODE_F8                 = 138,
+    AKEYCODE_F9                 = 139,
+    AKEYCODE_F10                = 140,
+    AKEYCODE_F11                = 141,
+    AKEYCODE_F12                = 142,
+    AKEYCODE_NUM_LOCK           = 143,
+    AKEYCODE_NUMPAD_0           = 144,
+    AKEYCODE_NUMPAD_1           = 145,
+    AKEYCODE_NUMPAD_2           = 146,
+    AKEYCODE_NUMPAD_3           = 147,
+    AKEYCODE_NUMPAD_4           = 148,
+    AKEYCODE_NUMPAD_5           = 149,
+    AKEYCODE_NUMPAD_6           = 150,
+    AKEYCODE_NUMPAD_7           = 151,
+    AKEYCODE_NUMPAD_8           = 152,
+    AKEYCODE_NUMPAD_9           = 153,
+    AKEYCODE_NUMPAD_DIVIDE      = 154,
+    AKEYCODE_NUMPAD_MULTIPLY    = 155,
+    AKEYCODE_NUMPAD_SUBTRACT    = 156,
+    AKEYCODE_NUMPAD_ADD         = 157,
+    AKEYCODE_NUMPAD_DOT         = 158,
+    AKEYCODE_NUMPAD_COMMA       = 159,
+    AKEYCODE_NUMPAD_ENTER       = 160,
+    AKEYCODE_NUMPAD_EQUALS      = 161,
+    AKEYCODE_NUMPAD_LEFT_PAREN  = 162,
+    AKEYCODE_NUMPAD_RIGHT_PAREN = 163,
+    AKEYCODE_VOLUME_MUTE        = 164,
+    AKEYCODE_INFO               = 165,
+    AKEYCODE_CHANNEL_UP         = 166,
+    AKEYCODE_CHANNEL_DOWN       = 167,
+    AKEYCODE_ZOOM_IN            = 168,
+    AKEYCODE_ZOOM_OUT           = 169,
+    AKEYCODE_TV                 = 170,
+    AKEYCODE_WINDOW             = 171,
+    AKEYCODE_GUIDE              = 172,
+    AKEYCODE_DVR                = 173,
+    AKEYCODE_BOOKMARK           = 174,
+    AKEYCODE_CAPTIONS           = 175,
+    AKEYCODE_SETTINGS           = 176,
+    AKEYCODE_TV_POWER           = 177,
+    AKEYCODE_TV_INPUT           = 178,
+    AKEYCODE_STB_POWER          = 179,
+    AKEYCODE_STB_INPUT          = 180,
+    AKEYCODE_AVR_POWER          = 181,
+    AKEYCODE_AVR_INPUT          = 182,
+    AKEYCODE_PROG_RED           = 183,
+    AKEYCODE_PROG_GREEN         = 184,
+    AKEYCODE_PROG_YELLOW        = 185,
+    AKEYCODE_PROG_BLUE          = 186,
+    AKEYCODE_APP_SWITCH         = 187,
+    AKEYCODE_BUTTON_1           = 188,
+    AKEYCODE_BUTTON_2           = 189,
+    AKEYCODE_BUTTON_3           = 190,
+    AKEYCODE_BUTTON_4           = 191,
+    AKEYCODE_BUTTON_5           = 192,
+    AKEYCODE_BUTTON_6           = 193,
+    AKEYCODE_BUTTON_7           = 194,
+    AKEYCODE_BUTTON_8           = 195,
+    AKEYCODE_BUTTON_9           = 196,
+    AKEYCODE_BUTTON_10          = 197,
+    AKEYCODE_BUTTON_11          = 198,
+    AKEYCODE_BUTTON_12          = 199,
+    AKEYCODE_BUTTON_13          = 200,
+    AKEYCODE_BUTTON_14          = 201,
+    AKEYCODE_BUTTON_15          = 202,
+    AKEYCODE_BUTTON_16          = 203,
+    AKEYCODE_LANGUAGE_SWITCH    = 204,
+    AKEYCODE_MANNER_MODE        = 205,
+    AKEYCODE_3D_MODE            = 206,
+    AKEYCODE_CONTACTS           = 207,
+    AKEYCODE_CALENDAR           = 208,
+    AKEYCODE_MUSIC              = 209,
+    AKEYCODE_CALCULATOR         = 210,
+    AKEYCODE_ZENKAKU_HANKAKU    = 211,
+    AKEYCODE_EISU               = 212,
+    AKEYCODE_MUHENKAN           = 213,
+    AKEYCODE_HENKAN             = 214,
+    AKEYCODE_KATAKANA_HIRAGANA  = 215,
+    AKEYCODE_YEN                = 216,
+    AKEYCODE_RO                 = 217,
+    AKEYCODE_KANA               = 218,
+    AKEYCODE_ASSIST             = 219,
 
-        ACTION_DOWN                = 0,
-        ACTION_UP                  = 1,
-        ACTION_MULTIPLE            = 2,
-        META_ALT_ON                = 0x00000002,
-        META_ALT_LEFT_ON           = 0x00000010,
-        META_ALT_RIGHT_ON          = 0x00000020,
-        META_ALT_MASK              = META_ALT_RIGHT_ON | META_ALT_LEFT_ON | META_ALT_ON,
-        META_SHIFT_ON              = 0x00000001,
-        META_SHIFT_LEFT_ON         = 0x00000040,
-        META_SHIFT_RIGHT_ON        = 0x00000080,
-        META_SHIFT_MASK            = META_SHIFT_RIGHT_ON | META_SHIFT_LEFT_ON | META_SHIFT_ON,
-        META_CTRL_ON               = 0x00001000,
-        META_CTRL_LEFT_ON          = 0x00002000,
-        META_CTRL_RIGHT_ON         = 0x00004000,
-        META_CTRL_MASK             = META_CTRL_RIGHT_ON | META_CTRL_LEFT_ON | META_CTRL_ON,
-        META_META_ON               = 0x00010000,
-        META_META_LEFT_ON          = 0x00020000,
-        META_META_RIGHT_ON         = 0x00040000,
-        META_META_MASK             = META_META_RIGHT_ON | META_META_LEFT_ON | META_META_ON,
-        META_SYM_ON                = 0x00000004,
-        FLAG_WOKE_HERE             = 0x00000001,
-        FLAG_SOFT_KEYBOARD         = 0x00000002,
-        FLAG_KEEP_TOUCH_MODE       = 0x00000004,
-        FLAG_FROM_SYSTEM           = 0x00000008,
-        FLAG_EDITOR_ACTION         = 0x00000010,
-        FLAG_CANCELED              = 0x00000020,
-        FLAG_VIRTUAL_HARD_KEY      = 0x00000040,
-        FLAG_LONG_PRESS            = 0x00000080,
-        FLAG_CANCELED_LONG_PRESS   = 0x00000100,
-        FLAG_TRACKING              = 0x00000200,
-        FLAG_START_TRACKING        = 0x40000000,
-        dummy_java_enum_list_end
-    };
+    AMETA_CTRL_ON               = 0x00001000,
+    AMETA_CTRL_LEFT_ON          = 0x00002000,
+    AMETA_CTRL_RIGHT_ON         = 0x00004000,
+    AMETA_META_ON               = 0x00010000,
+    AMETA_META_LEFT_ON          = 0x00020000,
+    AMETA_META_RIGHT_ON         = 0x00040000,
+
+    AMETA_ALT_MASK              = AMETA_ALT_LEFT_ON   | AMETA_ALT_RIGHT_ON   | AMETA_ALT_ON,
+    AMETA_CTRL_MASK             = AMETA_CTRL_LEFT_ON  | AMETA_CTRL_RIGHT_ON  | AMETA_CTRL_ON,
+    AMETA_META_MASK             = AMETA_META_LEFT_ON  | AMETA_META_RIGHT_ON  | AMETA_META_ON,
+    AMETA_SHIFT_MASK            = AMETA_SHIFT_LEFT_ON | AMETA_SHIFT_RIGHT_ON | AMETA_SHIFT_ON,
 };
 
 class AndroidMotionEvent
 {
 public:
     enum {
-        ACTION_MASK = 0xff,
         ACTION_DOWN = 0,
         ACTION_UP = 1,
         ACTION_MOVE = 2,
@@ -559,8 +443,6 @@ public:
         ACTION_MAGNIFY_START = 11,
         ACTION_MAGNIFY = 12,
         ACTION_MAGNIFY_END = 13,
-        ACTION_POINTER_ID_MASK = 0xff00,
-        ACTION_POINTER_ID_SHIFT = 8,
         EDGE_TOP = 0x00000001,
         EDGE_BOTTOM = 0x00000002,
         EDGE_LEFT = 0x00000004,
@@ -591,31 +473,57 @@ public:
 
 class AndroidGeckoEvent : public WrappedJavaObject
 {
-public:
-    static void InitGeckoEventClass(JNIEnv *jEnv);
-
-    AndroidGeckoEvent(int aType) {
-        Init(aType);
-    }
-    AndroidGeckoEvent(int aType, int aAction) {
-        Init(aType, aAction);
-    }
-    AndroidGeckoEvent(int aType, const nsIntRect &aRect) {
-        Init(aType, aRect);
-    }
-    AndroidGeckoEvent(JNIEnv *jenv, jobject jobj) {
-        Init(jenv, jobj);
-    }
-    AndroidGeckoEvent(AndroidGeckoEvent *aResizeEvent) {
-        Init(aResizeEvent);
+private:
+    AndroidGeckoEvent() {
     }
 
     void Init(JNIEnv *jenv, jobject jobj);
     void Init(int aType);
-    void Init(int aType, int aAction);
-    void Init(int aType, const nsIntRect &aRect);
     void Init(AndroidGeckoEvent *aResizeEvent);
 
+public:
+    static void InitGeckoEventClass(JNIEnv *jEnv);
+
+    static AndroidGeckoEvent* MakeNativePoke() {
+        AndroidGeckoEvent *event = new AndroidGeckoEvent();
+        event->Init(NATIVE_POKE);
+        return event;
+    }
+
+    static AndroidGeckoEvent* MakeIMEEvent(int aAction) {
+        AndroidGeckoEvent *event = new AndroidGeckoEvent();
+        event->Init(IME_EVENT);
+        event->mAction = aAction;
+        return event;
+    }
+
+    static AndroidGeckoEvent* MakeDrawEvent(const nsIntRect& aRect) {
+        AndroidGeckoEvent *event = new AndroidGeckoEvent();
+        event->Init(DRAW);
+        event->mRect = aRect;
+        return event;
+    }
+
+    static AndroidGeckoEvent* MakeFromJavaObject(JNIEnv *jenv, jobject jobj) {
+        AndroidGeckoEvent *event = new AndroidGeckoEvent();
+        event->Init(jenv, jobj);
+        return event;
+    }
+
+    static AndroidGeckoEvent* CopyResizeEvent(AndroidGeckoEvent *aResizeEvent) {
+        AndroidGeckoEvent *event = new AndroidGeckoEvent();
+        event->Init(aResizeEvent);
+        return event;
+    }
+
+    static AndroidGeckoEvent* MakeBroadcastEvent(const nsCString& topic, const nsCString& data) {
+        AndroidGeckoEvent* event = new AndroidGeckoEvent();
+        event->Init(BROADCAST);
+        CopyUTF8toUTF16(topic, event->mCharacters);
+        CopyUTF8toUTF16(data, event->mCharactersExtra);
+        return event;
+    }
+
     int Action() { return mAction; }
     int Type() { return mType; }
     bool AckNeeded() { return mAckNeeded; }
@@ -633,11 +541,11 @@ public:
     nsAString& CharactersExtra() { return mCharactersExtra; }
     int KeyCode() { return mKeyCode; }
     int MetaState() { return mMetaState; }
-    int DomKeyLocation() { return mDomKeyLocation; }
-    bool IsAltPressed() const { return (mMetaState & AndroidKeyEvent::META_ALT_MASK) != 0; }
-    bool IsShiftPressed() const { return (mMetaState & AndroidKeyEvent::META_SHIFT_MASK) != 0; }
-    bool IsCtrlPressed() const { return (mMetaState & AndroidKeyEvent::META_CTRL_MASK) != 0; }
-    bool IsMetaPressed() const { return (mMetaState & AndroidKeyEvent::META_META_MASK) != 0; }
+    uint32_t DomKeyLocation() { return mDomKeyLocation; }
+    bool IsAltPressed() const { return (mMetaState & AMETA_ALT_MASK) != 0; }
+    bool IsShiftPressed() const { return (mMetaState & AMETA_SHIFT_MASK) != 0; }
+    bool IsCtrlPressed() const { return (mMetaState & AMETA_CTRL_MASK) != 0; }
+    bool IsMetaPressed() const { return (mMetaState & AMETA_META_MASK) != 0; }
     int Flags() { return mFlags; }
     int UnicodeChar() { return mUnicodeChar; }
     int BaseUnicodeChar() { return mBaseUnicodeChar; }
@@ -660,6 +568,9 @@ public:
     RefCountedJavaObject* ByteBuffer() { return mByteBuffer; }
     int Width() { return mWidth; }
     int Height() { return mHeight; }
+    nsTouchEvent MakeTouchEvent(nsIWidget* widget);
+    MultiTouchInput MakeMultiTouchInput(nsIWidget* widget);
+    void UnionRect(nsIntRect const& aRect);
 
 protected:
     int mAction;
@@ -673,7 +584,7 @@ protected:
     nsTArray<float> mPressures;
     nsIntRect mRect;
     int mFlags, mMetaState;
-    int mDomKeyLocation;
+    uint32_t mDomKeyLocation;
     int mKeyCode, mUnicodeChar, mBaseUnicodeChar;
     int mRepeatCount;
     int mCount;
@@ -707,6 +618,8 @@ protected:
     void ReadCharactersField(JNIEnv *jenv);
     void ReadCharactersExtraField(JNIEnv *jenv);
 
+    uint32_t ReadDomKeyLocation(JNIEnv* jenv, jobject jGeckoEventObj);
+
     static jclass jGeckoEventClass;
     static jfieldID jActionField;
     static jfieldID jTypeField;
@@ -755,6 +668,9 @@ protected:
     static jfieldID jWidthField;
     static jfieldID jHeightField;
 
+    static jclass jDomKeyLocationClass;
+    static jfieldID jDomKeyLocationValueField;
+
 public:
     enum {
         NATIVE_POKE = 0,
