@@ -34,6 +34,7 @@ using mozilla::plugins::PluginInstanceParent;
 #include "nsRenderingContext.h"
 #include "prmem.h"
 #include "WinUtils.h"
+#include "nsIWidgetListener.h"
 #include "mozilla/unused.h"
 
 #include "LayerManagerOGL.h"
@@ -70,16 +71,16 @@ using namespace mozilla::widget;
 /**************************************************************
  *
  * SECTION: nsWindow statics
- * 
+ *
  **************************************************************/
 
-static nsAutoPtr<PRUint8>  sSharedSurfaceData;
+static nsAutoPtr<uint8_t>  sSharedSurfaceData;
 static gfxIntSize          sSharedSurfaceSize;
 
 struct IconMetrics {
-  PRInt32 xMetric;
-  PRInt32 yMetric;
-  PRInt32 defaultSize;
+  int32_t xMetric;
+  int32_t yMetric;
+  int32_t defaultSize;
 };
 
 // Corresponds 1:1 to the IconSizeType enum
@@ -104,36 +105,6 @@ IsRenderMode(gfxWindowsPlatform::RenderMode rmode)
   return gfxWindowsPlatform::GetPlatform()->GetRenderMode() == rmode;
 }
 
-nsIntRegion
-nsWindowGfx::ConvertHRGNToRegion(HRGN aRgn)
-{
-  NS_ASSERTION(aRgn, "Don't pass NULL region here");
-
-  nsIntRegion rgn;
-
-  DWORD size = ::GetRegionData(aRgn, 0, NULL);
-  nsAutoTArray<PRUint8,100> buffer;
-  if (!buffer.SetLength(size))
-    return rgn;
-
-  RGNDATA* data = reinterpret_cast<RGNDATA*>(buffer.Elements());
-  if (!::GetRegionData(aRgn, size, data))
-    return rgn;
-
-  if (data->rdh.nCount > MAX_RECTS_IN_REGION) {
-    rgn = ToIntRect(data->rdh.rcBound);
-    return rgn;
-  }
-
-  RECT* rects = reinterpret_cast<RECT*>(data->Buffer);
-  for (PRUint32 i = 0; i < data->rdh.nCount; ++i) {
-    RECT* r = rects + i;
-    rgn.Or(rgn, ToIntRect(*r));
-  }
-
-  return rgn;
-}
-
 /**************************************************************
  **************************************************************
  **
@@ -151,7 +122,7 @@ nsIntRegion nsWindow::GetRegionToPaint(bool aForceFullRepaint,
   if (aForceFullRepaint) {
     RECT paintRect;
     ::GetClientRect(mWnd, &paintRect);
-    return nsIntRegion(nsWindowGfx::ToIntRect(paintRect));
+    return nsIntRegion(WinUtils::ToIntRect(paintRect));
   }
 
   HRGN paintRgn = ::CreateRectRgn(0, 0, 0, 0);
@@ -162,11 +133,11 @@ nsIntRegion nsWindow::GetRegionToPaint(bool aForceFullRepaint,
       ::MapWindowPoints(NULL, mWnd, &pt, 1);
       ::OffsetRgn(paintRgn, pt.x, pt.y);
     }
-    nsIntRegion rgn(nsWindowGfx::ConvertHRGNToRegion(paintRgn));
+    nsIntRegion rgn(WinUtils::ConvertHRGNToRegion(paintRgn));
     ::DeleteObject(paintRgn);
     return rgn;
   }
-  return nsIntRegion(nsWindowGfx::ToIntRect(ps.rcPaint));
+  return nsIntRegion(WinUtils::ToIntRect(ps.rcPaint));
 }
 
 #define WORDSSIZE(x) ((x).width * (x).height)
@@ -185,14 +156,21 @@ EnsureSharedSurfaceSize(gfxIntSize size)
 
   if (!sSharedSurfaceData || (WORDSSIZE(size) > WORDSSIZE(sSharedSurfaceSize))) {
     sSharedSurfaceSize = size;
-    sSharedSurfaceData = nsnull;
-    sSharedSurfaceData = (PRUint8 *)malloc(WORDSSIZE(sSharedSurfaceSize) * 4);
+    sSharedSurfaceData = nullptr;
+    sSharedSurfaceData = (uint8_t *)malloc(WORDSSIZE(sSharedSurfaceSize) * 4);
   }
 
-  return (sSharedSurfaceData != nsnull);
+  return (sSharedSurfaceData != nullptr);
 }
 
-bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
+nsIWidgetListener* nsWindow::GetPaintListener()
+{
+  if (mDestroyCalled)
+    return nullptr;
+  return mAttachedWidgetListener ? mAttachedWidgetListener : mWidgetListener;
+}
+
+bool nsWindow::OnPaint(HDC aDC, uint32_t aNestingLevel)
 {
   // We never have reentrant paint events, except when we're running our RPC
   // windows event spin loop. If we don't trap for this, we'll try to paint,
@@ -234,13 +212,17 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
     return true;
   }
 
-  nsPaintEvent willPaintEvent(true, NS_WILL_PAINT, this);
-  willPaintEvent.willSendDidPaint = true;
-  DispatchWindowEvent(&willPaintEvent);
+  nsIWidgetListener* listener = GetPaintListener();
+  if (listener) {
+    listener->WillPaintWindow(this);
+  }
+  // Re-get the listener since the will paint notification may have killed it.
+  listener = GetPaintListener();
+  if (!listener)
+    return false;
 
   bool result = true;
   PAINTSTRUCT ps;
-  nsEventStatus eventStatus = nsEventStatus_eIgnore;
 
 #ifdef MOZ_XUL
   if (!aDC && (eTransparencyTransparent == mTransparencyMode))
@@ -277,20 +259,13 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
     mPaintDC = hDC;
   }
 
-  // generate the event and call the event callback
-  nsPaintEvent event(true, NS_PAINT, this);
-  InitEvent(event);
-
 #ifdef MOZ_XUL
   bool forceRepaint = aDC || (eTransparencyTransparent == mTransparencyMode);
 #else
   bool forceRepaint = NULL != aDC;
 #endif
-  event.region = GetRegionToPaint(forceRepaint, ps, hDC);
-  event.willSendDidPaint = true;
-  event.didSendWillPaint = true;
-
-  if (!event.region.IsEmpty() && mEventCallback)
+  nsIntRegion region = GetRegionToPaint(forceRepaint, ps, hDC);
+  if (!region.IsEmpty() && listener)
   {
     // Should probably pass in a real region here, using GetRandomRgn
     // http://msdn.microsoft.com/library/default.asp?url=/library/en-us/gdi/clipping_4q0e.asp
@@ -298,13 +273,13 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
 #ifdef WIDGET_DEBUG_OUTPUT
     debug_DumpPaintEvent(stdout,
                          this,
-                         &event,
-                         nsCAutoString("noname"),
-                         (PRInt32) mWnd);
+                         region,
+                         nsAutoCString("noname"),
+                         (int32_t) mWnd);
 #endif // WIDGET_DEBUG_OUTPUT
 
     switch (GetLayerManager()->GetBackendType()) {
-      case LayerManager::LAYERS_BASIC:
+      case LAYERS_BASIC:
         {
           nsRefPtr<gfxASurface> targetSurface;
 
@@ -313,7 +288,7 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
           if ((IsRenderMode(gfxWindowsPlatform::RENDER_GDI) ||
                IsRenderMode(gfxWindowsPlatform::RENDER_DIRECT2D)) &&
               eTransparencyTransparent == mTransparencyMode) {
-            if (mTransparentSurface == nsnull)
+            if (mTransparentSurface == nullptr)
               SetupTranslucentWindowMemoryBitmap(mTransparencyMode);
             targetSurface = mTransparentSurface;
           }
@@ -335,7 +310,7 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
             if (!mD2DWindowSurface->CairoStatus()) {
               targetSurface = mD2DWindowSurface;
             } else {
-              mD2DWindowSurface = nsnull;
+              mD2DWindowSurface = nullptr;
             }
           }
 #endif
@@ -345,7 +320,7 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
               (IsRenderMode(gfxWindowsPlatform::RENDER_GDI) ||
                IsRenderMode(gfxWindowsPlatform::RENDER_DIRECT2D)))
           {
-            PRUint32 flags = (mTransparencyMode == eTransparencyOpaque) ? 0 :
+            uint32_t flags = (mTransparencyMode == eTransparencyOpaque) ? 0 :
                 gfxWindowsSurface::FLAG_IS_TRANSPARENT;
             targetSurfaceWin = new gfxWindowsSurface(hDC, flags);
             targetSurface = targetSurfaceWin;
@@ -385,8 +360,8 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
           nsRefPtr<gfxContext> thebesContext = new gfxContext(targetSurface);
           if (IsRenderMode(gfxWindowsPlatform::RENDER_DIRECT2D)) {
             const nsIntRect* r;
-            for (nsIntRegionRectIterator iter(event.region);
-                 (r = iter.Next()) != nsnull;) {
+            for (nsIntRegionRectIterator iter(region);
+                 (r = iter.Next()) != nullptr;) {
               thebesContext->Rectangle(gfxRect(r->x, r->y, r->width, r->height), true);
             }
             thebesContext->Clip();
@@ -396,8 +371,7 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
           }
 
           // don't need to double buffer with anything but GDI
-          BasicLayerManager::BufferMode doubleBuffering =
-            BasicLayerManager::BUFFER_NONE;
+          BufferMode doubleBuffering = mozilla::layers::BUFFER_NONE;
           if (IsRenderMode(gfxWindowsPlatform::RENDER_GDI)) {
 #ifdef MOZ_XUL
             switch (mTransparencyMode) {
@@ -405,7 +379,7 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
               case eTransparencyBorderlessGlass:
               default:
                 // If we're not doing translucency, then double buffer
-                doubleBuffering = BasicLayerManager::BUFFER_BUFFERED;
+                doubleBuffering = mozilla::layers::BUFFER_BUFFERED;
                 break;
               case eTransparencyTransparent:
                 // If we're rendering with translucency, we're going to be
@@ -416,14 +390,14 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
                 break;
             }
 #else
-            doubleBuffering = BasicLayerManager::BUFFER_BUFFERED;
+            doubleBuffering = mozilla::layers::BUFFER_BUFFERED;
 #endif
           }
 
           {
             AutoLayerManagerSetup
                 setupLayerManager(this, thebesContext, doubleBuffering);
-            result = DispatchWindowEvent(&event, eventStatus);
+            result = listener->PaintWindow(this, region, 0);
           }
 
 #ifdef MOZ_XUL
@@ -535,21 +509,21 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
           }
         }
         break;
-      case LayerManager::LAYERS_OPENGL:
+      case LAYERS_OPENGL:
         static_cast<mozilla::layers::LayerManagerOGL*>(GetLayerManager())->
-          SetClippingRegion(event.region);
-        result = DispatchWindowEvent(&event, eventStatus);
+          SetClippingRegion(region);
+        result = listener->PaintWindow(this, region, 0);
         break;
 #ifdef MOZ_ENABLE_D3D9_LAYER
-      case LayerManager::LAYERS_D3D9:
+      case LAYERS_D3D9:
         {
-          LayerManagerD3D9 *layerManagerD3D9 =
+          nsRefPtr<LayerManagerD3D9> layerManagerD3D9 =
             static_cast<mozilla::layers::LayerManagerD3D9*>(GetLayerManager());
-          layerManagerD3D9->SetClippingRegion(event.region);
-          result = DispatchWindowEvent(&event, eventStatus);
+          layerManagerD3D9->SetClippingRegion(region);
+          result = listener->PaintWindow(this, region, 0);
           if (layerManagerD3D9->DeviceWasRemoved()) {
             mLayerManager->Destroy();
-            mLayerManager = nsnull;
+            mLayerManager = nullptr;
             // When our device was removed, we should have gfxWindowsPlatform
             // check if its render mode is up to date!
             gfxWindowsPlatform::GetPlatform()->UpdateRenderMode();
@@ -559,18 +533,22 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
         break;
 #endif
 #ifdef MOZ_ENABLE_D3D10_LAYER
-      case LayerManager::LAYERS_D3D10:
+      case LAYERS_D3D10:
         {
           gfxWindowsPlatform::GetPlatform()->UpdateRenderMode();
           LayerManagerD3D10 *layerManagerD3D10 = static_cast<mozilla::layers::LayerManagerD3D10*>(GetLayerManager());
           if (layerManagerD3D10->device() != gfxWindowsPlatform::GetPlatform()->GetD3D10Device()) {
             Invalidate();
           } else {
-            result = DispatchWindowEvent(&event, eventStatus);
+            result = listener->PaintWindow(this, region, 0);
           }
         }
         break;
 #endif
+      case LAYERS_CLIENT:
+        // Do nothing, the compositor will handle drawing.
+        break;
+
       default:
         NS_ERROR("Unknown layers backend used!");
         break;
@@ -581,7 +559,7 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
     ::EndPaint(mWnd, &ps);
   }
 
-  mPaintDC = nsnull;
+  mPaintDC = nullptr;
   mLastPaintEndTime = TimeStamp::Now();
 
 #if defined(WIDGET_DEBUG_OUTPUT)
@@ -590,7 +568,7 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
     // Only flash paint events which have not ignored the paint message.
     // Those that ignore the paint message aren't painting anything so there
     // is only the overhead of the dispatching the paint event.
-    if (nsEventStatus_eIgnore != eventStatus) {
+    if (result) {
       ::InvertRgn(debugPaintFlashDC, debugPaintFlashRegion);
       PR_Sleep(PR_MillisecondsToInterval(30));
       ::InvertRgn(debugPaintFlashDC, debugPaintFlashRegion);
@@ -603,8 +581,10 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
 
   mPainting = false;
 
-  nsPaintEvent didPaintEvent(true, NS_DID_PAINT, this);
-  DispatchWindowEvent(&didPaintEvent);
+  // Re-get the listener since painting may have killed it.
+  listener = GetPaintListener();
+  if (listener)
+    listener->DidPaintWindow();
 
   if (aNestingLevel == 0 && ::GetUpdateRect(mWnd, NULL, false)) {
     OnPaint(aDC, 1);
@@ -614,8 +594,8 @@ bool nsWindow::OnPaint(HDC aDC, PRUint32 aNestingLevel)
 }
 
 gfxIntSize nsWindowGfx::GetIconMetrics(IconSizeType aSizeType) {
-  PRInt32 width = ::GetSystemMetrics(sIconMetrics[aSizeType].xMetric);
-  PRInt32 height = ::GetSystemMetrics(sIconMetrics[aSizeType].yMetric);
+  int32_t width = ::GetSystemMetrics(sIconMetrics[aSizeType].xMetric);
+  int32_t height = ::GetSystemMetrics(sIconMetrics[aSizeType].yMetric);
 
   if (width == 0 || height == 0) {
     width = height = sIconMetrics[aSizeType].defaultSize;
@@ -626,25 +606,29 @@ gfxIntSize nsWindowGfx::GetIconMetrics(IconSizeType aSizeType) {
 
 nsresult nsWindowGfx::CreateIcon(imgIContainer *aContainer,
                                   bool aIsCursor,
-                                  PRUint32 aHotspotX,
-                                  PRUint32 aHotspotY,
+                                  uint32_t aHotspotX,
+                                  uint32_t aHotspotY,
                                   gfxIntSize aScaledSize,
                                   HICON *aIcon) {
 
   // Get the image data
-  nsRefPtr<gfxImageSurface> frame;
-  aContainer->CopyFrame(imgIContainer::FRAME_CURRENT,
-                        imgIContainer::FLAG_SYNC_DECODE,
-                        getter_AddRefs(frame));
-  if (!frame)
-    return NS_ERROR_NOT_AVAILABLE;
+  nsRefPtr<gfxASurface> surface;
+  aContainer->GetFrame(imgIContainer::FRAME_CURRENT,
+                       imgIContainer::FLAG_SYNC_DECODE,
+                       getter_AddRefs(surface));
+  NS_ENSURE_TRUE(surface, NS_ERROR_NOT_AVAILABLE);
 
-  PRInt32 width = frame->Width();
-  PRInt32 height = frame->Height();
+  nsRefPtr<gfxImageSurface> frame(surface->GetAsReadableARGB32ImageSurface());
+  NS_ENSURE_TRUE(frame, NS_ERROR_NOT_AVAILABLE);
+
+  int32_t width = frame->Width();
+  int32_t height = frame->Height();
   if (!width || !height)
     return NS_ERROR_FAILURE;
 
-  PRUint8 *data;
+  uint8_t *data;
+  nsRefPtr<gfxImageSurface> dest;
+
   if ((aScaledSize.width == 0 && aScaledSize.height == 0) ||
       (aScaledSize.width == width && aScaledSize.height == height)) {
     // We're not scaling the image. The data is simply what's in the frame.
@@ -654,8 +638,7 @@ nsresult nsWindowGfx::CreateIcon(imgIContainer *aContainer,
     NS_ENSURE_ARG(aScaledSize.width > 0);
     NS_ENSURE_ARG(aScaledSize.height > 0);
     // Draw a scaled version of the image to a temporary surface
-    nsRefPtr<gfxImageSurface> dest = new gfxImageSurface(aScaledSize,
-                                                         gfxASurface::ImageFormatARGB32);
+    dest = new gfxImageSurface(aScaledSize, gfxASurface::ImageFormatARGB32);
     if (!dest)
       return NS_ERROR_OUT_OF_MEMORY;
 
@@ -677,7 +660,7 @@ nsresult nsWindowGfx::CreateIcon(imgIContainer *aContainer,
   }
 
   HBITMAP bmp = DataToBitmap(data, width, -height, 32);
-  PRUint8* a1data = Data32BitTo1Bit(data, width, height);
+  uint8_t* a1data = Data32BitTo1Bit(data, width, height);
   if (!a1data) {
     return NS_ERROR_FAILURE;
   }
@@ -702,23 +685,23 @@ nsresult nsWindowGfx::CreateIcon(imgIContainer *aContainer,
 }
 
 // Adjust cursor image data
-PRUint8* nsWindowGfx::Data32BitTo1Bit(PRUint8* aImageData,
-                                      PRUint32 aWidth, PRUint32 aHeight)
+uint8_t* nsWindowGfx::Data32BitTo1Bit(uint8_t* aImageData,
+                                      uint32_t aWidth, uint32_t aHeight)
 {
   // We need (aWidth + 7) / 8 bytes plus zero-padding up to a multiple of
   // 4 bytes for each row (HBITMAP requirement). Bug 353553.
-  PRUint32 outBpr = ((aWidth + 31) / 8) & ~3;
+  uint32_t outBpr = ((aWidth + 31) / 8) & ~3;
 
   // Allocate and clear mask buffer
-  PRUint8* outData = (PRUint8*)PR_Calloc(outBpr, aHeight);
+  uint8_t* outData = (uint8_t*)PR_Calloc(outBpr, aHeight);
   if (!outData)
     return NULL;
 
-  PRInt32 *imageRow = (PRInt32*)aImageData;
-  for (PRUint32 curRow = 0; curRow < aHeight; curRow++) {
-    PRUint8 *outRow = outData + curRow * outBpr;
-    PRUint8 mask = 0x80;
-    for (PRUint32 curCol = 0; curCol < aWidth; curCol++) {
+  int32_t *imageRow = (int32_t*)aImageData;
+  for (uint32_t curRow = 0; curRow < aHeight; curRow++) {
+    uint8_t *outRow = outData + curRow * outBpr;
+    uint8_t mask = 0x80;
+    for (uint32_t curCol = 0; curCol < aWidth; curCol++) {
       // Use sign bit to test for transparency, as alpha byte is highest byte
       if (*imageRow++ < 0)
         *outRow |= mask;
@@ -748,10 +731,10 @@ PRUint8* nsWindowGfx::Data32BitTo1Bit(PRUint8* aImageData,
  *         DeleteObject when done with the bitmap.
  *         On failure, NULL will be returned.
  */
-HBITMAP nsWindowGfx::DataToBitmap(PRUint8* aImageData,
-                                  PRUint32 aWidth,
-                                  PRUint32 aHeight,
-                                  PRUint32 aDepth)
+HBITMAP nsWindowGfx::DataToBitmap(uint8_t* aImageData,
+                                  uint32_t aWidth,
+                                  uint32_t aHeight,
+                                  uint32_t aDepth)
 {
   HDC dc = ::GetDC(NULL);
 
