@@ -20,13 +20,16 @@
 #include <string.h>
 #include "nsBinaryStream.h"
 #include "nsCRT.h"
-#include "nsIStreamBufferAccess.h"
 #include "prlong.h"
 #include "nsString.h"
 #include "nsISerializable.h"
 #include "nsIClassInfo.h"
 #include "nsComponentManagerUtils.h"
 #include "nsIURI.h" // for NS_IURI_IID
+#include "mozilla/Endian.h"
+
+#include "jsapi.h"
+#include "jsfriendapi.h"
 
 NS_IMPL_ISUPPORTS3(nsBinaryOutputStream, nsIObjectOutputStream, nsIBinaryOutputStream, nsIOutputStream)
 
@@ -45,21 +48,21 @@ nsBinaryOutputStream::Close()
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::Write(const char *aBuf, PRUint32 aCount, PRUint32 *aActualBytes)
+nsBinaryOutputStream::Write(const char *aBuf, uint32_t aCount, uint32_t *aActualBytes)
 {
     NS_ENSURE_STATE(mOutputStream);
     return mOutputStream->Write(aBuf, aCount, aActualBytes);
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::WriteFrom(nsIInputStream *inStr, PRUint32 count, PRUint32 *_retval)
+nsBinaryOutputStream::WriteFrom(nsIInputStream *inStr, uint32_t count, uint32_t *_retval)
 {
     NS_NOTREACHED("WriteFrom");
     return NS_ERROR_NOT_IMPLEMENTED;
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::WriteSegments(nsReadSegmentFun reader, void * closure, PRUint32 count, PRUint32 *_retval)
+nsBinaryOutputStream::WriteSegments(nsReadSegmentFun reader, void * closure, uint32_t count, uint32_t *_retval)
 {
     NS_NOTREACHED("WriteSegments");
     return NS_ERROR_NOT_IMPLEMENTED;
@@ -73,12 +76,12 @@ nsBinaryOutputStream::IsNonBlocking(bool *aNonBlocking)
 }
 
 nsresult
-nsBinaryOutputStream::WriteFully(const char *aBuf, PRUint32 aCount)
+nsBinaryOutputStream::WriteFully(const char *aBuf, uint32_t aCount)
 {
     NS_ENSURE_STATE(mOutputStream);
 
     nsresult rv;
-    PRUint32 bytesWritten;
+    uint32_t bytesWritten;
 
     rv = mOutputStream->Write(aBuf, aCount, &bytesWritten);
     if (NS_FAILED(rv)) return rv;
@@ -103,32 +106,32 @@ nsBinaryOutputStream::WriteBoolean(bool aBoolean)
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::Write8(PRUint8 aByte)
+nsBinaryOutputStream::Write8(uint8_t aByte)
 {
     return WriteFully((const char*)&aByte, sizeof aByte);
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::Write16(PRUint16 a16)
+nsBinaryOutputStream::Write16(uint16_t a16)
 {
-    a16 = NS_SWAP16(a16);
+    a16 = mozilla::NativeEndian::swapToBigEndian(a16);
     return WriteFully((const char*)&a16, sizeof a16);
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::Write32(PRUint32 a32)
+nsBinaryOutputStream::Write32(uint32_t a32)
 {
-    a32 = NS_SWAP32(a32);
+    a32 = mozilla::NativeEndian::swapToBigEndian(a32);
     return WriteFully((const char*)&a32, sizeof a32);
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::Write64(PRUint64 a64)
+nsBinaryOutputStream::Write64(uint64_t a64)
 {
     nsresult rv;
-    PRUint32 bytesWritten;
+    uint32_t bytesWritten;
 
-    a64 = NS_SWAP64(a64);
+    a64 = mozilla::NativeEndian::swapToBigEndian(a64);
     rv = Write(reinterpret_cast<char*>(&a64), sizeof a64, &bytesWritten);
     if (NS_FAILED(rv)) return rv;
     if (bytesWritten != sizeof a64)
@@ -139,23 +142,23 @@ nsBinaryOutputStream::Write64(PRUint64 a64)
 NS_IMETHODIMP
 nsBinaryOutputStream::WriteFloat(float aFloat)
 {
-    NS_ASSERTION(sizeof(float) == sizeof (PRUint32),
+    NS_ASSERTION(sizeof(float) == sizeof (uint32_t),
                  "False assumption about sizeof(float)");
-    return Write32(*reinterpret_cast<PRUint32*>(&aFloat));
+    return Write32(*reinterpret_cast<uint32_t*>(&aFloat));
 }
 
 NS_IMETHODIMP
 nsBinaryOutputStream::WriteDouble(double aDouble)
 {
-    NS_ASSERTION(sizeof(double) == sizeof(PRUint64),
+    NS_ASSERTION(sizeof(double) == sizeof(uint64_t),
                  "False assumption about sizeof(double)");
-    return Write64(*reinterpret_cast<PRUint64*>(&aDouble));
+    return Write64(*reinterpret_cast<uint64_t*>(&aDouble));
 }
 
 NS_IMETHODIMP
 nsBinaryOutputStream::WriteStringZ(const char *aString)
 {
-    PRUint32 length;
+    uint32_t length;
     nsresult rv;
 
     length = strlen(aString);
@@ -167,7 +170,7 @@ nsBinaryOutputStream::WriteStringZ(const char *aString)
 NS_IMETHODIMP
 nsBinaryOutputStream::WriteWStringZ(const PRUnichar* aString)
 {
-    PRUint32 length, byteCount;
+    uint32_t length, byteCount;
     nsresult rv;
 
     length = NS_strlen(aString);
@@ -190,9 +193,8 @@ nsBinaryOutputStream::WriteWStringZ(const PRUnichar* aString)
         if (!copy)
             return NS_ERROR_OUT_OF_MEMORY;
     }
-    NS_ASSERTION((PRUptrdiff(aString) & 0x1) == 0, "aString not properly aligned");
-    for (PRUint32 i = 0; i < length; i++)
-        copy[i] = NS_SWAP16(aString[i]);
+    NS_ASSERTION((uintptr_t(aString) & 0x1) == 0, "aString not properly aligned");
+    mozilla::NativeEndian::copyAndSwapToBigEndian(copy, aString, length);
     rv = WriteBytes(reinterpret_cast<const char*>(copy), byteCount);
     if (copy != temp)
         moz_free(copy);
@@ -208,10 +210,10 @@ nsBinaryOutputStream::WriteUtf8Z(const PRUnichar* aString)
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::WriteBytes(const char *aString, PRUint32 aLength)
+nsBinaryOutputStream::WriteBytes(const char *aString, uint32_t aLength)
 {
     nsresult rv;
-    PRUint32 bytesWritten;
+    uint32_t bytesWritten;
 
     rv = Write(aString, aLength, &bytesWritten);
     if (NS_FAILED(rv)) return rv;
@@ -221,7 +223,7 @@ nsBinaryOutputStream::WriteBytes(const char *aString, PRUint32 aLength)
 }
 
 NS_IMETHODIMP
-nsBinaryOutputStream::WriteByteArray(PRUint8 *aBytes, PRUint32 aLength)
+nsBinaryOutputStream::WriteByteArray(uint8_t *aBytes, uint32_t aLength)
 {
     return WriteBytes(reinterpret_cast<char *>(aBytes), aLength);
 }
@@ -287,15 +289,15 @@ nsBinaryOutputStream::WriteID(const nsIID& aIID)
 }
 
 NS_IMETHODIMP_(char*)
-nsBinaryOutputStream::GetBuffer(PRUint32 aLength, PRUint32 aAlignMask)
+nsBinaryOutputStream::GetBuffer(uint32_t aLength, uint32_t aAlignMask)
 {
     if (mBufferAccess)
         return mBufferAccess->GetBuffer(aLength, aAlignMask);
-    return nsnull;
+    return nullptr;
 }
 
 NS_IMETHODIMP_(void)
-nsBinaryOutputStream::PutBuffer(char* aBuffer, PRUint32 aLength)
+nsBinaryOutputStream::PutBuffer(char* aBuffer, uint32_t aLength)
 {
     if (mBufferAccess)
         mBufferAccess->PutBuffer(aBuffer, aLength);
@@ -304,21 +306,21 @@ nsBinaryOutputStream::PutBuffer(char* aBuffer, PRUint32 aLength)
 NS_IMPL_ISUPPORTS3(nsBinaryInputStream, nsIObjectInputStream, nsIBinaryInputStream, nsIInputStream)
 
 NS_IMETHODIMP
-nsBinaryInputStream::Available(PRUint32* aResult)
+nsBinaryInputStream::Available(uint64_t* aResult)
 {
     NS_ENSURE_STATE(mInputStream);
     return mInputStream->Available(aResult);
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::Read(char* aBuffer, PRUint32 aCount, PRUint32 *aNumRead)
+nsBinaryInputStream::Read(char* aBuffer, uint32_t aCount, uint32_t *aNumRead)
 {
     NS_ENSURE_STATE(mInputStream);
 
     // mInputStream might give us short reads, so deal with that.
-    PRUint32 totalRead = 0;
+    uint32_t totalRead = 0;
 
-    PRUint32 bytesRead;
+    uint32_t bytesRead;
     do {
         nsresult rv = mInputStream->Read(aBuffer, aCount, &bytesRead);
         if (rv == NS_BASE_STREAM_WOULD_BLOCK && totalRead != 0) {
@@ -351,7 +353,7 @@ struct ReadSegmentsClosure {
     void* mRealClosure;
     nsWriteSegmentFun mRealWriter;
     nsresult mRealResult;
-    PRUint32 mBytesRead;  // to properly implement aToOffset
+    uint32_t mBytesRead;  // to properly implement aToOffset
 };
 
 // the thunking function
@@ -359,9 +361,9 @@ static NS_METHOD
 ReadSegmentForwardingThunk(nsIInputStream* aStream,
                            void *aClosure,
                            const char* aFromSegment,
-                           PRUint32 aToOffset,
-                           PRUint32 aCount,
-                           PRUint32 *aWriteCount)
+                           uint32_t aToOffset,
+                           uint32_t aCount,
+                           uint32_t *aWriteCount)
 {
     ReadSegmentsClosure* thunkClosure =
         reinterpret_cast<ReadSegmentsClosure*>(aClosure);
@@ -381,14 +383,14 @@ ReadSegmentForwardingThunk(nsIInputStream* aStream,
 
 
 NS_IMETHODIMP
-nsBinaryInputStream::ReadSegments(nsWriteSegmentFun writer, void * closure, PRUint32 count, PRUint32 *_retval)
+nsBinaryInputStream::ReadSegments(nsWriteSegmentFun writer, void * closure, uint32_t count, uint32_t *_retval)
 {
     NS_ENSURE_STATE(mInputStream);
 
     ReadSegmentsClosure thunkClosure = { this, closure, writer, NS_OK, 0 };
     
     // mInputStream might give us short reads, so deal with that.
-    PRUint32 bytesRead;
+    uint32_t bytesRead;
     do {
         nsresult rv = mInputStream->ReadSegments(ReadSegmentForwardingThunk,
                                                  &thunkClosure,
@@ -439,7 +441,7 @@ nsBinaryInputStream::SetInputStream(nsIInputStream *aInputStream)
 NS_IMETHODIMP
 nsBinaryInputStream::ReadBoolean(bool* aBoolean)
 {
-    PRUint8 byteResult;
+    uint8_t byteResult;
     nsresult rv = Read8(&byteResult);
     if (NS_FAILED(rv)) return rv;
     *aBoolean = !!byteResult;
@@ -447,10 +449,10 @@ nsBinaryInputStream::ReadBoolean(bool* aBoolean)
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::Read8(PRUint8* aByte)
+nsBinaryInputStream::Read8(uint8_t* aByte)
 {
     nsresult rv;
-    PRUint32 bytesRead;
+    uint32_t bytesRead;
 
     rv = Read(reinterpret_cast<char*>(aByte), sizeof(*aByte), &bytesRead);
     if (NS_FAILED(rv)) return rv;
@@ -460,70 +462,70 @@ nsBinaryInputStream::Read8(PRUint8* aByte)
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::Read16(PRUint16* a16)
+nsBinaryInputStream::Read16(uint16_t* a16)
 {
     nsresult rv;
-    PRUint32 bytesRead;
+    uint32_t bytesRead;
 
     rv = Read(reinterpret_cast<char*>(a16), sizeof *a16, &bytesRead);
     if (NS_FAILED(rv)) return rv;
     if (bytesRead != sizeof *a16)
         return NS_ERROR_FAILURE;
-    *a16 = NS_SWAP16(*a16);
+    *a16 = mozilla::NativeEndian::swapFromBigEndian(*a16);
     return rv;
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::Read32(PRUint32* a32)
+nsBinaryInputStream::Read32(uint32_t* a32)
 {
     nsresult rv;
-    PRUint32 bytesRead;
+    uint32_t bytesRead;
 
     rv = Read(reinterpret_cast<char*>(a32), sizeof *a32, &bytesRead);
     if (NS_FAILED(rv)) return rv;
     if (bytesRead != sizeof *a32)
         return NS_ERROR_FAILURE;
-    *a32 = NS_SWAP32(*a32);
+    *a32 = mozilla::NativeEndian::swapFromBigEndian(*a32);
     return rv;
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::Read64(PRUint64* a64)
+nsBinaryInputStream::Read64(uint64_t* a64)
 {
     nsresult rv;
-    PRUint32 bytesRead;
+    uint32_t bytesRead;
 
     rv = Read(reinterpret_cast<char*>(a64), sizeof *a64, &bytesRead);
     if (NS_FAILED(rv)) return rv;
     if (bytesRead != sizeof *a64)
         return NS_ERROR_FAILURE;
-    *a64 = NS_SWAP64(*a64);
+    *a64 = mozilla::NativeEndian::swapFromBigEndian(*a64);
     return rv;
 }
 
 NS_IMETHODIMP
 nsBinaryInputStream::ReadFloat(float* aFloat)
 {
-    NS_ASSERTION(sizeof(float) == sizeof (PRUint32),
+    NS_ASSERTION(sizeof(float) == sizeof (uint32_t),
                  "False assumption about sizeof(float)");
-    return Read32(reinterpret_cast<PRUint32*>(aFloat));
+    return Read32(reinterpret_cast<uint32_t*>(aFloat));
 }
 
 NS_IMETHODIMP
 nsBinaryInputStream::ReadDouble(double* aDouble)
 {
-    NS_ASSERTION(sizeof(double) == sizeof(PRUint64),
+    NS_ASSERTION(sizeof(double) == sizeof(uint64_t),
                  "False assumption about sizeof(double)");
-    return Read64(reinterpret_cast<PRUint64*>(aDouble));
+    return Read64(reinterpret_cast<uint64_t*>(aDouble));
 }
 
 static NS_METHOD
 WriteSegmentToCString(nsIInputStream* aStream,
                       void *aClosure,
                       const char* aFromSegment,
-                      PRUint32 aToOffset,
-                      PRUint32 aCount,
-                      PRUint32 *aWriteCount)
+                      uint32_t aToOffset,
+                      uint32_t aCount,
+                      uint32_t *aWriteCount)
 {
     nsACString* outString = static_cast<nsACString*>(aClosure);
 
@@ -538,7 +540,7 @@ NS_IMETHODIMP
 nsBinaryInputStream::ReadCString(nsACString& aString)
 {
     nsresult rv;
-    PRUint32 length, bytesRead;
+    uint32_t length, bytesRead;
 
     rv = Read32(&length);
     if (NS_FAILED(rv)) return rv;
@@ -582,9 +584,9 @@ static NS_METHOD
 WriteSegmentToString(nsIInputStream* aStream,
                      void *aClosure,
                      const char* aFromSegment,
-                     PRUint32 aToOffset,
-                     PRUint32 aCount,
-                     PRUint32 *aWriteCount)
+                     uint32_t aToOffset,
+                     uint32_t aCount,
+                     uint32_t *aWriteCount)
 {
     NS_PRECONDITION(aCount > 0, "Why are we being told to write 0 bytes?");
     NS_PRECONDITION(sizeof(PRUnichar) == 2, "We can't handle other sizes!");
@@ -603,9 +605,7 @@ WriteSegmentToString(nsIInputStream* aStream,
         char bytes[2] = { closure->mCarryoverByte, *aFromSegment };
         *cursor = *(PRUnichar*)bytes;
         // Now the little endianness dance
-#ifdef IS_LITTLE_ENDIAN
-        *cursor = (PRUnichar) NS_SWAP16(*cursor);
-#endif
+        mozilla::NativeEndian::swapToBigEndianInPlace(cursor, 1);
         ++cursor;
         
         // now skip past the first byte of the buffer.. code from here
@@ -622,15 +622,13 @@ WriteSegmentToString(nsIInputStream* aStream,
         reinterpret_cast<const PRUnichar*>(aFromSegment);
 
     // calculate number of full characters in segment (aCount could be odd!)
-    PRUint32 segmentLength = aCount / sizeof(PRUnichar);
+    uint32_t segmentLength = aCount / sizeof(PRUnichar);
 
     // copy all data into our aligned buffer.  byte swap if necessary.
+    // cursor may be unaligned, so we cannot use copyAndSwapToBigEndian directly
     memcpy(cursor, unicodeSegment, segmentLength * sizeof(PRUnichar));
     PRUnichar *end = cursor + segmentLength;
-#ifdef IS_LITTLE_ENDIAN
-    for (; cursor < end; ++cursor)
-        *cursor = (PRUnichar) NS_SWAP16(*cursor);
-#endif
+    mozilla::NativeEndian::swapToBigEndianInPlace(cursor, segmentLength);
     closure->mWriteCursor = end;
 
     // remember this is the modifed aCount and aFromSegment,
@@ -651,7 +649,7 @@ NS_IMETHODIMP
 nsBinaryInputStream::ReadString(nsAString& aString)
 {
     nsresult rv;
-    PRUint32 length, bytesRead;
+    uint32_t length, bytesRead;
 
     rv = Read32(&length);
     if (NS_FAILED(rv)) return rv;
@@ -662,7 +660,7 @@ nsBinaryInputStream::ReadString(nsAString& aString)
     }
 
     // pre-allocate output buffer, and get direct access to buffer...
-    if (!EnsureStringLength(aString, length))
+    if (!aString.SetLength(length, mozilla::fallible_t()))
         return NS_ERROR_OUT_OF_MEMORY;
 
     nsAString::iterator start;
@@ -685,10 +683,10 @@ nsBinaryInputStream::ReadString(nsAString& aString)
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::ReadBytes(PRUint32 aLength, char* *_rval)
+nsBinaryInputStream::ReadBytes(uint32_t aLength, char* *_rval)
 {
     nsresult rv;
-    PRUint32 bytesRead;
+    uint32_t bytesRead;
     char* s;
 
     s = reinterpret_cast<char*>(moz_malloc(aLength));
@@ -710,11 +708,37 @@ nsBinaryInputStream::ReadBytes(PRUint32 aLength, char* *_rval)
 }
 
 NS_IMETHODIMP
-nsBinaryInputStream::ReadByteArray(PRUint32 aLength, PRUint8* *_rval)
+nsBinaryInputStream::ReadByteArray(uint32_t aLength, uint8_t* *_rval)
 {
     return ReadBytes(aLength, reinterpret_cast<char **>(_rval));
 }
 
+NS_IMETHODIMP
+nsBinaryInputStream::ReadArrayBuffer(uint32_t aLength, const JS::Value& aBuffer, JSContext* cx)
+{
+    JSAutoRequest ar(cx);
+    if (!aBuffer.isObject()) {
+        return NS_ERROR_FAILURE;
+    }
+    JS::RootedObject buffer(cx, &aBuffer.toObject());
+    if (!JS_IsArrayBufferObject(buffer) ||
+        JS_GetArrayBufferByteLength(buffer) < aLength) {
+        return NS_ERROR_FAILURE;
+    }
+    uint8_t* data = JS_GetArrayBufferData(&aBuffer.toObject());
+    if (!data) {
+        return NS_ERROR_FAILURE;
+    }
+
+    uint32_t bytesRead;
+    nsresult rv = Read(reinterpret_cast<char*>(data), aLength, &bytesRead);
+    NS_ENSURE_SUCCESS(rv, rv);
+    if (bytesRead != aLength) {
+        return NS_ERROR_FAILURE;
+    }
+    return NS_OK;
+}
+
 NS_IMETHODIMP
 nsBinaryInputStream::ReadObject(bool aIsStrongRef, nsISupports* *aObject)
 {
@@ -785,15 +809,15 @@ nsBinaryInputStream::ReadID(nsID *aResult)
 }
 
 NS_IMETHODIMP_(char*)
-nsBinaryInputStream::GetBuffer(PRUint32 aLength, PRUint32 aAlignMask)
+nsBinaryInputStream::GetBuffer(uint32_t aLength, uint32_t aAlignMask)
 {
     if (mBufferAccess)
         return mBufferAccess->GetBuffer(aLength, aAlignMask);
-    return nsnull;
+    return nullptr;
 }
 
 NS_IMETHODIMP_(void)
-nsBinaryInputStream::PutBuffer(char* aBuffer, PRUint32 aLength)
+nsBinaryInputStream::PutBuffer(char* aBuffer, uint32_t aLength)
 {
     if (mBufferAccess)
         mBufferAccess->PutBuffer(aBuffer, aLength);
